@@ -158,14 +158,14 @@ static void NewAircraftWndProc(Window *w, WindowEvent *e)
 }
 
 static const Widget _new_aircraft_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   239,     0,    13, STR_A005_NEW_AIRCRAFT, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_MATRIX,    14,     0,   228,    14,   109, 0x401, STR_A025_AIRCRAFT_SELECTION_LIST},
-{  WWT_SCROLLBAR,    14,   229,   239,    14,   109, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{     WWT_IMGBTN,    14,     0,   239,   110,   161, 0x0, 0},
-{ WWT_PUSHTXTBTN,    14,     0,   119,   162,   173, STR_A006_BUILD_AIRCRAFT, STR_A026_BUILD_THE_HIGHLIGHTED_AIRCRAFT},
-{ WWT_PUSHTXTBTN,    14,   120,   239,   162,   173, STR_A037_RENAME, STR_A038_RENAME_AIRCRAFT_TYPE},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,								STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   239,     0,    13, STR_A005_NEW_AIRCRAFT,		STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_MATRIX,    14,     0,   228,    14,   109, 0x401,										STR_A025_AIRCRAFT_SELECTION_LIST},
+{  WWT_SCROLLBAR,    14,   229,   239,    14,   109, 0x0,											STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{     WWT_IMGBTN,    14,     0,   239,   110,   161, 0x0,											STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,   119,   162,   173, STR_A006_BUILD_AIRCRAFT,	STR_A026_BUILD_THE_HIGHLIGHTED_AIRCRAFT},
+{ WWT_PUSHTXTBTN,    14,   120,   239,   162,   173, STR_A037_RENAME,					STR_A038_RENAME_AIRCRAFT_TYPE},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _new_aircraft_desc = {
@@ -272,12 +272,12 @@ static void AircraftRefitWndProc(Window *w, WindowEvent *e)
 }
 
 static const Widget _aircraft_refit_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   239,     0,    13, STR_A03C_REFIT, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    14,     0,   239,    14,   135, 0x0, STR_A03E_SELECT_TYPE_OF_CARGO_FOR},
-{     WWT_IMGBTN,    14,     0,   239,   136,   157, 0x0},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,				STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   239,     0,    13, STR_A03C_REFIT,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    14,     0,   239,    14,   135, 0x0,							STR_A03E_SELECT_TYPE_OF_CARGO_FOR},
+{     WWT_IMGBTN,    14,     0,   239,   136,   157, 0x0,							STR_NULL},
 { WWT_PUSHTXTBTN,    14,     0,   239,   158,   169, STR_A03D_REFIT_AIRCRAFT, STR_A03F_REFIT_AIRCRAFT_TO_CARRY},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _aircraft_refit_desc = {
@@ -309,6 +309,9 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 	switch(e->event) {
 	case WE_PAINT:
 		w->disabled_state = v->owner == _local_player ? 0 : (1 << 2);
+		if (!_patches.servint_aircraft) // disable service-scroller when interval is set to disabled
+			w->disabled_state |= (1 << 5) | (1 << 6);
+
 		SET_DPARAM16(0, v->string_id);
 		SET_DPARAM16(1, v->unitnumber);
 		DrawWindowWidgets(w);
@@ -356,7 +359,7 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 		{
 			SET_DPARAM16(0, v->service_interval);
 			SET_DPARAM16(1, v->date_of_last_service);
-			DrawString(13, 103, STR_883C_SERVICING_INTERVAL_DAYS, 0);
+			DrawString(13, 103, _patches.servint_ispercent?STR_SERVICING_INTERVAL_PERCENT:STR_883C_SERVICING_INTERVAL_DAYS, 0);
 		}
 
 		DrawAircraftImage(v, 3, 57, INVALID_VEHICLE);
@@ -403,14 +406,19 @@ static void AircraftDetailsWndProc(Window *w, WindowEvent *e)
 			break;
 
 		case 5: /* increase int */
-			mod = 10;
+			mod = _ctrl_pressed? 5 : 10;
 			goto change_int;
 		case 6: /* decrease int */
-			mod = -10;
+			mod = _ctrl_pressed?- 5 : -10;
 change_int:
 			mod += v->service_interval;
-			if (!IS_INT_INSIDE(mod, 30, 800+1))
+
+			/*	%-based service interval max 5%-90%
+					day-based service interval max 30-800 days */
+			mod = _patches.servint_ispercent ? clamp(mod, MIN_SERVINT_PERCENT, MAX_SERVINT_PERCENT) : clamp(mod, MIN_SERVINT_DAYS, MAX_SERVINT_DAYS+1);
+			if (mod == v->service_interval)
 				return;
+
 			DoCommandP(v->tile, v->index, mod, NULL,
 				CMD_CHANGE_AIRCRAFT_SERVICE_INT | CMD_MSG(STR_018A_CAN_T_CHANGE_SERVICING));
 			break;
@@ -435,15 +443,15 @@ change_int:
 
 
 static const Widget _aircraft_details_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   349,     0,    13, STR_A00C_DETAILS, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{ WWT_PUSHTXTBTN,    14,   350,   389,     0,    13, STR_01AA_NAME, STR_A032_NAME_AIRCRAFT},
-{     WWT_IMGBTN,    14,     0,   389,    14,    55, 0x0},
-{     WWT_IMGBTN,    14,     0,   389,    56,   101, 0x0},
-{ WWT_PUSHTXTBTN,    14,     0,    10,   102,   107, STR_0188, STR_884D_INCREASE_SERVICING_INTERVAL},
-{ WWT_PUSHTXTBTN,    14,     0,    10,   108,   113, STR_0189, STR_884E_DECREASE_SERVICING_INTERVAL},
-{     WWT_IMGBTN,    14,    11,   389,   102,   113, 0x0},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,					STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   349,     0,    13, STR_A00C_DETAILS,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{ WWT_PUSHTXTBTN,    14,   350,   389,     0,    13, STR_01AA_NAME,			STR_A032_NAME_AIRCRAFT},
+{     WWT_IMGBTN,    14,     0,   389,    14,    55, 0x0,								STR_NULL},
+{     WWT_IMGBTN,    14,     0,   389,    56,   101, 0x0,								STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,    10,   102,   107, STR_0188,					STR_884D_INCREASE_SERVICING_INTERVAL},
+{ WWT_PUSHTXTBTN,    14,     0,    10,   108,   113, STR_0189,					STR_884E_DECREASE_SERVICING_INTERVAL},
+{     WWT_IMGBTN,    14,    11,   389,   102,   113, 0x0,								STR_NULL},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _aircraft_details_desc = {
@@ -473,17 +481,17 @@ static void ShowAircraftDetailsWindow(Vehicle *v)
 
 
 static const Widget _aircraft_view_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   249,     0,    13, STR_A00A, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    14,     0,   231,    14,   103, 0x0, 0},
-{          WWT_6,    14,     2,   229,    16,   101, 0x0, 0},
-{ WWT_PUSHIMGBTN,    14,     0,   249,   104,   115, 0x0, STR_A027_CURRENT_AIRCRAFT_ACTION},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    14,    31, 0x2AB, STR_A029_CENTER_MAIN_VIEW_ON_AIRCRAFT},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    32,    49, 0x2AF, STR_A02A_SEND_AIRCRAFT_TO_HANGAR},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    50,    67, 0x2B4, STR_A03B_REFIT_AIRCRAFT_TO_CARRY},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    68,    85, 0x2B2, STR_A028_SHOW_AIRCRAFT_S_ORDERS},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    86,   103, 0x2B3, STR_A02B_SHOW_AIRCRAFT_DETAILS},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,	STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   249,     0,    13, STR_A00A,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    14,     0,   231,    14,   103, 0x0,				STR_NULL},
+{          WWT_6,    14,     2,   229,    16,   101, 0x0,				STR_NULL},
+{ WWT_PUSHIMGBTN,    14,     0,   249,   104,   115, 0x0,				STR_A027_CURRENT_AIRCRAFT_ACTION},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    14,    31, 0x2AB,			STR_A029_CENTER_MAIN_VIEW_ON_AIRCRAFT},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    32,    49, 0x2AF,			STR_A02A_SEND_AIRCRAFT_TO_HANGAR},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    50,    67, 0x2B4,			STR_A03B_REFIT_AIRCRAFT_TO_CARRY},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    68,    85, 0x2B2,			STR_A028_SHOW_AIRCRAFT_S_ORDERS},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    86,   103, 0x2B3,			STR_A02B_SHOW_AIRCRAFT_DETAILS},
+{   WIDGETS_END},
 };
 
 static void AircraftViewWndProc(Window *w, WindowEvent *e)
@@ -694,12 +702,16 @@ static int GetVehicleFromAircraftDepotWndPt(Window *w, int x, int y, Vehicle **v
 static void AircraftDepotClickAircraft(Window *w, int x, int y)
 {
 	Vehicle *v;
+	int mode = GetVehicleFromAircraftDepotWndPt(w, x, y, &v);
 
-	switch(GetVehicleFromAircraftDepotWndPt(w, x, y, &v)) {
+	// share / copy orders
+	if (_thd.place_mode && mode <= 0) { _place_clicked_vehicle = v; return; }
+
+	switch(mode) {
 	case 1:
 		return;
 
-	case 0:
+	case 0: // start dragging of vehicle
 		if (v != NULL) {
 			WP(w,traindepot_d).sel = v->index;
 			SetWindowDirty(w);
@@ -707,11 +719,11 @@ static void AircraftDepotClickAircraft(Window *w, int x, int y)
 		}
 		break;
 
-	case -1:
+	case -1: // show info window
 		ShowAircraftViewWindow(v);
 		break;
 
-	case -2:
+	case -2: // click start/stop flag
 		DoCommandP(v->tile, v->index, 0, NULL, CMD_START_STOP_AIRCRAFT | CMD_MSG(STR_A016_CAN_T_STOP_START_AIRCRAFT));
 		break;
 
@@ -728,7 +740,7 @@ static const Widget _aircraft_depot_widgets[] = {
 {  WWT_SCROLLBAR,    14,   320,   330,    14,    61, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
 { WWT_PUSHTXTBTN,    14,     0,   164,    62,    73, STR_A003_NEW_AIRCRAFT, STR_A022_BUILD_NEW_AIRCRAFT},
 { WWT_PUSHTXTBTN,    14,   165,   330,    62,    73, STR_00E4_LOCATION, STR_A024_CENTER_MAIN_VIEW_ON_HANGAR},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 
@@ -852,102 +864,186 @@ static void DrawSmallSchedule(Vehicle *v, int x, int y) {
 	}
 }
 
+// used to get a sorted list of the vehicles
+static SortStruct _aircraft_sort[NUM_NORMAL_VEHICLES];
+static uint16 _num_aircraft_sort[MAX_PLAYERS];
+
+static void GlobalSortAircraftList()
+{
+	const Vehicle *v;
+	uint16 *i;
+	uint32 n = 0;
+
+	// reset #-of aircraft to 0 because ++ is used for value-assignment
+	for (i = _num_aircraft_sort; i != endof(_num_aircraft_sort); i++) {*i = 0;}
+
+	FOR_ALL_VEHICLES(v) {
+		if(v->type == VEH_Aircraft && v->subtype <= 2) {
+			_aircraft_sort[n].index = v->index;
+			_aircraft_sort[n++].owner = v->owner;
+			_num_aircraft_sort[v->owner]++; // add number of aircraft of player
+		}
+	}
+
+	// create cumulative aircraft-ownership
+	// aircraft are stored as a cummulative index, eg 25, 41, 43. This means
+	// Player0: 25; Player1: (41-25) 16; Player2: (43-41) 2
+	for (i = &_num_aircraft_sort[1]; i != endof(_num_aircraft_sort); i++) {*i += *(i-1);}
+
+	qsort(_aircraft_sort, n, sizeof(_aircraft_sort[0]), GeneralOwnerSorter); // sort by owner
+
+	// since indexes are messed up after adding/removing a station, mark all lists dirty
+	memset(_aircraft_sort_dirty, true, sizeof(_aircraft_sort_dirty));
+	_vehicle_sort_dirty[VEHAIRCRAFT] = false;
+
+	DEBUG(misc, 1) ("Resorting global aircraft list...");	
+}
+
+static void MakeSortedAircraftList(byte owner)
+{
+	SortStruct *firstelement;
+	uint32 n = 0;
+
+	if (owner == 0) { // first element starts at 0th element and has n elements as described above
+		firstelement =	&_aircraft_sort[0];
+		n =							_num_aircraft_sort[0];
+	}	else { // nth element starts at the end of the previous one, and has n elements as described above
+		firstelement =	&_aircraft_sort[_num_aircraft_sort[owner-1]];
+		n =							_num_aircraft_sort[owner] - _num_aircraft_sort[owner-1];
+	}
+
+	_internal_sort_order			= _aircraft_sort_order[owner];
+	_internal_name_sorter_id	= STR_SV_AIRCRAFT_NAME;
+	_last_vehicle_idx = 0; // used for "cache" in namesorting
+	qsort(firstelement, n, sizeof(_aircraft_sort[0]), _vehicle_sorter[_aircraft_sort_type[owner]]);
+
+	_aircraft_sort_dirty[owner] = false;
+
+	DEBUG(misc, 1) ("Resorting Aircraft list player %d...", owner+1);
+}
+
 static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 {
 	switch(e->event) {
-	case WE_PAINT:
-		/* determine amount of items for scroller */
-		{
-			Vehicle *v;
-			int num = 0;
-			byte owner = (byte)w->window_number;
+	case WE_PAINT: {
+		uint32 i;
+		const byte window_number = (byte)w->window_number;
 
-			FOR_ALL_VEHICLES(v) {
-				if (v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == owner)
-					num++;
-			}
+		if (_aircraft_sort_type[window_number] == SORT_BY_UNSORTED) // disable 'Sort By' tooltip on Unsorted sorting criteria
+			w->disabled_state |= (1 << 2);
 
-			SetVScrollCount(w, num);
+		// resort shipps window if roadvehicles have been added/removed
+		if (_vehicle_sort_dirty[VEHAIRCRAFT])
+			GlobalSortAircraftList();
+
+		if (_aircraft_sort_dirty[window_number]) {
+			MakeSortedAircraftList(window_number);
+			/* reset sorting timeout */
+			w->custom[0] = DAY_TICKS;
+			w->custom[1] = PERIODIC_RESORT_DAYS;
 		}
 
+		// aircraft are stored as a cummulative index, eg 25, 41, 43. This means
+		// Player0: 25; Player1: (41-25) 16; Player2: (43-41) 2 aircraft
+		i = (window_number == 0) ? 0 : _num_aircraft_sort[window_number-1];
+		SetVScrollCount(w, _num_aircraft_sort[window_number] - i);
+
 		/* draw the widgets */
 		{
-			Player *p = DEREF_PLAYER(w->window_number);
+			Player *p = DEREF_PLAYER(window_number);
+			/* Company Name -- (###) Aircraft */
 			SET_DPARAM16(0, p->name_1);
 			SET_DPARAM32(1, p->name_2);
+			SET_DPARAM16(2, w->vscroll.count);
 			DrawWindowWidgets(w);
 		}
+		/* draw sorting criteria string */
+		DrawString(85, 15, _vehicle_sort_listing[_aircraft_sort_type[window_number]], 0x10);
+		/* draw arrow pointing up/down for ascending/descending soring */
+		DoDrawString(_aircraft_sort_order[window_number] & 1 ? "\xAA" : "\xA0", 69, 15, 0x10);
 
 		/* draw the aircraft */
 		{
 			Vehicle *v;
-			int pos = w->vscroll.pos;
-			byte owner = (byte)w->window_number;
-			int x = 2;
-			int y = 15;
+			int n = 0;
+			const int x = 2;			// offset from left side of widget
+			int y = PLY_WND_PRC__OFFSET_TOP_WIDGET;	// offset from top of widget
+			i += w->vscroll.pos;	// offset from sorted aircraft list of current player
 
-			FOR_ALL_VEHICLES(v) {
-				if (v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == owner &&
-						--pos < 0 && pos >= -4) {
-					StringID str;
+			while (i < _num_aircraft_sort[window_number]) {
+				StringID str;
+				v = DEREF_VEHICLE(_aircraft_sort[i].index);
 
-					DrawAircraftImage(v, x + 19, y + 6, INVALID_VEHICLE);
-					DrawVehicleProfitButton(v, x, y+13);
+				assert(v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == window_number);
 
-					SET_DPARAM16(0, v->unitnumber);
-					if (IsAircraftHangarTile(v->tile)) {
-						str = STR_021F;
-					} else {
-						str = v->age > v->max_age - 366 ? STR_00E3 : STR_00E2;
-					}
-					DrawString(x, y+2, str, 0);
+				DrawAircraftImage(v, x + 19, y + 6, INVALID_VEHICLE);
+				DrawVehicleProfitButton(v, x, y+13);
 
-					SET_DPARAM32(0, v->profit_this_year);
-					SET_DPARAM32(1, v->profit_last_year);
-					DrawString(x+19, y + 28, STR_0198_PROFIT_THIS_YEAR_LAST_YEAR, 0);
-
-					if (v->string_id != STR_SV_AIRCRAFT_NAME) {
-						SET_DPARAM16(0, v->string_id);
-						DrawString(x+19, y, STR_01AB, 0);
-					}
-
-					DrawSmallSchedule(v, x+136, y);
-
-					y += 36;
+				SET_DPARAM16(0, v->unitnumber);
+				if (IsAircraftHangarTile(v->tile)) {
+					str = STR_021F;
+				} else {
+					str = v->age > v->max_age - 366 ? STR_00E3 : STR_00E2;
 				}
+				DrawString(x, y+2, str, 0);
+
+
+				SET_DPARAM32(0, v->profit_this_year);
+				SET_DPARAM32(1, v->profit_last_year);
+				DrawString(x+19, y + 28, STR_0198_PROFIT_THIS_YEAR_LAST_YEAR, 0);
+
+				if (v->string_id != STR_SV_AIRCRAFT_NAME) {
+					SET_DPARAM16(0, v->string_id);
+					DrawString(x+19, y, STR_01AB, 0);
+				}
+
+				DrawSmallSchedule(v, x+136, y);
+
+				y += PLY_WND_PRC__SIZE_OF_ROW_BIG;
+				i++; // next aircraft
+				if (++n == w->vscroll.cap) { break;} // max number of aircraft in the window
 			}
 		}
-		break;
+		}	break;
 
-	case WE_CLICK:
+	case WE_CLICK: {
 		switch(e->click.widget) {
-		case 2: { /* click on aircraft */
-			int sel;
-			Vehicle *v;
-			byte owner;
-
-			sel = (e->click.pt.y - 14) / 36;
-
-			if ((uint)sel >= 4)
-				break;
-			sel += w->vscroll.pos;
-			owner = (byte)w->window_number;
-			FOR_ALL_VEHICLES(v) {
-				if (v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == owner &&
-						--sel < 0) {
-					ShowAircraftViewWindow(v);
-					break;
-				}
-			}
+		case 2: /* Flip sorting method ascending/descending */
+			_aircraft_sort_order[(byte)w->window_number] ^= 1;
+			_aircraft_sort_dirty[(byte)w->window_number] = true;
+			SetWindowDirty(w);
 			break;
-		}
-		case 4: { /* build new */
+		case 3: case 4:/* Select sorting criteria dropdown menu */
+			ShowDropDownMenu(w, _vehicle_sort_listing, _aircraft_sort_type[(byte)w->window_number], 4, 0); // do it for widget 4
+			return;
+		case 6: { /* Matrix to show vehicles */
+			uint32 id_v = (e->click.pt.y - PLY_WND_PRC__OFFSET_TOP_WIDGET) / PLY_WND_PRC__SIZE_OF_ROW_BIG;
+
+			if (id_v >= w->vscroll.cap) { return;} // click out of bounds
+
+			id_v += w->vscroll.pos;
+
+			{
+				const byte owner = (byte)w->window_number;
+				Vehicle *v;
+				id_v += (owner == 0) ? 0 : _num_aircraft_sort[owner - 1]; // first element in list
+
+				if (id_v >= _num_aircraft_sort[owner]) { return;} // click out of vehicle bound
+
+				v	= DEREF_VEHICLE(_aircraft_sort[id_v].index); // add the offset id_x to that
+
+				assert(v->type == VEH_Aircraft && v->subtype <= 2 && v->owner == owner);
+
+				ShowAircraftViewWindow(v);
+			}
+		} break;
+
+		case 8: { /* Build new Vehicle */
 			uint tile;
 
 			tile = _last_built_aircraft_depot_tile;
 			do {
-				if (_map_owner[tile] == _local_player &&
-						IsAircraftHangarTile(tile)) {
+				if (_map_owner[tile] == _local_player && IsAircraftHangarTile(tile)) {
 					ShowAircraftDepotWindow(tile);
 					ShowBuildAircraftWindow(tile);
 					return;
@@ -955,25 +1051,57 @@ static void PlayerAircraftWndProc(Window *w, WindowEvent *e)
 
 				tile = TILE_MASK(tile + 1);
 			} while(tile != _last_built_aircraft_depot_tile);
+
 			ShowBuildAircraftWindow(0);
 		} break;
 		}
+	}	break;
+
+	case WE_DROPDOWN_SELECT: /* we have selected a dropdown item in the list */
+		if (_aircraft_sort_type[(byte)w->window_number] != e->dropdown.index) // if value hasn't changed, dont resort list
+			_aircraft_sort_dirty[(byte)w->window_number] = true;
+
+		_aircraft_sort_type[(byte)w->window_number] = e->dropdown.index;
+
+		if (_aircraft_sort_type[(byte)w->window_number] != SORT_BY_UNSORTED) // enable 'Sort By' if a sorter criteria is chosen
+			w->disabled_state &= ~(1 << 2);
+
+		SetWindowDirty(w);
+		break;
+	case WE_CREATE: /* set up resort timer */
+		w->custom[0] = DAY_TICKS;
+		w->custom[1] = PERIODIC_RESORT_DAYS;
+		break;
+	case WE_TICK: /* resort the list every 20 seconds orso (10 days) */
+		if (--w->custom[0] == 0) {
+			w->custom[0] = DAY_TICKS;
+			if (--w->custom[1] == 0) {
+				w->custom[1] = PERIODIC_RESORT_DAYS;
+				_aircraft_sort_dirty[(byte)w->window_number] = true;
+				DEBUG(misc, 1) ("Periodic resort Aircraft list player %d...", w->window_number+1);
+				SetWindowDirty(w);
+			}
+		}
 		break;
 	}
 }
 
 static const Widget _player_aircraft_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_A009_AIRCRAFT, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_MATRIX,    14,     0,   248,    14,   157, 0x401, STR_A01F_AIRCRAFT_CLICK_ON_AIRCRAFT},
-{  WWT_SCROLLBAR,    14,   249,   259,    14,   157, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHTXTBTN,    14,     0,   129,   158,   169, STR_A003_NEW_AIRCRAFT, STR_A020_BUILD_NEW_AIRCRAFT_REQUIRES},
-{     WWT_IMGBTN,    14,   130,   259,   158,   169, 0x0, 0},
-{      WWT_LAST},
+{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,							STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_A009_AIRCRAFT,			STR_018C_WINDOW_TITLE_DRAG_THIS},
+{ WWT_PUSHTXTBTN,    14,     0,    80,    14,    25, SRT_SORT_BY,						STR_SORT_TIP},
+{      WWT_PANEL,    14,    81,   237,    14,    25, 0x0,										STR_SORT_TIP},
+{   WWT_CLOSEBOX,    14,   238,   248,    14,    25, STR_0225,							STR_SORT_TIP},
+{      WWT_PANEL,    14,   249,   259,    14,    25, 0x0,										STR_NULL},
+{     WWT_MATRIX,    14,     0,   248,    26,   169, 0x401,									STR_A01F_AIRCRAFT_CLICK_ON_AIRCRAFT},
+{  WWT_SCROLLBAR,    14,   249,   259,    26,   169, 0x0,										STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHTXTBTN,    14,     0,   129,   170,   181, STR_A003_NEW_AIRCRAFT,	STR_A020_BUILD_NEW_AIRCRAFT_REQUIRES},
+{      WWT_PANEL,    14,   130,   259,   170,   181, 0x0,										STR_NULL},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _player_aircraft_desc = {
-	-1, -1, 260, 170,
+	-1, -1, 260, 182,
 	WC_AIRCRAFT_LIST,0,
 	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
 	_player_aircraft_widgets,
@@ -981,17 +1109,21 @@ static const WindowDesc _player_aircraft_desc = {
 };
 
 static const Widget _other_player_aircraft_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_A009_AIRCRAFT, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_MATRIX,    14,     0,   248,    14,   157, 0x401, STR_A01F_AIRCRAFT_CLICK_ON_AIRCRAFT},
-{  WWT_SCROLLBAR,    14,   249,   259,    14,   157, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{      WWT_LAST},
+{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,							STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_A009_AIRCRAFT,			STR_018C_WINDOW_TITLE_DRAG_THIS},
+{ WWT_PUSHTXTBTN,    14,     0,    80,    14,    25, SRT_SORT_BY,						STR_SORT_TIP},
+{      WWT_PANEL,    14,    81,   237,    14,    25, 0x0,										STR_SORT_TIP},
+{   WWT_CLOSEBOX,    14,   238,   248,    14,    25, STR_0225,							STR_SORT_TIP},
+{      WWT_PANEL,    14,   249,   259,    14,    25, 0x0,										STR_NULL},
+{     WWT_MATRIX,    14,     0,   248,    26,   169, 0x401,									STR_A01F_AIRCRAFT_CLICK_ON_AIRCRAFT},
+{  WWT_SCROLLBAR,    14,   249,   259,    26,   169, 0x0,										STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _other_player_aircraft_desc = {
-	-1, -1, 260, 158,
+	-1, -1, 260, 170,
 	WC_AIRCRAFT_LIST,0,
-	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
 	_other_player_aircraft_widgets,
 	PlayerAircraftWndProc
 };
@@ -1000,7 +1132,7 @@ void ShowPlayerAircraft(int player)
 {
 	Window *w;
 
-	if ( player == _local_player) {
+	if (player == _local_player) {
 		w = AllocateWindowDescFront(&_player_aircraft_desc, player);
 	} else  {
 		w = AllocateWindowDescFront(&_other_player_aircraft_desc, player);
@@ -1008,6 +1140,6 @@ void ShowPlayerAircraft(int player)
 
 	if (w) {
 		w->caption_color = w->window_number;
-		w->vscroll.cap = 4;
+		w->vscroll.cap = 4; // maximum number of vehicles shown
 	}
 }
