@@ -10,9 +10,10 @@
 #include "saveload.h"
 #include "economy.h"
 #include "industry.h"
-#include "city.h"
+#include "town.h"
+#include "network.h"
 
-static void UpdatePlayerHouse(Player *p, uint score)
+void UpdatePlayerHouse(Player *p, uint score)
 {
 	byte val;
 	uint tile = p->location_of_house;
@@ -41,14 +42,14 @@ static void UpdatePlayerHouse(Player *p, uint score)
 	MarkTileDirtyByTile(tile + TILE_XY(1,1));
 }
 
-uint32 CalculateCompanyValue(Player *p) {
+int64 CalculateCompanyValue(Player *p) {
 	byte owner = p->index;
-	uint32 value;
-	
+	int64 value;
+
 	{
 		Station *st;
 		uint num = 0;
-		
+
 		FOR_ALL_STATIONS(st) {
 			if (st->xy != 0 && st->owner == owner) {
 				uint facil = st->facilities;
@@ -75,15 +76,19 @@ uint32 CalculateCompanyValue(Player *p) {
 	}
 
 	if (p->player_money > 0)
-		value += p->player_money;
+		value += p->money64; // add real money value
 
 	return value;
 }
 
-static void UpdateCompanyRatingAndValue(Player *p)
+// if update is set to true, the economy is updated with this score
+//  (also the house is updated, should only be true in the on-tick event)
+int UpdateCompanyRatingAndValue(Player *p, bool update)
 {
 	byte owner = p->index;
-	uint score;
+	int score = 0;
+
+	memset(_score_part[owner], 0, sizeof(_score_part[owner]));
 
 /* Count vehicles */
 	{
@@ -100,16 +105,16 @@ static void UpdateCompanyRatingAndValue(Player *p)
 					v->type == VEH_Ship) {
 				num++;
 				if (v->age > 730) {
-					if (min_profit >= v->profit_last_year)
+					if (min_profit > v->profit_last_year)
 						min_profit = v->profit_last_year;
 				}
 			}
 		}
 
-		score = (num < 120) ? num * 100 / 120 : 100;
+		_score_part[owner][SCORE_VEHICLES] = num;
 
 		if (min_profit != MAX_INT && min_profit > 0)
-			score += (min_profit < 10000) ? min_profit/100 : 100;
+			_score_part[owner][SCORE_MIN_PROFIT] = min_profit;
 	}
 
 /* Count stations */
@@ -123,7 +128,7 @@ static void UpdateCompanyRatingAndValue(Player *p)
 				do num += facil&1; while (facil>>=1);
 			}
 		}
-		score += (num < 80) ? num * 100 / 80 : 100;
+		_score_part[owner][SCORE_STATIONS] = num;
 	}
 
 /* Generate statistics depending on recent income statistics */
@@ -144,9 +149,9 @@ static void UpdateCompanyRatingAndValue(Player *p)
 			} while (++pee,--numec);
 
 			if (min_income > 0)
-				score += min_income >= 50000 ? 50 : min_income / 1000;
+				_score_part[owner][SCORE_MIN_INCOME] = min_income;
 
-			score += max_income >= 100000 ? 100 : max_income / 1000;
+			_score_part[owner][SCORE_MAX_INCOME] = max_income;
 		}
 	}
 
@@ -164,35 +169,64 @@ static void UpdateCompanyRatingAndValue(Player *p)
 				total_delivered += pee->delivered_cargo;
 			} while (++pee,--numec);
 
-			score += total_delivered >= 40000 ? 400 : total_delivered / 100;
+			_score_part[owner][SCORE_DELIVERED] = total_delivered;
 		}
 	}
-	
+
 /* Generate score for variety of cargo */
 	{
 		uint cargo = p->cargo_types;
 		uint num = 0;
 		do num += cargo&1; while (cargo>>=1);
-		score += num < 8 ? num * 50 / 8 : 50;
-		p->cargo_types = 0;
+		_score_part[owner][SCORE_CARGO] = num;
+		if (update)
+			p->cargo_types = 0;
 	}
 
 /* Generate score for player money */
 	{
 		int32 money = p->player_money;
 		if (money > 0) {
-			score += money < 10000000 ? money / (10000000/50) : 50;
+			_score_part[owner][SCORE_MONEY] = money;
 		}
 	}
 
 /* Generate score for loan */
 	{
-		score += (250000 - p->current_loan) >= 0 ? (250000 - p->current_loan) / 5000 : 0;
+		_score_part[owner][SCORE_LOAN] = score_info[SCORE_LOAN].needed - p->current_loan;
 	}
 
-	p->old_economy[0].performance_history = score;
-	UpdatePlayerHouse(p, score);
-	p->old_economy[0].company_value = CalculateCompanyValue(p);
+	// Now we calculate the score for each item..
+	{
+		int i;
+		int total_score = 0;
+		int s;
+		score = 0;
+		for (i=0;i<NUM_SCORE;i++) {
+			// Skip the total
+			if (i == SCORE_TOTAL) continue;
+			// Check the score
+			s = (_score_part[owner][i] >= score_info[i].needed) ? score_info[i].score : ((_score_part[owner][i] * score_info[i].score) / score_info[i].needed);
+			if (s < 0) s = 0;
+			score += s;
+			total_score += score_info[i].score;
+		}
+
+		_score_part[owner][SCORE_TOTAL] = score;
+
+		// We always want the score scaled to SCORE_MAX (1000)
+		if (total_score != SCORE_MAX)
+			score = score * SCORE_MAX / total_score;
+	}
+
+	if (update) {
+    	p->old_economy[0].performance_history = score;
+    	UpdatePlayerHouse(p, score);
+    	p->old_economy[0].company_value = CalculateCompanyValue(p);
+    }
+
+	InvalidateWindow(WC_PERFORMANCE_DETAIL, 0);
+	return score;
 }
 
 // use 255 as new_player to delete the player.
@@ -203,7 +237,7 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 
 	if (new_player == 255) {
 		Subsidy *s;
-		
+
 		for(s=_subsidies; s != endof(_subsidies); s++) {
 			if (s->cargo_type != 0xff && s->age >= 12) {
 				Station *st = DEREF_STATION(s->to);
@@ -215,28 +249,28 @@ void ChangeOwnershipOfPlayerItems(byte old_player, byte new_player)
 
 	// take care of rating in towns
 	{
-		City *c;
-		
+		Town *t;
+
 		// if a player takes over, give the ratings to that player.
 		if (new_player != 255) {
-			FOR_ALL_CITIES(c) if (c->xy && HASBIT(c->have_ratings, old_player)) {
-				if (HASBIT(c->have_ratings, new_player)) {
+			FOR_ALL_TOWNS(t) if (t->xy && HASBIT(t->have_ratings, old_player)) {
+				if (HASBIT(t->have_ratings, new_player)) {
 					// use max of the two ratings.
-					c->ratings[new_player] = max(c->ratings[new_player], c->ratings[old_player]);
+					t->ratings[new_player] = max(t->ratings[new_player], t->ratings[old_player]);
 				} else {
-					SETBIT(c->have_ratings, new_player);
-					c->ratings[new_player] = c->ratings[old_player];
+					SETBIT(t->have_ratings, new_player);
+					t->ratings[new_player] = t->ratings[old_player];
 				}
 			}
 		}
 
 		// reset rating for the towns.
-		FOR_ALL_CITIES(c) if (c->xy) {
-			c->ratings[old_player] = 500;
-			CLRBIT(c->have_ratings, old_player);
+		FOR_ALL_TOWNS(t) if (t->xy) {
+			t->ratings[old_player] = 500;
+			CLRBIT(t->have_ratings, old_player);
 		}
 	}
-	
+
 	{
 		int num_train = 0;
 		int num_road = 0;
@@ -307,7 +341,7 @@ extern void DeletePlayerWindows(int pi);
 static void PlayersCheckBankrupt(Player *p)
 {
 	int owner;
-	uint32 val;
+	int64 val;
 
 	if (p->player_money >= 0) {
 		p->quarters_of_bankrupcy = 0;
@@ -320,7 +354,7 @@ static void PlayersCheckBankrupt(Player *p)
 
 	if (p->quarters_of_bankrupcy == 2) {
 year_2:
-		AddNewsItem( (StringID)(owner + 16), 
+		AddNewsItem( (StringID)(owner + 16),
 			NEWS_FLAGS(NM_CALLBACK, 0, NT_COMPANY_INFO, DNC_BANKRUPCY),0,0);
 
 	} else if (p->quarters_of_bankrupcy == 3) {
@@ -353,7 +387,7 @@ year_4:
 			p->bankrupt_timeout = 0x456;
 		} else {
 			p->money64 = p->player_money = 100000000;
-			ChangeOwnershipOfPlayerItems(owner, 0xFF);
+			ChangeOwnershipOfPlayerItems(owner, 0xFF); // 255 is no owner
 			p->is_active = false;
 		}
 	}
@@ -363,7 +397,7 @@ void DrawNewsBankrupcy(Window *w)
 {
 	Player *p;
 
-	DrawNewsBorder(w);		
+	DrawNewsBorder(w);
 
 	p = DEREF_PLAYER(WP(w,news_d).ni->string_id & 15);
 	DrawPlayerFace(p->face, p->player_color, 2, 23);
@@ -413,7 +447,7 @@ void DrawNewsBankrupcy(Window *w)
 			90,
 			STR_705D_HAS_BEEN_CLOSED_DOWN_BY,
 			w->width - 101);
-		break;		
+		break;
 
 	case 4:
 		DrawStringCentered(w->width>>1, 1, STR_705E_NEW_TRANSPORT_COMPANY_LAUNCHED, 0);
@@ -466,7 +500,7 @@ StringID GetNewsStringBankrupcy(NewsItem *ni)
 	default:
 		NOT_REACHED();
 	}
-  
+
 	/* useless, but avoids compiler warning this way */
 	return 0;
 }
@@ -495,7 +529,7 @@ static void PlayersGenStatistics()
 			if (p->num_valid_stat_ent != 24)
 				p->num_valid_stat_ent++;
 
-			UpdateCompanyRatingAndValue(p);
+			UpdateCompanyRatingAndValue(p, true);
 			PlayersCheckBankrupt(p);
 
 			if (p->block_preview != 0)
@@ -530,13 +564,13 @@ static void AddInflation()
 	}
 
 	_economy.max_loan_unround += BIGMULUS(_economy.max_loan_unround, inf, 16);
-	
+
 	if (_economy.max_loan + 50000 <= _economy.max_loan_unround)
 		_economy.max_loan += 50000;
 
 	inf = _economy.infl_amount_pr * 54;
 	for(i=0; i!=NUM_CARGO; i++) {
-		AddSingleInflation( 
+		AddSingleInflation(
 			_cargo_payment_rates + i,
 			_cargo_payment_rates_frac + i,
 			inf
@@ -559,7 +593,7 @@ static void PlayersPayInterest()
 
 		_current_player = p->index;
 		SET_EXPENSES_TYPE(EXPENSES_LOAN_INT);
-		
+
 		SubtractMoneyFromPlayer(BIGMULUS(p->current_loan, interest, 16));
 
 		SET_EXPENSES_TYPE(EXPENSES_OTHER);
@@ -592,13 +626,55 @@ static byte _price_category[NUM_PRICES] = {
 };
 
 static const int32 _price_base[NUM_PRICES] = {
-	100, 100, 95, 65, 275, 600, 500, 700,
-	450, 200, 180, 600, 200, 200, 350, 400000,
-	2000, 700000, 14000, 65000, 20, 250, 20, 40,
-	200, 500, 20, -70, 10, 50, 80, 80,
-	90, 30, 10000, 50, 30, 50, 50, 55,
-	1600, 40, 5600, 5200, 4800, 9600, 1600, 5600,
-	1000000,
+	100,		// station_value
+	100,		// build_rail
+	95,			// build_road
+	65,			// build_signals
+	275,		// build_bridge
+	600,		// build_train_depot
+	500,		// build_road_depot
+	700,		// build_ship_depot
+	450,		// build_tunnel
+	200,		// train_station_track
+	180,		// train_station_length
+	600,		// build_airport
+	200,		// build_bus_station
+	200,		// build_truck_station
+	350,		// build_dock
+	400000,	// build_railvehicle
+	2000,		// build_railwagon
+	700000,	// aircraft_base
+	14000,	// roadveh_base
+	65000,	// ship_base
+	20,			// build_trees
+	250,		// terraform
+	20,			// clear_1
+	40,			// purchase_land
+	200,		// clear_2
+	500,		// clear_3
+	20,			// remove_trees
+	-70,		// remove_rail
+	10,			// remove_signals
+	50,			// clear_bridge
+	80,			// remove_train_depot
+	80,			// remove_road_depot
+	90,			// remove_ship_depot
+	30,			// clear_tunnel
+	10000,	// clear_water
+	50,			// remove_rail_station
+	30,			// remove_airport
+	50,			// remove_bus_station
+	50,			// remove_truck_station
+	55,			// remove_dock
+	1600,		// remove_house
+	40,			// remove_road
+	5600,		// running_rail[0] railroad
+	5200,		// running_rail[1] monorail
+	4800,		// running_rail[2] maglev
+	9600,		// aircraft_running
+	1600,		// roadveh_running
+	5600,		// ship_running
+	1000000, // build_industry
 };
 
 void StartupEconomy()
@@ -632,7 +708,7 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 {
 	int tile, tile2;
 	Industry *i;
-	City *c;
+	Town *t;
 	Station *st;
 	Pair tp;
 
@@ -644,8 +720,8 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 			SET_DPARAM16(1, STR_2029);
 			i = DEREF_INDUSTRY(s->from);
 			tile = i->xy;
-			SET_DPARAM16(2, i->city->citynametype);
-			SET_DPARAM32(3, i->city->citynameparts);
+			SET_DPARAM16(2, i->town->townnametype);
+			SET_DPARAM32(3, i->town->townnameparts);
 
 			SET_DPARAM16(4, i->type + STR_4802_COAL_MINE);
 
@@ -654,24 +730,24 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 				i = DEREF_INDUSTRY(s->to);
 				tile2 = i->xy;
 				SET_DPARAM16(8, i->type + STR_4802_COAL_MINE);
-				SET_DPARAM16(6, i->city->citynametype);
-				SET_DPARAM32(7, i->city->citynameparts);
+				SET_DPARAM16(6, i->town->townnametype);
+				SET_DPARAM32(7, i->town->townnameparts);
 			} else {
-				c = DEREF_CITY(s->to);
-				tile2 = c->xy;
-				SET_DPARAM16(5, c->citynametype);
-				SET_DPARAM32(6, c->citynameparts);
+				t = DEREF_TOWN(s->to);
+				tile2 = t->xy;
+				SET_DPARAM16(5, t->townnametype);
+				SET_DPARAM32(6, t->townnameparts);
 			}
 		} else {
-			c = DEREF_CITY(s->from);
-			tile = c->xy;
-			SET_DPARAM16(1, c->citynametype);
-			SET_DPARAM32(2, c->citynameparts);
+			t = DEREF_TOWN(s->from);
+			tile = t->xy;
+			SET_DPARAM16(1, t->townnametype);
+			SET_DPARAM32(2, t->townnameparts);
 
-			c = DEREF_CITY(s->to);
-			tile2 = c->xy;
-			SET_DPARAM16(3, c->citynametype);
-			SET_DPARAM32(4, c->citynameparts);
+			t = DEREF_TOWN(s->to);
+			tile2 = t->xy;
+			SET_DPARAM16(3, t->townnametype);
+			SET_DPARAM32(4, t->townnameparts);
 		}
 	} else {
 		st = DEREF_STATION(s->from);
@@ -692,28 +768,28 @@ Pair SetupSubsidyDecodeParam(Subsidy *s, bool mode)
 void DeleteSubsidyWithIndustry(byte index)
 {
 	Subsidy *s;
-	
+
 	for(s=_subsidies; s != endof(_subsidies); s++) {
 		if (s->cargo_type != 0xFF && s->age < 12 &&
 				s->cargo_type != CT_PASSENGERS && s->cargo_type != CT_MAIL &&
 				(index == s->from || (s->cargo_type!=CT_GOODS && s->cargo_type!=CT_FOOD && index==s->to))) {
 			s->cargo_type = 0xFF;
 		}
-	}	
+	}
 }
 
-void DeleteSubsidyWithStation(byte index) 
+void DeleteSubsidyWithStation(byte index)
 {
 	Subsidy *s;
 	bool dirty = false;
-	
+
 	for(s=_subsidies; s != endof(_subsidies); s++) {
 		if (s->cargo_type != 0xFF && s->age >= 12 &&
 				(s->from == index || s->to == index)) {
 			s->cargo_type = 0xFF;
 			dirty = true;
 		}
-	}	
+	}
 
 	if (dirty)
 		InvalidateWindow(WC_SUBSIDIES_LIST, 0);
@@ -728,15 +804,15 @@ typedef struct FoundRoute {
 
 static void FindSubsidyPassengerRoute(FoundRoute *fr)
 {
-	City *from,*to;
+	Town *from,*to;
 
 	fr->distance = (uint)-1;
 
-	fr->from = from = DEREF_CITY(RandomRange(_total_cities));
+	fr->from = from = DEREF_TOWN(InteractiveRandomRange(_total_towns));
 	if (from->xy == 0 || from->population < 400)
 		return;
 
-	fr->to = to = DEREF_CITY(RandomRange(_total_cities));
+	fr->to = to = DEREF_TOWN(InteractiveRandomRange(_total_towns));
 	if (from==to || to->xy == 0 || to->population < 400 || to->pct_pass_transported > 42)
 		return;
 
@@ -751,12 +827,12 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 
 	fr->distance = (uint)-1;
 
-	fr->from = i = DEREF_INDUSTRY(RandomRange(_total_industries));
+	fr->from = i = DEREF_INDUSTRY(InteractiveRandomRange(_total_industries));
 	if (i->xy == 0)
 		return;
 
 	// Randomize cargo type
-	if (Random()&1 && i->produced_cargo[1] != 0xFF) {
+	if (InteractiveRandom()&1 && i->produced_cargo[1] != 0xFF) {
 		cargo = i->produced_cargo[1];
 		trans = i->pct_transported[1];
 		total = i->total_production[1];
@@ -775,20 +851,20 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 	fr->cargo = cargo;
 
 	if (cargo == CT_GOODS || cargo == CT_FOOD) {
-		// The destination is a city
-		City *c = DEREF_CITY(RandomRange(_total_cities));
-		
-		// Only want big cities
-		if (c->xy == 0 || c->population < 900)
+		// The destination is a town
+		Town *t = DEREF_TOWN(InteractiveRandomRange(_total_towns));
+
+		// Only want big towns
+		if (t->xy == 0 || t->population < 900)
 			return;
-		fr->distance = GetTileDist(i->xy, c->xy);
-		fr->to = c;
+		fr->distance = GetTileDist(i->xy, t->xy);
+		fr->to = t;
 	} else {
 		// The destination is an industry
-		Industry *i2 = DEREF_INDUSTRY(RandomRange(_total_industries));
-		
+		Industry *i2 = DEREF_INDUSTRY(InteractiveRandomRange(_total_industries));
+
 		// The industry must accept the cargo
-		if (i == i2 || i2->xy == 0 || 
+		if (i == i2 || i2->xy == 0 ||
 				(cargo != i2->accepts_cargo[0] &&
 				cargo != i2->accepts_cargo[1] &&
 				cargo != i2->accepts_cargo[2]))
@@ -798,15 +874,15 @@ static void FindSubsidyCargoRoute(FoundRoute *fr)
 	}
 }
 
-static bool CheckSubsidyDuplicate(Subsidy *s) 
+static bool CheckSubsidyDuplicate(Subsidy *s)
 {
 	Subsidy *ss;
 
 	for(ss=_subsidies; ss != endof(_subsidies); ss++) {
-		if (s != ss && 
-//				ss->age < 12 && 
-				ss->from == s->from && 
-				ss->to == s->to && 
+		if (s != ss &&
+//				ss->age < 12 &&
+				ss->from == s->from &&
+				ss->to == s->to &&
 				ss->cargo_type == s->cargo_type) {
 			s->cargo_type = 0xFF;
 			return true;
@@ -815,6 +891,24 @@ static bool CheckSubsidyDuplicate(Subsidy *s)
 	return false;
 }
 
+void RemoteSubsidyAdd(Subsidy *s_new)
+{
+	Subsidy *s;
+	Pair pair;
+
+	// search the first free subsidy
+	for(s=_subsidies; s != endof(_subsidies); s++)
+		if (s->cargo_type == 0xFF)
+			break;
+
+	memcpy(s,s_new,sizeof(Subsidy));
+
+	pair = SetupSubsidyDecodeParam(s, 0);
+	AddNewsItem(STR_2030_SERVICE_SUBSIDY_OFFERED, NEWS_FLAGS(NM_NORMAL, NF_TILE, NT_SUBSIDIES, 0), pair.a, pair.b);
+
+	InvalidateWindow(WC_SUBSIDIES_LIST, 0);
+}
+
 static void SubsidyMonthlyHandler()
 {
 	Subsidy *s;
@@ -846,39 +940,42 @@ static void SubsidyMonthlyHandler()
 		}
 	}
 
+	if ((_networking) && (!_networking_server)) return;
+
 	// 25% chance to go on
-	if (CHANCE16(1,4)) {
+	if (ICHANCE16(1,4)) {
 		// Find a free slot
 		s = _subsidies;
 		while (s->cargo_type != 0xFF) {
 			if (++s == endof(_subsidies))
 				goto no_add;
 		}
-		
+
 		n = 1000;
 		do {
 			FindSubsidyPassengerRoute(&fr);
 			if (fr.distance <= 70) {
 				s->cargo_type = CT_PASSENGERS;
-				s->from = ((City*)fr.from)->index;
-				s->to = ((City*)fr.to)->index;
+				s->from = ((Town*)fr.from)->index;
+				s->to = ((Town*)fr.to)->index;
 				goto add_subsidy;
 			}
 			FindSubsidyCargoRoute(&fr);
 			if (fr.distance <= 70) {
 				s->cargo_type = fr.cargo;
 				s->from = (Industry*)fr.from - _industries;
-				s->to = (fr.cargo == CT_GOODS || fr.cargo == CT_FOOD) ? ((City*)fr.to)->index : (Industry*)fr.to - _industries;
+				s->to = (fr.cargo == CT_GOODS || fr.cargo == CT_FOOD) ? ((Town*)fr.to)->index : (Industry*)fr.to - _industries;
 	add_subsidy:
 				if (!CheckSubsidyDuplicate(s)) {
 					s->age = 0;
 					pair = SetupSubsidyDecodeParam(s, 0);
+					if (_networking_server) NetworkSendEvent(NET_EVENT_SUBSIDY,sizeof(Subsidy),s);
 					AddNewsItem(STR_2030_SERVICE_SUBSIDY_OFFERED, NEWS_FLAGS(NM_NORMAL, NF_TILE, NT_SUBSIDIES, 0), pair.a, pair.b);
 					modified = true;
 					break;
 				}
 			}
-		} while (--n);
+		} while (n--);
 	}
 no_add:;
 	if (modified)
@@ -927,7 +1024,7 @@ int32 GetTransportedGoodsIncome(uint num_pieces, uint dist, byte transit_days, b
 	if (transit_days > _cargoc.transit_days_1[cargo]) {
 		transit_days -= _cargoc.transit_days_1[cargo];
 		f -= transit_days;
-		
+
 		if (transit_days > _cargoc.transit_days_2[cargo]) {
 			transit_days -= _cargoc.transit_days_2[cargo];
 
@@ -944,19 +1041,19 @@ int32 GetTransportedGoodsIncome(uint num_pieces, uint dist, byte transit_days, b
 
 static void DeliverGoodsToIndustry(TileIndex xy, byte cargo_type, int num_pieces)
 {
-	Industry *ind, *best;	
+	Industry *ind, *best;
 	int t, u;
 
 	/* Check if there's an industry close to the station that accepts
 	 * the cargo */
 	best = NULL;
-	u = 0x11;
+	u = _patches.station_spread + 8;
 	for(ind = _industries; ind != endof(_industries); ind++) {
 		if (ind->xy != 0 && (cargo_type == ind->accepts_cargo[0] || cargo_type
 				 == ind->accepts_cargo[1] || cargo_type == ind->accepts_cargo[2]) &&
 				 ind->produced_cargo[0] != 0xFF &&
 				 ind->produced_cargo[0] != cargo_type &&
-				 (t=GetTileDist(ind->xy, xy)) < u) {			
+				 (t=GetTileDist(ind->xy, xy)) < u) {
 			u = t;
 			best = ind;
 		}
@@ -988,19 +1085,19 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 	/* check if there's a new subsidy that applies.. */
 	for(s=_subsidies; s != endof(_subsidies); s++) {
 		if (s->cargo_type == cargo_type && s->age < 12) {
-		
+
 			/* Check distance from source */
 			if (cargo_type == CT_PASSENGERS || cargo_type == CT_MAIL) {
-				xy = DEREF_CITY(s->from)->xy;
+				xy = DEREF_TOWN(s->from)->xy;
 			} else {
 				xy = (DEREF_INDUSTRY(s->from))->xy;
 			}
 			if (GetTileDist1D(xy, from->xy) > 9)
 				continue;
-			
+
 			/* Check distance from dest */
 			if (cargo_type == CT_PASSENGERS || cargo_type == CT_MAIL || cargo_type == CT_GOODS || cargo_type == CT_FOOD) {
-				xy = DEREF_CITY(s->to)->xy;
+				xy = DEREF_TOWN(s->to)->xy;
 			} else {
 				xy = (DEREF_INDUSTRY(s->to))->xy;
 			}
@@ -1021,7 +1118,7 @@ static bool CheckSubsidised(Station *from, Station *to, byte cargo_type)
 			SET_DPARAM16(0, p->name_1);
 			SET_DPARAM32(1, p->name_2);
 			AddNewsItem(
-				STR_2031_SERVICE_SUBSIDY_AWARDED + _opt.diff.subsidy_multiplier, 
+				STR_2031_SERVICE_SUBSIDY_AWARDED + _opt.diff.subsidy_multiplier,
 				NEWS_FLAGS(NM_NORMAL, NF_TILE, NT_SUBSIDIES, 0),
 				pair.a, pair.b);
 
@@ -1054,16 +1151,16 @@ static int32 DeliverGoods(int num_pieces, byte cargo_type, byte source, byte des
 	// Check if a subsidy applies.
 	subsidised = CheckSubsidised(s_from, s_to, cargo_type);
 
-	// Increase city's counter for some special goods types
+	// Increase town's counter for some special goods types
 	{
-		City *city = s_to->city;
-		if (cargo_type == CT_FOOD) city->new_act_food += num_pieces;
-		if (cargo_type == CT_STEEL)  city->new_act_paper += num_pieces;
+		Town *t = s_to->town;
+		if (cargo_type == CT_FOOD) t->new_act_food += num_pieces;
+		if (cargo_type == CT_STEEL)  t->new_act_water += num_pieces;
 	}
 
 	// Give the goods to the industry.
 	DeliverGoodsToIndustry(s_to->xy, cargo_type, num_pieces);
-	
+
 	// Determine profit
 	{
 		int t = GetTileDist(s_from->xy, s_to->xy);
@@ -1122,7 +1219,7 @@ int LoadUnloadVehicle(Vehicle *v)
 	for(;;) {
 		if (v->cargo_cap == 0)
 			goto next_vehicle;
-		
+
 		ge = &st->goods[v->cargo_type];
 
 		/* unload? */
@@ -1136,7 +1233,7 @@ int LoadUnloadVehicle(Vehicle *v)
 			} else if (u->next_order & OF_UNLOAD) {
 				/* unload goods and let it wait at the station */
 				st->time_since_unload = 0;
-				
+
 				if ((t=ge->waiting_acceptance & 0xFFF) == 0) {
 					// No goods waiting at station
 					ge->enroute_time = v->cargo_days;
@@ -1154,7 +1251,7 @@ int LoadUnloadVehicle(Vehicle *v)
 				v->cargo_count = 0;
 			}
 		}
-	
+
 		/* don't pick up goods that we unloaded */
 		if (u->next_order & OF_UNLOAD)
 			goto next_vehicle;
@@ -1163,7 +1260,7 @@ int LoadUnloadVehicle(Vehicle *v)
 		ge->days_since_pickup = 0;
 		t = u->max_speed;
 		if (u->type == VEH_Road) t >>=1;
-		
+
 		// if last speed is 0, we treat that as if no vehicle has ever visited the station.
 		ge->last_speed = t < 255 ? t : 255;
 		ge->last_age = _cur_year - v->build_year;
@@ -1177,7 +1274,7 @@ int LoadUnloadVehicle(Vehicle *v)
 			ge->waiting_acceptance -= cap;
 			unloading_time += cap;
 			st->time_since_load = 0;
-			
+
 			// And record the source of the cargo, and the days in travel.
 			v->cargo_source = ge->enroute_from;
 			v->cargo_days = ge->enroute_time;
@@ -1227,9 +1324,11 @@ next_vehicle:;
 void PlayersMonthlyLoop()
 {
 	PlayersGenStatistics();
-	if (!_patches.noinflation)
+	if (_patches.inflation)
 		AddInflation();
 	PlayersPayInterest();
+	// Reset the _current_player flag
+	_current_player = OWNER_NONE;
 	HandleEconomyFluctuations();
 	SubsidyMonthlyHandler();
 }
@@ -1238,7 +1337,7 @@ static void DoAcquireCompany(Player *p)
 {
 	Player *owner;
 	int i,pi;
-	int32 value;
+	int64 value;
 
 	SET_DPARAM16(0, p->name_1);
 	SET_DPARAM32(1, p->name_2);
@@ -1272,7 +1371,7 @@ static void DoAcquireCompany(Player *p)
 int32 CmdBuyShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	Player *p;
-	int32 cost;
+	int64 cost;
 	byte *b;
 	int i;
 
@@ -1300,7 +1399,7 @@ int32 CmdBuyShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 int32 CmdSellShareInCompany(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	Player *p;
-	int32 cost;
+	int64 cost;
 	byte *b;
 
 	SET_EXPENSES_TYPE(EXPENSES_OTHER);
