@@ -3,7 +3,7 @@
 #include "viewport.h"
 #include "command.h"
 #include "industry.h"
-#include "city.h"
+#include "town.h"
 #include "vehicle.h"
 #include "news.h"
 #include "saveload.h"
@@ -194,7 +194,7 @@ static void IndustryDrawTileProc1(TileInfo *ti)
 
 	if (!(_map_owner[ti->tile] & 0x80))
 		return;
-	
+
 	d = &_draw_industry_spec1[_map3_lo[ti->tile]];
 
 	AddChildSpriteScreen(0x12A7 + d->image_1, d->x, 0);
@@ -203,14 +203,14 @@ static void IndustryDrawTileProc1(TileInfo *ti)
 		AddChildSpriteScreen(0x12B0 + image - 1, 8, 41);
 
 	if ( (image = d->image_3) != 0)
-		AddChildSpriteScreen(0x12AC + image - 1, 
+		AddChildSpriteScreen(0x12AC + image - 1,
 			_drawtile_proc1_x[image-1], _drawtile_proc1_y[image-1]);
 }
 
 static void IndustryDrawTileProc2(TileInfo *ti)
 {
 	int x = 0;
-	
+
 	if (_map_owner[ti->tile] & 0x80) {
 		x = _industry_anim_offs[_map3_lo[ti->tile]];
 		if ( (byte)x == 0xFF)
@@ -224,7 +224,7 @@ static void IndustryDrawTileProc2(TileInfo *ti)
 static void IndustryDrawTileProc3(TileInfo *ti)
 {
 	if (_map_owner[ti->tile] & 0x80) {
-		AddChildSpriteScreen(0x128B, 5, 
+		AddChildSpriteScreen(0x128B, 5,
 			_industry_anim_offs_2[_map3_lo[ti->tile]]);
 	}
 	AddChildSpriteScreen(0x30B928A, 3, 67);
@@ -254,7 +254,7 @@ static void DrawCoalPlantSparkles(TileInfo *ti)
 	if (image & 0x80) {
 		image = (image >> 2) & 0x1F;
 		if (image != 0 && image < 7) {
-			AddChildSpriteScreen(image + 0x806, 
+			AddChildSpriteScreen(image + 0x806,
 				_coal_plant_sparkles_x[image-1],
 				_coal_plant_sparkles_y[image-1]
 			);
@@ -305,11 +305,11 @@ static void DrawTile_Industry(TileInfo *ti)
 
 		if (image&0x8000 && (image & 0xFFFF0000) == 0)
 			image |= ormod;
-			
+
 		if (!(_display_opt & DO_TRANS_BUILDINGS))
 			image = (image & 0x3FFF) | 0x3224000;
-		
-		AddSortableSpriteToDraw(image, 
+
+		AddSortableSpriteToDraw(image,
 			ti->x | (dits->subtile_xy>>4),
 			ti->y | (dits->subtile_xy&0xF),
 			(dits->width_height>>4)+1,
@@ -321,7 +321,7 @@ static void DrawTile_Industry(TileInfo *ti)
 			return;
 	}
 
-	/* TTDBUG: strange code here, return if AddSortableSpriteToDraw failed?  */	
+	/* TTDBUG: strange code here, return if AddSortableSpriteToDraw failed?  */
 		{
 		int proc;
 		if ((proc=dits->proc-1) >= 0 )
@@ -334,6 +334,10 @@ static uint GetSlopeZ_Industry(TileInfo *ti) {
 	return GetPartialZ(ti->x&0xF, ti->y&0xF, ti->tileh) + ti->z;
 }
 
+static uint GetSlopeTileh_Industry(TileInfo *ti) {
+	return 0;
+}
+
 static void GetAcceptedCargo_Industry(uint tile, AcceptedCargo *ac)
 {
 	int m5 = _map5[tile];
@@ -374,7 +378,14 @@ static int32 ClearTile_Industry(uint tile, byte flags)
 {
 	Industry *i = DEREF_INDUSTRY(_map2[tile]);
 
- if ((_current_player == OWNER_WATER || _game_mode != GM_EDITOR) && !_cheats.magic_bulldozer.value) {
+	/*	* water can destroy industries
+			* in editor you can bulldoze industries
+			* with magic_bulldozer cheat you can destroy industries
+			* (area around OILRIG is water, so water shouldn't flood it
+	*/
+	if ((_current_player != OWNER_WATER && _game_mode != GM_EDITOR &&
+			!_cheats.magic_bulldozer.value) ||
+			(_current_player == OWNER_WATER && i->type == IT_OIL_RIG) ) {
  		SET_DPARAM16(0, STR_4802_COAL_MINE + i->type);
 		return_cmd_error(STR_4800_IN_THE_WAY);
 	}
@@ -384,28 +395,26 @@ static int32 ClearTile_Industry(uint tile, byte flags)
 	}
 	return 0;
 }
- 
+
 /* p1 index of industry to destroy */
 /* Destroy Industry button costing money removed per request of dominik
 int32 CmdDestroyIndustry(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	uint tile = TILE_FROM_XY(x,y);
 	Industry *i = DEREF_INDUSTRY((uint16)p1);
-  City *c = ClosestCityFromTile(tile, (uint)-1); // find closest city to penaltize (ALWAYS penaltize)
+  Town *t = ClosestTownFromTile(tile, (uint)-1); // find closest town to penaltize (ALWAYS penaltize)
+
+	// check if you're allowed to remove the industry. Minimum amount
+	// of ratings to remove the industry depends on difficulty setting
+	if (!CheckforTownRating(tile, flags, t, INDUSTRY_REMOVE))
+		return CMD_ERROR;
 
-	// check if you're allowed to remove the industry. you'll have to have a 
-	// city rating of "Very Good" to remove industries. the "Very Good" should depend on diff. settings
-	if (c->ratings[_current_player] < 450) {
-		SET_DPARAM16(0, c->index);
-		return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
-	}
-		
 	if (flags & DC_EXEC) {
 		DeleteIndustry(i);
 		CreateEffectVehicleAbove(x + 8,y + 8, 2, EV_DEMOLISH);
-		ChangeCityRating(c, -1500, -1000);	// penalty is 1500
+		ChangeTownRating(t, -1500, -1000);	// penalty is 1500
 	}
-	
+
 	return (_price.build_industry >> 5) * _industry_type_costs[i->type]*2;
 }
 */
@@ -426,7 +435,7 @@ void TransportIndustryGoods(uint tile)
 	byte m5;
 
 	i = DEREF_INDUSTRY(_map2[tile]);
-	
+
 	type = i->type;
 	cw = min(i->cargo_waiting[0], 255);
 	if (cw > _industry_min_cargo[type]/* && i->produced_cargo[0] != 0xFF*/) {
@@ -450,9 +459,9 @@ void TransportIndustryGoods(uint tile)
 	cw = min(i->cargo_waiting[1], 255);
 	if (cw > _industry_min_cargo[type]) {
 		i->cargo_waiting[1] -= cw;
-		
+
 		if (_economy.fluct <= 0) cw = (cw + 1) >> 1;
-		
+
 		i->last_mo_production[1] += cw;
 
 		am = MoveGoodsToStation(i->xy, i->width, i->height, i->produced_cargo[1], cw);
@@ -469,7 +478,7 @@ static void AnimateTile_Industry(uint tile)
 	case 174:
 		if ((_tick_counter & 1) == 0) {
 			m = _map3_lo[tile] + 1;
-			
+
 			switch(m & 7) {
 			case 2:	SndPlayTileFx(45, tile); break;
 			case 6: SndPlayTileFx(41, tile); break;
@@ -488,7 +497,7 @@ static void AnimateTile_Industry(uint tile)
 	case 165:
 		if ((_tick_counter & 3) == 0) {
 			m = _map3_lo[tile];
-			
+
 			if (_industry_anim_offs[m] == 0xFF) {
 				SndPlayTileFx(48, tile);
 			}
@@ -547,7 +556,7 @@ static void AnimateTile_Industry(uint tile)
 				_map3_hi[tile] = 0;
 				DeleteAnimatedTile(tile);
 			}
-			_map3_lo[tile] = m;			
+			_map3_lo[tile] = m;
 			MarkTileDirtyByTile(tile);
 		}
 		break;
@@ -572,7 +581,7 @@ static void AnimateTile_Industry(uint tile)
 			if (m == 4 && (m=0,++n) == 32+1 && (n=30,b)) {
 				_map_owner[tile] = 0x83;
 				_map5[tile] = 29;
-				DeleteAnimatedTile(tile);	
+				DeleteAnimatedTile(tile);
 			} else {
 				_map5[tile] = n;
 				_map_owner[tile] = (_map_owner[tile] & ~3) | m;
@@ -639,7 +648,7 @@ static void MakeIndustryTileBigger(uint tile, byte size)
 	size = (size + 1) & 3;
 	if (size == 3) size |= 0x80;
 	_map_owner[tile] = size | b;
-	
+
 	MarkTileDirtyByTile(tile);
 
 	if (!(_map_owner[tile] & 0x80))
@@ -683,7 +692,7 @@ static void TileLoopIndustryCase161(uint tile)
 	};
 
 	SndPlayTileFx(46, tile);
-	
+
 	dir = Random() & 3;
 
 	v = CreateEffectVehicleAbove(
@@ -723,6 +732,14 @@ static void TileLoop_Industry(uint tile)
 #define SET_AND_UNANIMATE(tile,a,b) { _map5[tile]=a; _map_owner[tile]=b; DeleteAnimatedTile(tile); }
 
 	switch(_map5[tile]) {
+	case 0x18: // coast line at oilrigs
+	case 0x19:
+	case 0x1A:
+	case 0x1B:
+	case 0x1C:
+		TileLoop_Water(tile);
+		break;
+
 	case 0:
 		if (!(_tick_counter & 0x400) && CHANCE16(1,2))
 			SET_AND_ANIMATE(tile,1,0x80);
@@ -766,7 +783,7 @@ static void TileLoop_Industry(uint tile)
 		break;
 
 	case 49: {
-		CreateEffectVehicleAbove(GET_TILE_X(tile)*16 + 6, GET_TILE_Y(tile)*16 + 6, 43, EV_SMOKE_3); 
+		CreateEffectVehicleAbove(GET_TILE_X(tile)*16 + 6, GET_TILE_Y(tile)*16 + 6, 43, EV_SMOKE_3);
 	} break;
 
 
@@ -783,8 +800,8 @@ static void TileLoop_Industry(uint tile)
 
 	case 161:
 		TileLoopIndustryCase161(tile);
-		break;		
-	
+		break;
+
 	case 165:
 		AddAnimatedTile(tile);
 		break;
@@ -803,7 +820,7 @@ static void ClickTile_Industry(uint tile)
 	ShowIndustryViewWindow(_map2[tile]);
 }
 
-static uint32 GetTileTrackStatus_Industry(uint tile, int mode)
+static uint32 GetTileTrackStatus_Industry(uint tile, TransportType mode)
 {
 	return 0;
 }
@@ -823,7 +840,7 @@ static void ChangeTileOwner_Industry(uint tile, byte old_player, byte new_player
 void DeleteIndustry(Industry *i)
 {
 	int index = i - _industries;
-	
+
 	BEGIN_TILE_LOOP(tile_cur,	i->width, i->height, i->xy);
 		if (IS_TILETYPE(tile_cur, MP_INDUSTRY)) {
 			if (_map2[tile_cur] == (byte)index) {
@@ -833,7 +850,7 @@ void DeleteIndustry(Industry *i)
 			DeleteOilRig(tile_cur);
 		}
 	END_TILE_LOOP(tile_cur,	i->width, i->height, i->xy);
-	
+
 	i->xy = 0;
 	_industry_sort_dirty = true;
 	DeleteSubsidyWithIndustry(index);
@@ -877,9 +894,9 @@ static void SetupFarmFieldFence(uint tile, int size, byte type, int direction)
 
 	do {
 		tile = TILE_MASK(tile);
-		
+
 		if (IS_TILETYPE(tile, MP_CLEAR) || IS_TILETYPE(tile, MP_TREES)) {
-			
+
 			or = type;
 			if (or == 1 && (uint16)Random() <= 9362) or = 2;
 
@@ -891,7 +908,7 @@ static void SetupFarmFieldFence(uint tile, int size, byte type, int direction)
 			}
 			_map3_hi[tile] = (_map3_hi[tile] & and) | or;
 		}
-		
+
 		tile += direction ? TILE_XY(0,1) : TILE_XY(1,0);
 	} while (--size);
 }
@@ -916,7 +933,7 @@ static void PlantFarmField(uint tile)
 
 	/* offset tile to match size */
 	tile = tile - TILE_XY(size_x>>1, size_y>>1);
-	
+
 	/* check the amount of bad tiles */
 	count = 0;
 	BEGIN_TILE_LOOP(cur_tile, size_x, size_y, tile)
@@ -935,15 +952,15 @@ static void PlantFarmField(uint tile)
 	BEGIN_TILE_LOOP(cur_tile, size_x, size_y, tile)
 		cur_tile = TILE_MASK(cur_tile);
 		if (!IsBadFarmFieldTile2(cur_tile)) {
-			ModifyTile(cur_tile, 
-				MP_SETTYPE(MP_CLEAR) | 
+			ModifyTile(cur_tile,
+				MP_SETTYPE(MP_CLEAR) |
 				MP_MAP2_CLEAR | MP_MAP3LO | MP_MAP3HI_CLEAR | MP_MAPOWNER | MP_MAP5,
 				type2,			/* map3_lo */
 				OWNER_NONE,	/* map_owner */
 				type);			/* map5 */
 		}
 	END_TILE_LOOP(cur_tile, size_x, size_y, tile)
-	
+
 	type = 3;
 	if (_opt.landscape != LT_HILLY && _opt.landscape != LT_DESERT) {
 		type = _plantfarmfield_type[Random() & 0xF];
@@ -959,16 +976,16 @@ static void MaybePlantFarmField(Industry *i)
 {
 	uint tile;
 	if (CHANCE16(1,8)) {
-		tile = TileAddWrap(i->xy, TILE_XY(i->width>>1, i->height>>1) + (Random()&TILE_XY(31,31))-TILE_XY(16,16));
+		tile = TileAddWrap(i->xy, ((i->width>>1) + Random() % 31 - 16), ((i->height>>1) + Random() % 31 - 16));
 		if (tile != TILE_WRAPPED)
 			PlantFarmField(tile);
-	}	
+	}
 }
 
 static void ChopLumberMillTrees(Industry *i)
 {
 	static const TileIndexDiff _chop_dir[4] = { TILE_XY(0,1), TILE_XY(1,0), TILE_XY(0,-1), TILE_XY(-1,0) };
-	
+
 	uint tile = i->xy;
 	int dir, a, j;
 
@@ -1079,9 +1096,9 @@ void OnTick_Industry()
 		_industry_sound_ctr++;
 
 		if (_industry_sound_ctr == 75) {
-			SndPlayTileFx(55, _industry_sound_tile); 
+			SndPlayTileFx(55, _industry_sound_tile);
 		} else if (_industry_sound_ctr == 160) {
-			_industry_sound_ctr = 0;	
+			_industry_sound_ctr = 0;
 			SndPlayTileFx(54, _industry_sound_tile);
 		}
 	}
@@ -1112,9 +1129,15 @@ static bool CheckNewIndustry_Forest(uint tile, int type)
 	return true;
 }
 
+extern bool _ignore_restrictions;
+
 static bool CheckNewIndustry_Oilwell(uint tile, int type)
 {
 	int x,y;
+
+	if(_ignore_restrictions && _game_mode == GM_EDITOR)
+		return true;
+
 	if (type != IT_OIL_RIG && _game_mode == GM_EDITOR)
 		return true;
 
@@ -1201,26 +1224,26 @@ static bool CheckSuitableIndustryPos(uint tile)
 	return true;
 }
 
-static City *CheckMultipleIndustryInTown(uint tile, int type)
+static Town *CheckMultipleIndustryInTown(uint tile, int type)
 {
-	City *c;
+	Town *t;
 	Industry *i;
 
-	c = ClosestCityFromTile(tile, (uint)-1);
-	
+	t = ClosestTownFromTile(tile, (uint)-1);
+
 	if (_patches.multiple_industry_per_town)
-		return c;
+		return t;
 
 	for(i=_industries; i != endof(_industries); i++) {
 		if (i->xy != 0 &&
 				i->type == (byte)type &&
-				i->city == c) {
+				i->town == t) {
 			_error_message = STR_0287_ONLY_ONE_ALLOWED_PER_TOWN;
 			return NULL;
 		}
 	}
 
-	return c;
+	return t;
 }
 
 static const byte _industry_map5_bits[] = {
@@ -1248,7 +1271,7 @@ static const byte _industry_map5_bits[] = {
 	16, 16, 16, 16, 16, 16, 16,
 };
 
-static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it, int type, City *city)
+static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it, int type, Town *t)
 {
 	TileInfo ti;
 	uint cur_tile;
@@ -1262,9 +1285,9 @@ static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it,
 				continue;
 			return false;
 		}
-			
+
 		FindLandscapeHeightByTile(&ti, cur_tile);
-		
+
 		if (it->map5 == 0xFF) {
 			if (ti.type != MP_WATER || ti.tileh != 0)
 				return false;
@@ -1287,7 +1310,7 @@ static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it,
 
 					if (bits & 0x10)
 						return false;
-					
+
 					t = ~ti.tileh;
 
 					if (bits & 1 && (t & (1+8)))
@@ -1304,7 +1327,7 @@ static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it,
 				}
 
 				if (type == IT_BANK) {
-					if (ti.type != MP_HOUSE || city->population < 1200) {
+					if (ti.type != MP_HOUSE || t->population < 1200) {
 						_error_message = STR_029D_CAN_ONLY_BE_BUILT_IN_TOWNS;
 						return false;
 					}
@@ -1314,14 +1337,14 @@ static bool CheckIfIndustryTilesAreFree(uint tile, const IndustryTileTable *it,
 						return false;
 					}
 				} else if (type == IT_TOY_SHOP) {
-					if (GetTileDist1D(city->xy, cur_tile) > 9)
+					if (GetTileDist1D(t->xy, cur_tile) > 9)
 						return false;
 					if (ti.type != MP_HOUSE) goto do_clear;
 				} else if (type == IT_WATER_TOWER) {
 					if (ti.type != MP_HOUSE) {
 						_error_message = STR_0316_CAN_ONLY_BE_BUILT_IN_TOWNS;
 						return false;
-					}					
+					}
 				} else {
 do_clear:
 					if (DoCommandByTile(cur_tile, 0, 0, DC_AUTO, CMD_LANDSCAPE_CLEAR) == CMD_ERROR)
@@ -1343,7 +1366,7 @@ static bool CheckIfTooCloseToIndustry(uint tile, int type)
 	// accepting industries won't be close, not even with patch
 	if (_patches.same_industry_close && (spec->accepts_cargo[0] == 0xFF) )
 		return true;
-	
+
 	for(i=_industries; i != endof(_industries); i++) {
 		// check if an industry that accepts the same goods is nearby
 		if (i->xy != 0 &&
@@ -1379,7 +1402,7 @@ static Industry *AllocateIndustry()
 	return NULL;
 }
 
-static void DoCreateNewIndustry(Industry *i, uint tile, int type, const IndustryTileTable *it, City *city, byte owner)
+static void DoCreateNewIndustry(Industry *i, uint tile, int type, const IndustryTileTable *it, Town *t, byte owner)
 {
 	const IndustrySpec *spec;
 	uint32 r;
@@ -1405,7 +1428,7 @@ static void DoCreateNewIndustry(Industry *i, uint tile, int type, const Industry
 		i->production_rate[1] = min((RandomRange(256) + 128) * i->production_rate[1] >> 8 , 255);
 	}
 
-	i->city = city;
+	i->town = t;
 	i->owner = owner;
 
 	r = Random() & 0xFFF;
@@ -1428,7 +1451,7 @@ static void DoCreateNewIndustry(Industry *i, uint tile, int type, const Industry
 
 	if (_generating_world == 0)
 		i->total_production[0] = i->total_production[1] = 0;
-	
+
 	i->prod_level = 0x10;
 
 	do {
@@ -1457,10 +1480,10 @@ static void DoCreateNewIndustry(Industry *i, uint tile, int type, const Industry
 	if (i->type == IT_FARM || i->type == IT_FARM_2) {
 		tile = i->xy + TILE_XY((i->width >> 1), (i->height >> 1));
 		for(j=0; j!=50; j++) {
-			uint new_tile = TileAddWrap(tile, (Random() & TILE_XY(31,31)) - TILE_XY(16,16));
+			uint new_tile = TileAddWrap(tile, Random() % 31 - 16, Random() % 31 - 16);
 			if (new_tile != TILE_WRAPPED)
 				PlantFarmField(new_tile);
-		}	
+		}
 	}
 	_industry_sort_dirty = true;
 	InvalidateWindow(WC_INDUSTRY_DIRECTORY, 0);
@@ -1470,7 +1493,7 @@ static void DoCreateNewIndustry(Industry *i, uint tile, int type, const Industry
 int32 CmdBuildIndustry(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	uint tile = TILE_FROM_XY(x,y);
-	City *city;
+	Town *t;
 	int num;
 	const IndustryTileTable * const *itt;
 	const IndustryTileTable *it;
@@ -1487,16 +1510,16 @@ int32 CmdBuildIndustry(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (!_check_new_industry_procs[spec->check_proc](tile, p1))
 		return CMD_ERROR;
 
-	if ((city=CheckMultipleIndustryInTown(tile, p1)) == NULL)
+	if ((t=CheckMultipleIndustryInTown(tile, p1)) == NULL)
 		return CMD_ERROR;
-	
+
 	num = spec->num_table;
 	itt = spec->table;
 
 	do {
 		if (--num < 0)
 			return_cmd_error(STR_0239_SITE_UNSUITABLE);
-	} while (!CheckIfIndustryTilesAreFree(tile, it=itt[num], p1, city));
+	} while (!CheckIfIndustryTilesAreFree(tile, it=itt[num], p1, t));
 
 
 	if (!CheckIfTooCloseToIndustry(tile, p1))
@@ -1506,15 +1529,15 @@ int32 CmdBuildIndustry(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		return CMD_ERROR;
 
 	if (flags & DC_EXEC)
-		DoCreateNewIndustry(i, tile, p1, it, city, 0x10);
-	
+		DoCreateNewIndustry(i, tile, p1, it, t, 0x10);
+
 	return (_price.build_industry >> 5) * _industry_type_costs[p1];
 }
 
 
 Industry *CreateNewIndustry(uint tile, int type)
 {
-	City *city;
+	Town *t;
 	const IndustryTileTable *it;
 	Industry *i;
 
@@ -1528,13 +1551,13 @@ Industry *CreateNewIndustry(uint tile, int type)
 	if (!_check_new_industry_procs[spec->check_proc](tile, type))
 		return NULL;
 
-	if (!(city=CheckMultipleIndustryInTown(tile, type)))
+	if (!(t=CheckMultipleIndustryInTown(tile, type)))
 		return NULL;
 
 	/* pick a random layout */
 	it = spec->table[(spec->num_table * (uint16)Random()) >> 16];
 
-	if (!CheckIfIndustryTilesAreFree(tile, it, type, city))
+	if (!CheckIfIndustryTilesAreFree(tile, it, type, t))
 		return NULL;
 
 	if (!CheckIfTooCloseToIndustry(tile, type))
@@ -1543,12 +1566,13 @@ Industry *CreateNewIndustry(uint tile, int type)
 	if ( (i = AllocateIndustry()) == NULL)
 		return NULL;
 
-	DoCreateNewIndustry(i, tile, type, it, city, 0x10);
-	
+	DoCreateNewIndustry(i, tile, type, it, t, 0x10);
+
 	return i;
 }
 
-static const byte _numof_industry_table[3][12] = {
+static const byte _numof_industry_table[4][12] = {
+	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
 	{0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 5},
 	{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
 	{0, 2, 3, 4, 6, 7, 8, 9, 10, 10, 10},
@@ -1558,15 +1582,18 @@ static void PlaceInitialIndustry(byte type, int amount)
 {
 	int num = _numof_industry_table[_opt.diff.number_industries][amount];
 
-	assert(num > 0);
+	if (_opt.diff.number_industries != 0)
+	{
+		assert(num > 0);
 
-	do {
-		int i = 2000;
 		do {
-			if (CreateNewIndustry(TILE_MASK(Random()), type) != NULL)
-				break; 
-		} while (--i != 0);
-	} while (--num);
+			int i = 2000;
+			do {
+				if (CreateNewIndustry(TILE_MASK(Random()), type) != NULL)
+					break;
+			} while (--i != 0);
+		} while (--num);
+	}
 }
 
 void GenerateIndustries()
@@ -1600,7 +1627,7 @@ static void ExtChangeIndustryProduction(Industry *i)
 			change = old = i->production_rate[j];
 			if (CHANCE16R(20,1024,r))change -= ((RandomRange(50) + 10)*old) >> 8;
 			if (CHANCE16I(20+(i->pct_transported[j]*20>>8),1024,r>>16)) change += ((RandomRange(50) + 10)*old) >> 8;
-			
+
 			// make sure it doesn't exceed 255 or goes below 0
 			change = clamp(change, 0, 255);
 			if (change == old) {
@@ -1613,14 +1640,14 @@ static void ExtChangeIndustryProduction(Industry *i)
 
 			if (change >= _industry_spec[i->type].production_rate[j]/4)
 				closeit = false;
-				
+
 			mag = abs(percent);
 			if (mag >= 10) {
 				SET_DPARAM16(3, mag);
 				SET_DPARAM16(0,_cargoc.names_s[i->produced_cargo[j]]);
-				SET_DPARAM16(1, i->city->index);
+				SET_DPARAM16(1, i->town->index);
 				SET_DPARAM16(2, i->type + STR_4802_COAL_MINE);
-				AddNewsItem(percent>=0 ? STR_INDUSTRY_PROD_GOUP : STR_INDUSTRY_PROD_GODOWN, 
+				AddNewsItem(percent>=0 ? STR_INDUSTRY_PROD_GOUP : STR_INDUSTRY_PROD_GODOWN,
 						NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ECONOMY, 0),
 						i->xy + TILE_XY(1,1), 0);
 			}
@@ -1630,7 +1657,7 @@ static void ExtChangeIndustryProduction(Industry *i)
 	if (closeit) {
 		i->prod_level = 0;
 		SET_DPARAM16(1, i->type + STR_4802_COAL_MINE);
-		SET_DPARAM16(0, i->city->index);
+		SET_DPARAM16(0, i->town->index);
 		AddNewsItem(_industry_close_strings[i->type], NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ECONOMY, 0), i->xy + TILE_XY(1,1), 0);
 	}
 }
@@ -1639,13 +1666,13 @@ static void ExtChangeIndustryProduction(Industry *i)
 static void UpdateIndustryStatistics(Industry *i)
 {
 	byte pct;
-	
-	if (i->produced_cargo[0] != 0xFF) {		
+
+	if (i->produced_cargo[0] != 0xFF) {
 		pct = 0;
 		if (i->last_mo_production[0] != 0) {
 			i->last_prod_year = _cur_year;
 			pct = min(i->last_mo_transported[0] * 256 / i->last_mo_production[0],255);
-		} 
+		}
 		i->pct_transported[0] = pct;
 
 		i->total_production[0] = i->last_mo_production[0];
@@ -1660,7 +1687,7 @@ static void UpdateIndustryStatistics(Industry *i)
 		if (i->last_mo_production[1] != 0) {
 			i->last_prod_year = _cur_year;
 			pct = min(i->last_mo_transported[1] * 256 / i->last_mo_production[1],255);
-		} 
+		}
 		i->pct_transported[1] = pct;
 
 		i->total_production[1] = i->last_mo_production[1];
@@ -1711,7 +1738,7 @@ static void MaybeNewIndustry(uint32 r)
 	}
 
 	SET_DPARAM16(0, type + STR_4802_COAL_MINE);
-	SET_DPARAM16(1, i->city->index);
+	SET_DPARAM16(1, i->town->index);
 	AddNewsItem(	STR_482D_NEW_UNDER_CONSTRUCTION + (type == IT_FOREST), NEWS_FLAGS(NM_THIN,NF_VIEWPORT|NF_TILE,NT_ECONOMY,0), i->xy, 0);
 }
 
@@ -1720,7 +1747,7 @@ static void MaybeCloseIndustry(Industry *i)
 	uint32 r;
 	StringID str;
 	int type = i->type;
-			
+
 	if (_industry_close_mode[type] == 1) {
 		/* decrease or increase */
 		if (type == IT_OIL_WELL && _opt.landscape == LT_NORMAL) goto decrease_production;
@@ -1732,14 +1759,14 @@ static void MaybeCloseIndustry(Industry *i)
 					byte b;
 
 					i->prod_level <<= 1;
-					
+
 					b = i->production_rate[0]*2;
 					if (i->production_rate[0] >= 128) b=255;
 					i->production_rate[0] = b;
 					b = i->production_rate[1]*2;
 					if (i->production_rate[1] >= 128) b=255;
 					i->production_rate[1] = b;
-					
+
 					str = _industry_prod_up_strings[type];
 					goto add_news;
 				}
@@ -1755,7 +1782,7 @@ decrease_production:
 				str = _industry_prod_down_strings[type];
 				goto add_news;
 			}
-		} 
+		}
 	} else if (_industry_close_mode[type] > 1) {
 		/* maybe close */
 		if ( (byte)(_cur_year - i->last_prod_year) >= 5 && CHANCE16(1,2)) {
@@ -1764,7 +1791,7 @@ close_industry:
 			str = _industry_close_strings[type];
 add_news:
 			SET_DPARAM16(1, type + STR_4802_COAL_MINE);
-			SET_DPARAM16(0, i->city->index);
+			SET_DPARAM16(0, i->town->index);
 			AddNewsItem(str, NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_TILE, NT_ECONOMY, 0), i->xy + TILE_XY(1,1), 0);
 		}
 	}
@@ -1815,13 +1842,14 @@ const TileTypeProcs _tile_type_industry_procs = {
 	GetProducedCargo_Industry,  /* get_produced_cargo_proc */
 	NULL,												/* vehicle_enter_tile_proc */
 	NULL,												/* vehicle_leave_tile_proc */
+	GetSlopeTileh_Industry,			/* get_slope_tileh_proc */
 };
 
 static const byte _industry_desc[] = {
 	SLE_VAR(Industry,xy,							SLE_UINT16),
 	SLE_VAR(Industry,width,						SLE_UINT8),
 	SLE_VAR(Industry,height,					SLE_UINT8),
-	SLE_REF(Industry,city,						REF_CITY),
+	SLE_REF(Industry,town,						REF_TOWN),
 	SLE_ARR(Industry,produced_cargo,  SLE_UINT8, 2),
 	SLE_ARR(Industry,cargo_waiting,   SLE_UINT16, 2),
 	SLE_ARR(Industry,production_rate, SLE_UINT8, 2),
