@@ -11,7 +11,7 @@ static void AppendPart(byte **buf, int num, const char *names)
 	while (--num>=0) {
 		do names++; while (names[-1]);
 	}
-	
+
 	for(s=*buf; (*s++ = *names++) != 0;) {}
 	*buf = s - 1;
 }
@@ -19,14 +19,14 @@ static void AppendPart(byte **buf, int num, const char *names)
 #define MK(x) x "\x0"
 
 #define NUM_ENGLISH_1 4
-static const char english_1[] = 
+static const char english_1[] =
 	MK("Great ")
 	MK("Little ")
 	MK("New ")
 	MK("Fort ")
 ;
 
-#define NUM_ENGLISH_2 26 
+#define NUM_ENGLISH_2 26
 static const char english_2[] =
 	MK("Wr")
 	MK("B")
@@ -57,7 +57,7 @@ static const char english_2[] =
 ;
 
 #define NUM_ENGLISH_3 8
-static const char english_3[] = 
+static const char english_3[] =
 	MK("ar")
 	MK("a")
 	MK("e")
@@ -69,7 +69,7 @@ static const char english_3[] =
 ;
 
 #define NUM_ENGLISH_4 7
-static const char english_4[] = 
+static const char english_4[] =
 	MK("n")
 	MK("ning")
 	MK("ding")
@@ -80,7 +80,7 @@ static const char english_4[] =
 ;
 
 #define NUM_ENGLISH_5 23
-static const char english_5[] = 
+static const char english_5[] =
 	MK("ville")
 	MK("ham")
 	MK("field")
@@ -107,7 +107,7 @@ static const char english_5[] =
 ;
 
 #define NUM_ENGLISH_6 9
-static const char english_6[] = 
+static const char english_6[] =
 	MK("-on-sea")
 	MK(" Bay")
 	MK(" Market")
@@ -119,7 +119,9 @@ static const char english_6[] =
 	MK(" Springs")
 ;
 
-static byte MakeEnglishCityName(byte *buf, uint32 seed)
+#define REPLACE_WORDS(a,b,c,d,e,f,g,h)  { if (start[0] == a && start[1] == b && start[2] == c && start[3] == d) { start[0] = e; start[1] = f; start[2] = g; start[3] = h; } }
+
+static byte MakeEnglishTownName(byte *buf, uint32 seed)
 {
 	int i;
 	byte result;
@@ -128,7 +130,7 @@ static byte MakeEnglishCityName(byte *buf, uint32 seed)
 	i = GETNUM(0, 54) - 50;
 	if (i >= 0)
 		AppendPart(&buf, i, english_1);
-	
+
 	start = buf;
 
 	AppendPart(&buf, GETNUM(4, NUM_ENGLISH_2), english_2);
@@ -139,25 +141,33 @@ static byte MakeEnglishCityName(byte *buf, uint32 seed)
 	i = GETNUM(15, NUM_ENGLISH_6 + 60) - 60;
 
 	result = 0;
-	
+
 	if (i >= 0) {
 		if (i <= 1) result = NG_EDGE;
 		AppendPart(&buf, i, english_6);
 	}
 
 	if (start[0]=='C' && (start[1] == 'e' || start[1] == 'i'))
-		start[0] = 'K'; 
+		start[0] = 'K';
 
-	/* FIXME: skip the banned words thing for now 
-	only replacing "Cunt" with "Cult"   */
-	if (start[0]=='C' && start[1] == 'u' && start[2] == 'n' && start[3] == 't')
-		start[2] = 'l';
+	/* Replace certain words (routine identical to TTD now) */
+
+	REPLACE_WORDS('C','u','n','t',  'E','a','s','t');
+	REPLACE_WORDS('S','l','a','g',  'P','i','t','s');
+	REPLACE_WORDS('S','l','u','t',  'E','d','i','n');
+//	REPLACE_WORDS('F','a','r','t',  'B','o','o','t');
+	REPLACE_WORDS('D','r','a','r',  'Q','u','a','r');
+	REPLACE_WORDS('D','r','e','h',  'B','a','s','h');
+	REPLACE_WORDS('F','r','a','r',  'S','h','o','r');
+	REPLACE_WORDS('G','r','a','r',  'A','b','e','r');
+	REPLACE_WORDS('B','r','a','r',  'O','v','e','r');
+	REPLACE_WORDS('W','r','a','r',  'I','n','v','e');
 
 	return result;
 }
 
 #define NUM_AUSTRIAN_A1 6
-static const char austrian_a1[] = 
+static const char austrian_a1[] =
 	MK("Bad ")
 	MK("Deutsch ")
 	MK("Gross ")
@@ -167,7 +177,7 @@ static const char austrian_a1[] =
 ;
 
 #define NUM_AUSTRIAN_A2 42
-static const char austrian_a2[] = 
+static const char austrian_a2[] =
 	MK("Aus")
 	MK("Alten")
 	MK("Braun")
@@ -213,7 +223,7 @@ static const char austrian_a2[] =
 ;
 
 #define NUM_AUSTRIAN_A3 16
-static const char austrian_a3[] = 
+static const char austrian_a3[] =
 	MK("see")
 	MK("bach")
 	MK("dorf")
@@ -320,7 +330,7 @@ static const char austrian_b2[] =
 	MK("Ziller")
 ;
 
-static byte MakeAustrianCityName(byte *buf, uint32 seed)
+static byte MakeAustrianTownName(byte *buf, uint32 seed)
 {
 	int i, j=0;
 
@@ -361,8 +371,8 @@ static byte MakeAustrianCityName(byte *buf, uint32 seed)
 }
 
 #define NUM_GERMAN_1 114
-#define NUM_GERMAN_1_HARDCODED 21 
-static const char german_1[] = 
+#define NUM_GERMAN_1_HARDCODED 21
+static const char german_1[] =
 	/* hardcoded names */
 	MK("Berlin")
 	MK("Bonn")
@@ -521,20 +531,20 @@ static const char german_4[] =
 	MK("Elbe")
 	/* use "am" */
 	MK("Main")
-	
+
 ;
 
-static byte MakeGermanCityName(byte *buf, uint32 seed)
+static byte MakeGermanTownName(byte *buf, uint32 seed)
 {
 	int i;
 	int ext;
-	
+
 	ext=GETNUM(7, 28); /* Extension - Prefix / Suffix */
 
 	if ((ext==12) || (ext==19)) {
 		i=GETNUM(2,NUM_GERMAN_3-2);
 		AppendPart(&buf, 2+i, german_3);
-		}
+	}
 
 
 	i=GETNUM(3,NUM_GERMAN_1);
@@ -543,7 +553,7 @@ static byte MakeGermanCityName(byte *buf, uint32 seed)
 
 	if (i>NUM_GERMAN_1_HARDCODED-1) {
 		AppendPart(&buf, GETNUM(5, NUM_GERMAN_2), german_2);
-		}
+	}
 
 	if (ext==24) {
 		i=GETNUM(9,NUM_GERMAN_4);
@@ -551,17 +561,17 @@ static byte MakeGermanCityName(byte *buf, uint32 seed)
 		if (i<=NUM_GERMAN_4_PRECHANGE-1) {
 			AppendPart(&buf, 0, german_3);
 			AppendPart(&buf, i, german_4);
-			} else {
+		} else {
 			AppendPart(&buf, 1, german_3);
 			AppendPart(&buf, i, german_4);
-			}
 		}
+	}
 
 	return 0;
 }
 
 #define NUM_SPANISH_1 86
-static const char spanish_1[] = 
+static const char spanish_1[] =
 	MK("Caracas")
 	MK("Maracay")
 	MK("Maracaibo")
@@ -649,14 +659,14 @@ static const char spanish_1[] =
 	MK("Mendoza")
 	MK("Santa Rosa");
 
-static byte MakeSpanishCityName(byte *buf, uint32 seed)
+static byte MakeSpanishTownName(byte *buf, uint32 seed)
 {
 	AppendPart(&buf, GETNUM(0, NUM_SPANISH_1), spanish_1);
-	return 0;	
+	return 0;
 }
 
 #define NUM_FRENCH_1 70
-static const char french_1[] = 
+static const char french_1[] =
 	MK("Agincourt")
 	MK("Lille")
 	MK("Dinan")
@@ -674,7 +684,7 @@ static const char french_1[] =
 	MK("St. Tropez")
 	MK("Marseilles")
 	MK("Narbonne")
-	MK("Ste") 
+	MK("Ste")
 	MK("Aurillac")
 	MK("Gueret")
 	MK("Le Creusot")
@@ -728,20 +738,20 @@ static const char french_1[] =
 	MK("Angoulme")
 	MK("Alenon");
 
-static byte MakeFrenchCityName(byte *buf, uint32 seed)
+static byte MakeFrenchTownName(byte *buf, uint32 seed)
 {
 	AppendPart(&buf, GETNUM(0, NUM_FRENCH_1), french_1);
 	return 0;
 }
 
-static byte MakeAmericanCityName(byte *buf, uint32 seed)
+static byte MakeAmericanTownName(byte *buf, uint32 seed)
 {
-	// make american city names equal to english for now.
-	return MakeEnglishCityName(buf, seed);
+	// make american town names equal to english for now.
+	return MakeEnglishTownName(buf, seed);
 }
 
 #define NUM_SILLY_1 88
-static const char silly_1[] = 
+static const char silly_1[] =
 	MK("Binky")
 	MK("Blubber")
 	MK("Bumble")
@@ -802,7 +812,7 @@ static const char silly_1[] =
 	MK("Scramble")
 	MK("Silly")
 	MK("Simple")
-	MK("Tricky")
+	MK("Trickle")
 	MK("Slippery")
 	MK("Slimey")
 	MK("Slumber")
@@ -841,7 +851,7 @@ static const char silly_1[] =
 ;
 
 #define NUM_SILLY_2 15
-static const char silly_2[] = 
+static const char silly_2[] =
 	MK("ton")
 	MK("bury")
 	MK("bottom")
@@ -852,7 +862,7 @@ static const char silly_2[] =
 	MK("wig")
 	MK("wick")
 	MK("wood")
-	
+
 	MK("pool")
 	MK("head")
 	MK("burg")
@@ -861,8 +871,8 @@ static const char silly_2[] =
 ;
 
 
-static byte MakeSillyCityName(byte *buf, uint32 seed)
-{		
+static byte MakeSillyTownName(byte *buf, uint32 seed)
+{
 	AppendPart(&buf, GETNUM(0, NUM_SILLY_1), silly_1);
 	AppendPart(&buf, GETNUM(16, NUM_SILLY_2),silly_2);
 	return 0;
@@ -1038,7 +1048,7 @@ static const char swedish_3[] =
 	MK("lv")
 	MK("s");
 
-static byte MakeSwedishCityName(byte *buf, uint32 seed)
+static byte MakeSwedishTownName(byte *buf, uint32 seed)
 {
 	int i;
 
@@ -1071,7 +1081,7 @@ static const char dutch_1[] =
 	MK("Klein ");
 
 #define NUM_DUTCH_2 57
-static const char dutch_2[] = 
+static const char dutch_2[] =
 	MK("Hoog")
 	MK("Laag")
 	MK("Klein")
@@ -1131,7 +1141,7 @@ static const char dutch_2[] =
 	MK("Hil");
 
 #define NUM_DUTCH_3 20
-static const char dutch_3[] = 
+static const char dutch_3[] =
 	MK("Drog")
 	MK("Nat")
 	MK("Valk")
@@ -1152,10 +1162,10 @@ static const char dutch_3[] =
 	MK("Wams")
 	MK("Heerhug")
 	MK("Koning");
-	
+
 
 #define NUM_DUTCH_4 6
-static const char dutch_4[] = 
+static const char dutch_4[] =
 	MK("e")
 	MK("er")
 	MK("el")
@@ -1164,7 +1174,7 @@ static const char dutch_4[] =
 	MK("s");
 
 #define NUM_DUTCH_5 56
-static const char dutch_5[] = 
+static const char dutch_5[] =
 	MK("stad")
 	MK("vorst")
 	MK("dorp")
@@ -1221,13 +1231,13 @@ static const char dutch_5[] =
 	MK("schede")
 	MK("mere")
 	MK("end");
-	
-static byte MakeDutchCityName(byte *buf, uint32 seed)
+
+static byte MakeDutchTownName(byte *buf, uint32 seed)
 {
 	int i;
 
 	i = GETNUM(0, 50 + NUM_DUTCH_1) - 50;
-	if (i >= 0) 
+	if (i >= 0)
 		AppendPart(&buf, i, dutch_1);
 
 	i = GETNUM(6, 9);
@@ -1243,7 +1253,7 @@ static byte MakeDutchCityName(byte *buf, uint32 seed)
 }
 
 #define NUM_FINNISH_1 25
-static const char finnish_1[] = 
+static const char finnish_1[] =
 	MK("Aijala")
 	MK("Kisko")
 	MK("Espoo")
@@ -1269,9 +1279,9 @@ static const char finnish_1[] =
 	MK("Imatra")
 	MK("Tapanila")
 	MK("Pasila");
- 
+
 #define NUM_FINNISH_2a 26
-static const char finnish_2a[] = 
+static const char finnish_2a[] =
 	MK("Hiekka")
 	MK("Haapa")
 	MK("Mylly")
@@ -1300,7 +1310,7 @@ static const char finnish_2a[] =
 	MK("Koivu");
 
 #define NUM_FINNISH_2b 18
-static const char finnish_2b[] = 
+static const char finnish_2b[] =
 	MK("harju")
 	MK("linna")
 	MK("jrvi")
@@ -1320,9 +1330,9 @@ static const char finnish_2b[] =
 	MK("ranta")
 	MK("koski");
 
-static byte MakeFinnishCityName(byte *buf, uint32 seed)
+static byte MakeFinnishTownName(byte *buf, uint32 seed)
 {
-// Select randomly if city name should consists of one or two parts.
+// Select randomly if town name should consists of one or two parts.
 	if (GETNUM(0, 15) >= 10)
 		AppendPart(&buf, GETNUM(2, NUM_FINNISH_1), finnish_1); // One part
 	else {
@@ -1566,7 +1576,7 @@ MK(" Malopolski")
 MK(" Podlaski")
 MK(" Lesny")
 ;
-static const char polish_3_f[] = 
+static const char polish_3_f[] =
 MK(" Wybudowanie")
 MK(" Swietokrzyska")
 MK(" Gorska")
@@ -1637,12 +1647,12 @@ static const char * const _polish_types[3][2] = {
 	{polish_1_n, polish_3_n}, // neutral
 };
 
-static byte MakePolishCityName(byte *buf, uint32 seed)
+static byte MakePolishTownName(byte *buf, uint32 seed)
 {
 	uint i, x;
 	const char *const (*t)[2];
 
-	// get a number ranging from 0 to all_cities
+	// get a number ranging from 0 to all_towns
 	i = GETNUM(0, NUM_POLISH_2);
 
 	// single name
@@ -1681,6 +1691,202 @@ static byte MakePolishCityName(byte *buf, uint32 seed)
 	return 0;
 }
 
+#define NUM_CZECH_1 82
+static const char czech_1[] =
+MK("As")
+MK("Benesov")
+MK("Beroun")
+MK("Bezdruzice")
+MK("Blansko")
+MK("Breclav")
+MK("Brno")
+MK("Bruntl")
+MK("Cesk Lpa")
+MK("Cesk Budejovice")
+MK("Cesk Krumlov")
+MK("Decn")
+MK("Domazlice")
+MK("Dub")
+MK("Frdek-Mstek")
+MK("Havlckuv Brod")
+MK("Hodonn")
+MK("Hradec Krlov")
+MK("Humpolec")
+MK("Cheb")
+MK("Chomutov")
+MK("Chrudim")
+MK("Jablonec nad Nisou")
+MK("Jesenk")
+MK("Jicn")
+MK("Jihlava")
+MK("Jindrichuv Hradec")
+MK("Karlovy Vary")
+MK("Karvin")
+MK("Kladno")
+MK("Klatovy")
+MK("Koln")
+MK("Kosmonosy")
+MK("Kromerz")
+MK("Kutn Hora")
+MK("Liberec")
+MK("Litomerice")
+MK("Louny")
+MK("Manetn")
+MK("Melnk")
+MK("Mlad Boleslav")
+MK("Most")
+MK("Nchod")
+MK("Nov Jicn")
+MK("Nymburk")
+MK("Olomouc")
+MK("Opava")
+MK("Orcov")
+MK("Ostrava")
+MK("Pardubice")
+MK("Pelhrimov")
+MK("Polzice")
+MK("Psek")
+MK("Plzen")
+MK("Praha")
+MK("Prachatice")
+MK("Prerov")
+MK("Prbram")
+MK("Prostejov")
+MK("Rakovnk")
+MK("Rokycany")
+MK("Rudn")
+MK("Rychnov nad Kneznou")
+MK("Semily")
+MK("Sokolov")
+MK("Strakonice")
+MK("Stredokluky")
+MK("Sumperk")
+MK("Svitavy")
+MK("Tbor")
+MK("Tachov")
+MK("Teplice")
+MK("Trebc")
+MK("Trutnov")
+MK("Uhersk Hradiste")
+MK("st nad Labem")
+MK("st nad Orlic")
+MK("Vsetn")
+MK("Vyskov")
+MK("Zdr nad Szavou")
+MK("Zln")
+MK("Znojmo")
+;
+
+static byte MakeCzechTownName(byte *buf, uint32 seed)
+{
+	AppendPart(&buf, GETNUM(0, NUM_CZECH_1), czech_1);
+	return 0;
+}
+
+#define NUM_ROMANIAN_1 94
+static const char romanian_1[]=
+MK("Adjud")
+MK("Alba Iulia")
+MK("Alexandria")
+MK("Babadag")
+MK("Bacu")
+MK("Baia Mare")
+MK("Bile Herculane")
+MK("Bilesti")
+MK("Brlad")
+MK("Bicaz")
+MK("Bistrita")
+MK("Blaj")
+MK("Borsec")
+MK("Botosani")
+MK("Brila")
+MK("Brasov")
+MK("Bucuresti")
+MK("Buftea")
+MK("Buzu")
+MK("Clrasi")
+MK("Caransebes")
+MK("Cernavod")
+MK("Cluj-Napoca")
+MK("Constanta")
+MK("Covasna")
+MK("Craiova")
+MK("Dej")
+MK("Deva")
+MK("Dorohoi")
+MK("Dr.-Tr. Severin")
+MK("Drgsani")
+MK("Fgras")
+MK("Flticeni")
+MK("Fetesti")
+MK("Focsani")
+MK("Galati")
+MK("Gheorgheni")
+MK("Giurgiu")
+MK("Hrsova")
+MK("Hunedoara")
+MK("Husi")
+MK("Iasi")
+MK("Isaccea")
+MK("Lugoj")
+MK("Mcin")
+MK("Mangalia")
+MK("Medgidia")
+MK("Medias")
+MK("Miercurea Ciuc")
+MK("Mizil")
+MK("Motru")
+MK("Nsud")
+MK("Nvodari")
+MK("Odobesti")
+MK("Oltenita")
+MK("Onesti")
+MK("Oradea")
+MK("Orsova")
+MK("Petrosani")
+MK("Piatra Neamt")
+MK("Pitesti")
+MK("Ploiesti")
+MK("Predeal")
+MK("Rmnicu Vlcea")
+MK("Reghin")
+MK("Resita")
+MK("Roman")
+MK("Rosiorii de Vede")
+MK("Satu Mare")
+MK("Sebes")
+MK("Sfntu Gheorghe")
+MK("Sibiu")
+MK("Sighisoara")
+MK("Sinaia")
+MK("Slatina")
+MK("Slobozia")
+MK("Sovata")
+MK("Suceava")
+MK("Sulina")
+MK("Tndrei")
+MK("Trgoviste")
+MK("Trgu Jiu")
+MK("Trgu Mures")
+MK("Tecuci")
+MK("Timisoara")
+MK("Tulcea")
+MK("Turda")
+MK("Turnu Mgurele")
+MK("Urziceni")
+MK("Vaslui")
+MK("Vatra Dornei")
+MK("Victoria")
+MK("Videle")
+MK("Zalu")
+;
+
+static byte MakeRomanianTownName(byte *buf, uint32 seed)
+{
+	AppendPart(&buf, GETNUM(0, NUM_ROMANIAN_1), romanian_1);
+	return 0;	
+}
+
 #define NUM_SLOVAKISH_1 87
 static const char slovakish_1[] =
 MK("Bratislava")
@@ -1772,15 +1978,15 @@ MK("Zlate Moravce")
 MK("Zvolen")
 ;
 
-static byte MakeSlovakishCityName(byte *buf, uint32 seed)
+static byte MakeSlovakishTownName(byte *buf, uint32 seed)
 {
 	AppendPart(&buf, GETNUM(0, NUM_SLOVAKISH_1), slovakish_1);
-	return 0;	
+	return 0;
 }
 
 // Modifiers
 #define NUM_HUNGARIAN_1 5
-static const char hungarian_1[] = 
+static const char hungarian_1[] =
 	MK("Nagy-")
 	MK("Kis-")
 	MK("Fels-")
@@ -1789,7 +1995,7 @@ static const char hungarian_1[] =
 ;
 
 #define NUM_HUNGARIAN_2 54
-static const char hungarian_2[] = 
+static const char hungarian_2[] =
 // River modifiers
 // 1 - 10
 	MK("Bodrog")
@@ -1867,7 +2073,7 @@ static const char hungarian_2[] =
 ;
 
 #define NUM_HUNGARIAN_3 16
-static const char hungarian_3[] = 
+static const char hungarian_3[] =
 	MK("apti")
 	MK("bba")
 	MK("bikk")
@@ -1887,7 +2093,7 @@ static const char hungarian_3[] =
 ;
 
 #define NUM_HUNGARIAN_4 5
-static const char hungarian_4[] = 
+static const char hungarian_4[] =
 	MK("alja")
 	MK("egyhza")
 	MK("hza")
@@ -1935,7 +2141,7 @@ static const char hungarian_real[] =
 	MK("Zirc")
 ;
 
-static byte MakeHungarianCityName(byte *buf, uint32 seed)
+static byte MakeHungarianTownName(byte *buf, uint32 seed)
 {
 	int i;
 
@@ -1950,25 +2156,54 @@ static byte MakeHungarianCityName(byte *buf, uint32 seed)
 
 		AppendPart(&buf, GETNUM(3, NUM_HUNGARIAN_2), hungarian_2);
 		AppendPart(&buf, GETNUM(6, NUM_HUNGARIAN_3), hungarian_3);
-		
+
 		i = GETNUM(10, NUM_HUNGARIAN_4 * 3);
 		if (i < NUM_HUNGARIAN_4) AppendPart(&buf, i, hungarian_4);
 	}
 	return 0;
 }
 
-CityNameGenerator * const _city_name_generators[] = {
-	MakeEnglishCityName,
-	MakeFrenchCityName,
-	MakeGermanCityName,
-	MakeAmericanCityName,
-	MakeSpanishCityName,
-	MakeSillyCityName,
-	MakeSwedishCityName,
-	MakeDutchCityName,
-	MakeFinnishCityName,
-	MakePolishCityName,
-	MakeSlovakishCityName,
-	MakeHungarianCityName,
-	MakeAustrianCityName
+TownNameGenerator * const _town_name_generators[] = {
+	MakeEnglishTownName,
+	MakeFrenchTownName,
+	MakeGermanTownName,
+	MakeAmericanTownName,
+	MakeSpanishTownName,
+	MakeSillyTownName,
+	MakeSwedishTownName,
+	MakeDutchTownName,
+	MakeFinnishTownName,
+	MakePolishTownName,
+	MakeCzechTownName,
+	MakeSlovakishTownName,
+	MakeHungarianTownName,
+	MakeAustrianTownName,
+	MakeRomanianTownName
 };
+
+#define FIXNUM(x, y, z) (((((x) << 16) / (y)) + 1) << z)
+
+uint32 GetOldTownName(uint32 townnameparts, byte old_town_name_type)
+{
+	switch (old_town_name_type) {
+		case 0: case 3: /* English, American */
+			/*	Already OK */
+			return townnameparts;
+		case 1: /* French */
+			/*	For some reason 86 needs to be subtracted from townnameparts
+			 *	0000 0000 0000 0000 0000 0000 1111 1111 */
+			return FIXNUM(townnameparts - 86, NUM_FRENCH_1, 0);
+		case 2: /* German */
+			DEBUG(misc, 0) ("German Townnames are buggy... (%d)", townnameparts);
+			return townnameparts;
+		case 4: /* Latin-American */
+			/*	0000 0000 0000 0000 0000 0000 1111 1111 */
+			return FIXNUM(townnameparts, NUM_SPANISH_1, 0);
+		case 5: /* Silly */
+			/*	NUM_SILLY_1	-	lower 16 bits
+			 *	NUM_SILLY_2	-	upper 16 bits without leading 1 (first 8 bytes)
+			 *	1000 0000 2222 2222 0000 0000 1111 1111 */
+			return FIXNUM(townnameparts, NUM_SILLY_1, 0) | FIXNUM(((townnameparts >> 16)&0xFF), NUM_SILLY_2, 16);
+	}
+	return 0;
+}
