@@ -34,8 +34,8 @@ static void CcPlaySound1E(bool success, uint tile, uint32 p1, uint32 p2)
 static void GenericPlaceRail(uint tile, int cmd)
 {
 	DoCommandP(tile, _cur_railtype, cmd, CcPlaySound1E,
-		_remove_button_clicked ? 
-		CMD_REMOVE_SINGLE_RAIL | CMD_MSG(STR_1012_CAN_T_REMOVE_RAILROAD_TRACK) | CMD_AUTO | CMD_NO_WATER : 
+		_remove_button_clicked ?
+		CMD_REMOVE_SINGLE_RAIL | CMD_MSG(STR_1012_CAN_T_REMOVE_RAILROAD_TRACK) | CMD_AUTO | CMD_NO_WATER :
 		CMD_BUILD_SINGLE_RAIL | CMD_MSG(STR_1011_CAN_T_BUILD_RAILROAD_TRACK) | CMD_AUTO | CMD_NO_WATER
 		);
 }
@@ -76,7 +76,7 @@ static void PlaceExtraDepotRail(uint tile, uint16 extra)
 	if (b & 0xC0 || !(b & (extra >> 8)))
 		return;
 
-	DoCommandP(tile, _cur_railtype, extra & 0xFF, NULL, CMD_BUILD_SINGLE_RAIL | CMD_AUTO | CMD_NO_WATER); 
+	DoCommandP(tile, _cur_railtype, extra & 0xFF, NULL, CMD_BUILD_SINGLE_RAIL | CMD_AUTO | CMD_NO_WATER);
 }
 
 static const uint16 _place_depot_extra[12] = {
@@ -101,12 +101,12 @@ static void CcDepot(bool success, uint tile, uint32 p1, uint32 p2)
 			PlaceExtraDepotRail(tile, _place_depot_extra[dir + 4]);
 			PlaceExtraDepotRail(tile, _place_depot_extra[dir + 8]);
 		}
-	}	
+	}
 }
 
 static void PlaceRail_Depot(uint tile)
 {
-	DoCommandP(tile, _cur_railtype, _build_depot_direction, CcDepot, 
+	DoCommandP(tile, _cur_railtype, _build_depot_direction, CcDepot,
 		CMD_BUILD_TRAIN_DEPOT | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_100E_CAN_T_BUILD_TRAIN_DEPOT));
 }
 
@@ -140,12 +140,12 @@ static void PlaceRail_Station(uint tile)
 	}
 }
 
-static void PlaceRail_Signals(uint tile)
+static void GenericPlaceSignals(uint tile)
 {
 	uint trackstat;
 	int i;
 
-	trackstat = (byte)GetTileTrackStatus(tile,0);
+	trackstat = (byte)GetTileTrackStatus(tile, TRANSPORT_RAIL);
 
 	if ((trackstat & 0x30) == 0x30) {
 		trackstat = (_tile_fract_coords.x <= _tile_fract_coords.y) ? 0x20 : 0x10;
@@ -194,16 +194,16 @@ void PlaceProc_BuyLand(uint tile)
 	DoCommandP(tile, 0, 0, CcPlaySound1E, CMD_PURCHASE_LAND_AREA | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_5806_CAN_T_PURCHASE_THIS_LAND));
 }
 
-void PlaceProc_DemolishArea(uint tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y);
-}
-
 static void PlaceRail_ConvertRail(uint tile)
 {
 	VpStartPlaceSizing(tile, VPM_X_AND_Y | (1<<4));
 }
 
+static void PlaceRail_AutoSignals(uint tile)
+{
+	VpStartPlaceSizing(tile, VPM_SIGNALDIRS);
+}
+
 static void BuildRailClick_AutoRail(Window *w)
 {
 	HandlePlacePushButton(w, 3, _cur_railtype + SPR_OPENTTD_BASE + 4, 1, PlaceRail_AutoRail);
@@ -239,19 +239,9 @@ static void BuildRailClick_Lower(Window *w)
 	HandlePlacePushButton(w, 9, ANIMCURSOR_LOWERLAND, 2, PlaceProc_LowerLand);
 }
 
-
-void PlaceProc_LevelLand(uint tile)
-{
-	VpStartPlaceSizing(tile, VPM_X_AND_Y | (2<<4));
-}
-
 static void BuildRailClick_Raise(Window *w)
 {
-	if (_ctrl_pressed) {
-		HandlePlacePushButton(w, 10, ANIMCURSOR_PICKSTATION, 2, PlaceProc_LevelLand);
-	} else {
-		HandlePlacePushButton(w, 10, ANIMCURSOR_RAISELAND, 2, PlaceProc_RaiseLand);
-	}
+	HandlePlacePushButton(w, 10, ANIMCURSOR_RAISELAND, 2, PlaceProc_RaiseLand);
 }
 
 static const SpriteID _depot_cursors[] = {
@@ -262,17 +252,17 @@ static const SpriteID _depot_cursors[] = {
 
 static void BuildRailClick_Depot(Window *w)
 {
-	if (HandlePlacePushButton(w, 11, _depot_cursors[_cur_railtype], 1, PlaceRail_Depot)) ShowBuildTrainDepotPicker();	
+	if (HandlePlacePushButton(w, 11, _depot_cursors[_cur_railtype], 1, PlaceRail_Depot)) ShowBuildTrainDepotPicker();
 }
 
 static void BuildRailClick_Station(Window *w)
 {
-	if (HandlePlacePushButton(w, 12, 0x514, 1, PlaceRail_Station)) ShowStationBuilder();	
+	if (HandlePlacePushButton(w, 12, 0x514, 1, PlaceRail_Station)) ShowStationBuilder();
 }
 
-static void BuildRailClick_Signals(Window *w)
+static void BuildRailClick_AutoSignals(Window *w)
 {
-	HandlePlacePushButton(w, 13, ANIMCURSOR_BUILDSIGNALS, 1, PlaceRail_Signals);
+	HandlePlacePushButton(w, 13, ANIMCURSOR_BUILDSIGNALS , 1, PlaceRail_AutoSignals);
 }
 
 static void BuildRailClick_Bridge(Window *w)
@@ -291,10 +281,10 @@ static void BuildRailClick_Remove(Window *w)
 		return;
 	SetWindowDirty(w);
 	SndPlayFx(0x13);
-	
+
 	_thd.make_square_red = !!((w->click_state ^= (1 << 16)) & (1<<16));
 	_remove_button_clicked = (w->click_state & (1 << 16)) != 0;
-	
+
 	// handle station builder
 	if( w->click_state & (1 << 12) )
 	{
@@ -323,8 +313,8 @@ static void BuildRailClick_Convert(Window *w)
 
 static void DoRailroadTrack(int mode)
 {
-	DoCommandP(TILE_FROM_XY(_thd.selstart.x, _thd.selstart.y), PACK_POINT(_thd.selend.x, _thd.selend.y), (mode << 4) | _cur_railtype, NULL, 
-		_remove_button_clicked ? 
+	DoCommandP(TILE_FROM_XY(_thd.selstart.x, _thd.selstart.y), PACK_POINT(_thd.selend.x, _thd.selend.y), (mode << 4) | _cur_railtype, NULL,
+		_remove_button_clicked ?
 		CMD_REMOVE_RAILROAD_TRACK | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_1012_CAN_T_REMOVE_RAILROAD_TRACK) :
 		CMD_BUILD_RAILROAD_TRACK | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_1011_CAN_T_BUILD_RAILROAD_TRACK)
 	);
@@ -345,11 +335,11 @@ static const BestFitStruct _bestfit[] = {
 
 	{0, M(0,1+8+16), M(2,1+4+32), 0}, // diag1 track
 	{1, M(3,2+8+32), M(1,2+4+16), 0}, // diag2 track
-	
+
 	// one edge with rail
 	{0, M(0,1), 0, 0}, // diag1 track
 	{0, M(2,1), 0, 0}, // diag1 track
-	
+
 	{1, M(1,2), 0, 0}, // diag2 track
 	{1, M(3,2), 0, 0}, // diag2 track
 
@@ -365,7 +355,7 @@ static const BestFitStruct _bestfit[] = {
 	{5, M(0,16), 0, 1<<3}, // right track
 	{5, M(1,16), 0, 1<<3}, // right track
 
-	{0xff,0,0},
+	{0xff, 0, 0, 0},
 };
 #undef M
 
@@ -404,7 +394,7 @@ int GetBestFit1x1(int x, int y)
 		};
 		tile += _tile_inc[i];
 		m[i] = 0;
-		if (IS_TILETYPE(tile, MP_RAILWAY) && _map5[tile] < 0x80) 
+		if (IS_TILETYPE(tile, MP_RAILWAY) && _map5[tile] < 0x80)
 			m[i] = _map5[tile]&0x3F;
 
 		// handle tracks under bridge
@@ -459,7 +449,7 @@ static bool Check2x1AutoRail(int mode)
 	int sxpy = (thd->selend.x & 0xF) + (thd->selend.y & 0xF);
 	int fxmy = _tile_fract_coords.x - _tile_fract_coords.y;
 	int sxmy = (thd->selend.x & 0xF) - (thd->selend.y & 0xF);
-	
+
 	switch(mode) {
 	case 0:
 		if (fxpy >= 20 && sxpy <= 12) { SwapSelection(); DoRailroadTrack(0); return true; }
@@ -469,7 +459,7 @@ static bool Check2x1AutoRail(int mode)
 	case 1:
 		if (fxmy > 3 && sxmy < -3) { SwapSelection(); DoRailroadTrack(0); return true; }
 		if (fxpy <= 12 && sxpy >= 20) { DoRailroadTrack(0); return true; }
-		break;	
+		break;
 
 	case 2:
 		if (fxmy > 3 && sxmy < -3) { DoRailroadTrack(3); return true; }
@@ -494,7 +484,7 @@ static void HandleAutodirPlacement()
 	if (thd->drawstyle == HT_RECT) {
 		int dx = thd->selstart.x - (thd->selend.x&~0xF);
 		int dy = thd->selstart.y - (thd->selend.y&~0xF);
-		
+
 		if (dx == 0 && dy == 0 ) {
 			// 1x1 tile
 			bit = GetBestFit1x1(thd->selend.x, thd->selend.y);
@@ -523,6 +513,44 @@ static void HandleAutodirPlacement()
 	}
 }
 
+static void HandleAutoSignalPlacement()
+{
+	TileHighlightData *thd = &_thd;
+	int mode = 0;
+	uint trackstat = 0;
+
+	int dx = thd->selstart.x - (thd->selend.x&~0xF);
+	int dy = thd->selstart.y - (thd->selend.y&~0xF);
+
+	if (dx == 0 && dy == 0 ) // 1x1 tile signals
+		GenericPlaceSignals(TILE_FROM_XY(thd->selend.x, thd->selend.y));
+	else { // signals have been dragged
+		if (thd->drawstyle == HT_RECT) { // X,Y direction
+			if (dx == 0)
+				mode = VPM_FIX_X;
+			else if (dy == 0)
+				mode = VPM_FIX_Y;
+
+			trackstat = 0xC0;
+		}	else { // W-E or N-S direction
+			mode = thd->drawstyle & 1 ? 0 : 3;
+
+			if (dx == dy || abs(dx - dy) == 16) // North<->South track |
+				trackstat = (thd->drawstyle & 1) ? 0x20 : 0x10;
+			else if (dx == -dy || abs(dx + dy) == 16) // East<->West track --
+				trackstat = (thd->drawstyle & 1) ? 4 : 8;
+		}
+
+		/* _patches.drag_signals_density is given as a parameter such that each user in a network
+		 * game can specify his/her own signal density */
+		DoCommandP(TILE_FROM_XY(thd->selstart.x, thd->selstart.y), TILE_FROM_XY(thd->selend.x, thd->selend.y),
+		(mode << 4) | (_remove_button_clicked + (_ctrl_pressed ? 8 : 0)) | (trackstat << 8) | (_patches.drag_signals_density << 24),
+		CcPlaySound1E,
+		(_remove_button_clicked ?	CMD_BUILD_MANY_SIGNALS | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_1013_CAN_T_REMOVE_SIGNALS_FROM) :
+															CMD_BUILD_MANY_SIGNALS | CMD_AUTO | CMD_NO_WATER | CMD_MSG(STR_1010_CAN_T_BUILD_SIGNALS_HERE) ) );
+	}
+}
+
 static OnButtonClick * const _build_railroad_button_proc[] = {
 	BuildRailClick_AutoRail,
 	BuildRailClick_N,
@@ -534,7 +562,7 @@ static OnButtonClick * const _build_railroad_button_proc[] = {
 	BuildRailClick_Raise,
 	BuildRailClick_Depot,
 	BuildRailClick_Station,
-	BuildRailClick_Signals,
+	BuildRailClick_AutoSignals,
 	BuildRailClick_Bridge,
 	BuildRailClick_Tunnel,
 	BuildRailClick_Remove,
@@ -576,6 +604,7 @@ static void BuildRailToolbWndProc(Window *w, WindowEvent *e)
 		}
 		DrawWindowWidgets(w);
 		break;
+
 	case WE_CLICK:
 		if (e->click.widget >= 3) {
 			_remove_button_clicked = false;
@@ -617,6 +646,8 @@ static void BuildRailToolbWndProc(Window *w, WindowEvent *e)
 				if (_ctrl_pressed) _remove_button_clicked = true;
 				HandleAutodirPlacement();
 				_remove_button_clicked = old;
+			} else if (e->place.userdata == VPM_SIGNALDIRS) {
+				HandleAutoSignalPlacement();
 			} else if (e->place.userdata == VPM_X_AND_Y) {
 				DoCommandP(end_tile, start_tile, 0, CcPlaySound10, CMD_CLEAR_AREA | CMD_MSG(STR_00B5_CAN_T_CLEAR_THIS_AREA));
 			} else if (e->place.userdata == (VPM_X_AND_Y | (1<<4))) {
@@ -650,33 +681,33 @@ static void BuildRailToolbWndProc(Window *w, WindowEvent *e)
 }
 
 static const Widget _build_railroad_widgets[] = {
-{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
+{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,     7,    11,   417,     0,    13, STR_100A_RAILROAD_CONSTRUCTION, STR_018C_WINDOW_TITLE_DRAG_THIS},
 
-{      WWT_PANEL,     7,    110,  113,    14,    35, 0x0, 0x0},
+{      WWT_PANEL,     7,    110,  113,    14,    35, 0x0,			STR_NULL},
 {      WWT_PANEL,     7,    88,   109,    14,    35, SPR_OPENTTD_BASE + 0, STR_BUILD_AUTORAIL_TIP},
 
-{      WWT_PANEL,     7,    0,     21,    14,    35, 0x4E3, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    22,    43,    14,    35, 0x4E4, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    44,    65,    14,    35, 0x4E5, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    66,    87,    14,    35, 0x4E6, STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    0,     21,    14,    35, 0x4E3,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    22,    43,    14,    35, 0x4E4,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    44,    65,    14,    35, 0x4E5,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    66,    87,    14,    35, 0x4E6,		STR_1018_BUILD_RAILROAD_TRACK},
 
-{      WWT_PANEL,     7,   114,   135,    14,    35, 0x2BF, STR_018D_DEMOLISH_BUILDINGS_ETC},
-{      WWT_PANEL,     7,   136,   157,    14,    35, 0x2B7, STR_018E_LOWER_A_CORNER_OF_LAND},
-{      WWT_PANEL,     7,   158,   179,    14,    35, 0x2B6, STR_018F_RAISE_A_CORNER_OF_LAND},
-{      WWT_PANEL,     7,   180,   201,    14,    35, 0x50E, STR_1019_BUILD_TRAIN_DEPOT_FOR_BUILDING},
+{      WWT_PANEL,     7,   114,   135,    14,    35, 0x2BF,		STR_018D_DEMOLISH_BUILDINGS_ETC},
+{      WWT_PANEL,     7,   136,   157,    14,    35, 0x2B7,		STR_018E_LOWER_A_CORNER_OF_LAND},
+{      WWT_PANEL,     7,   158,   179,    14,    35, 0x2B6,		STR_018F_RAISE_A_CORNER_OF_LAND},
+{      WWT_PANEL,     7,   180,   201,    14,    35, 0x50E,		STR_1019_BUILD_TRAIN_DEPOT_FOR_BUILDING},
 
-{      WWT_PANEL,     7,   224,   264,    14,    35, 0x512, STR_101A_BUILD_RAILROAD_STATION},
-{      WWT_PANEL,     7,   266,   287,    14,    35, 0x50B, STR_101B_BUILD_RAILROAD_SIGNALS},
-{      WWT_PANEL,     7,   288,   329,    14,    35, 0xA22, STR_101C_BUILD_RAILROAD_BRIDGE},
-{      WWT_PANEL,     7,   330,   351,    14,    35, 0x97E, STR_101D_BUILD_RAILROAD_TUNNEL},
-{      WWT_PANEL,     7,   352,   373,    14,    35, 0x2CA, STR_101E_TOGGLE_BUILD_REMOVE_FOR},
-{      WWT_PANEL,     7,   374,   395,    14,    35, 0x12B7, STR_0329_PURCHASE_LAND_FOR_FUTURE},
+{      WWT_PANEL,     7,   224,   265,    14,    35, 0x512,		STR_101A_BUILD_RAILROAD_STATION},
+{      WWT_PANEL,     7,   266,   287,    14,    35, 0x50B,		STR_101B_BUILD_RAILROAD_SIGNALS},
+{      WWT_PANEL,     7,   288,   329,    14,    35, 0xA22,		STR_101C_BUILD_RAILROAD_BRIDGE},
+{      WWT_PANEL,     7,   330,   351,    14,    35, 0x97E,		STR_101D_BUILD_RAILROAD_TUNNEL},
+{      WWT_PANEL,     7,   352,   373,    14,    35, 0x2CA,		STR_101E_TOGGLE_BUILD_REMOVE_FOR},
+{      WWT_PANEL,     7,   374,   395,    14,    35, 0x12B7,	STR_0329_PURCHASE_LAND_FOR_FUTURE},
 
 {      WWT_PANEL,     7,   202,   223,    14,    35, SPR_OPENTTD_BASE + 3, STR_CONVERT_RAIL_TO_CHECKPOINT_TIP},
 {      WWT_PANEL,     7,   396,   417,    14,    35, SPR_OPENTTD_BASE + 25, STR_CONVERT_RAIL_TIP},
 
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _build_railroad_desc = {
@@ -688,32 +719,32 @@ static const WindowDesc _build_railroad_desc = {
 };
 
 static const Widget _build_monorail_widgets[] = {
-{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
+{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,     7,    11,   417,     0,    13, STR_100B_MONORAIL_CONSTRUCTION, STR_018C_WINDOW_TITLE_DRAG_THIS},
 
-{      WWT_PANEL,     7,   110,   113,    14,    35, 0x0, 0x0},
+{      WWT_PANEL,     7,   110,   113,    14,    35, 0x0,			STR_NULL},
 {      WWT_PANEL,     7,    88,   109,    14,    35, SPR_OPENTTD_BASE + 1, STR_BUILD_AUTORAIL_TIP},
 
-{      WWT_PANEL,     7,     0,    21,    14,    35, 0x4E7, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    22,    43,    14,    35, 0x4E8, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    44,    65,    14,    35, 0x4E9, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    66,    87,    14,    35, 0x4EA, STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,     0,    21,    14,    35, 0x4E7,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    22,    43,    14,    35, 0x4E8,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    44,    65,    14,    35, 0x4E9,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    66,    87,    14,    35, 0x4EA,		STR_1018_BUILD_RAILROAD_TRACK},
 
-{      WWT_PANEL,     7,   114,   135,    14,    35, 0x2BF, STR_018D_DEMOLISH_BUILDINGS_ETC},
-{      WWT_PANEL,     7,   136,   157,    14,    35, 0x2B7, STR_018E_LOWER_A_CORNER_OF_LAND},
-{      WWT_PANEL,     7,   158,   179,    14,    35, 0x2B6, STR_018F_RAISE_A_CORNER_OF_LAND},
+{      WWT_PANEL,     7,   114,   135,    14,    35, 0x2BF,		STR_018D_DEMOLISH_BUILDINGS_ETC},
+{      WWT_PANEL,     7,   136,   157,    14,    35, 0x2B7,		STR_018E_LOWER_A_CORNER_OF_LAND},
+{      WWT_PANEL,     7,   158,   179,    14,    35, 0x2B6,		STR_018F_RAISE_A_CORNER_OF_LAND},
 {      WWT_PANEL,     7,   180,   201,    14,    35, SPR_OPENTTD_BASE + 12, STR_1019_BUILD_TRAIN_DEPOT_FOR_BUILDING},
 
-{      WWT_PANEL,     7,   224,   265,    14,    35, 0x512, STR_101A_BUILD_RAILROAD_STATION},
-{      WWT_PANEL,     7,   266,   287,    14,    35, 0x50B, STR_101B_BUILD_RAILROAD_SIGNALS},
-{      WWT_PANEL,     7,   288,   329,    14,    35, 0xA22, STR_101C_BUILD_RAILROAD_BRIDGE},
-{      WWT_PANEL,     7,   330,   351,    14,    35, 0x97F, STR_101D_BUILD_RAILROAD_TUNNEL},
-{      WWT_PANEL,     7,   352,   373,    14,    35, 0x2CA, STR_101E_TOGGLE_BUILD_REMOVE_FOR},
-{      WWT_PANEL,     7,   374,   395,    14,    35, 0x12B7, STR_0329_PURCHASE_LAND_FOR_FUTURE},
+{      WWT_PANEL,     7,   224,   265,    14,    35, 0x512,		STR_101A_BUILD_RAILROAD_STATION},
+{      WWT_PANEL,     7,   266,   287,    14,    35, 0x50B,		STR_101B_BUILD_RAILROAD_SIGNALS},
+{      WWT_PANEL,     7,   288,   329,    14,    35, 0xA22,		STR_101C_BUILD_RAILROAD_BRIDGE},
+{      WWT_PANEL,     7,   330,   351,    14,    35, 0x97F,		STR_101D_BUILD_RAILROAD_TUNNEL},
+{      WWT_PANEL,     7,   352,   373,    14,    35, 0x2CA,		STR_101E_TOGGLE_BUILD_REMOVE_FOR},
+{      WWT_PANEL,     7,   374,   395,    14,    35, 0x12B7,	STR_0329_PURCHASE_LAND_FOR_FUTURE},
 
 {      WWT_PANEL,     7,   202,   223,    14,    35, SPR_OPENTTD_BASE + 3, STR_CONVERT_RAIL_TO_CHECKPOINT_TIP},
 {      WWT_PANEL,     7,   396,   417,    14,    35, SPR_OPENTTD_BASE + 27, STR_CONVERT_RAIL_TIP},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _build_monorail_desc = {
@@ -725,32 +756,32 @@ static const WindowDesc _build_monorail_desc = {
 };
 
 static const Widget _build_maglev_widgets[] = {
-{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
+{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,     7,    11,   417,     0,    13, STR_100C_MAGLEV_CONSTRUCTION, STR_018C_WINDOW_TITLE_DRAG_THIS},
 
-{      WWT_PANEL,     7,   110,   113,    14,    35, 0x0, 0x0},
+{      WWT_PANEL,     7,   110,   113,    14,    35, 0x0,			STR_NULL},
 {      WWT_PANEL,     7,    88,   109,    14,    35, SPR_OPENTTD_BASE + 2, STR_BUILD_AUTORAIL_TIP},
 
-{      WWT_PANEL,     7,     0,    21,    14,    35, 0x4EB, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    22,    43,    14,    35, 0x4EC, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    44,    65,    14,    35, 0x4EE, STR_1018_BUILD_RAILROAD_TRACK},
-{      WWT_PANEL,     7,    66,    87,    14,    35, 0x4ED, STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,     0,    21,    14,    35, 0x4EB,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    22,    43,    14,    35, 0x4EC,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    44,    65,    14,    35, 0x4EE,		STR_1018_BUILD_RAILROAD_TRACK},
+{      WWT_PANEL,     7,    66,    87,    14,    35, 0x4ED,		STR_1018_BUILD_RAILROAD_TRACK},
 
-{      WWT_PANEL,     7,   114,   135,    14,    35, 0x2BF, STR_018D_DEMOLISH_BUILDINGS_ETC},
-{      WWT_PANEL,     7,   136,   157,    14,    35, 0x2B7, STR_018E_LOWER_A_CORNER_OF_LAND},
-{      WWT_PANEL,     7,   158,   179,    14,    35, 0x2B6, STR_018F_RAISE_A_CORNER_OF_LAND},
+{      WWT_PANEL,     7,   114,   135,    14,    35, 0x2BF,		STR_018D_DEMOLISH_BUILDINGS_ETC},
+{      WWT_PANEL,     7,   136,   157,    14,    35, 0x2B7,		STR_018E_LOWER_A_CORNER_OF_LAND},
+{      WWT_PANEL,     7,   158,   179,    14,    35, 0x2B6,		STR_018F_RAISE_A_CORNER_OF_LAND},
 {      WWT_PANEL,     7,   180,   201,    14,    35, SPR_OPENTTD_BASE + 13, STR_1019_BUILD_TRAIN_DEPOT_FOR_BUILDING},
 
-{      WWT_PANEL,     7,   224,   265,    14,    35, 0x512, STR_101A_BUILD_RAILROAD_STATION},
-{      WWT_PANEL,     7,   266,   287,    14,    35, 0x50B, STR_101B_BUILD_RAILROAD_SIGNALS},
-{      WWT_PANEL,     7,   288,   329,    14,    35, 0xA22, STR_101C_BUILD_RAILROAD_BRIDGE},
-{      WWT_PANEL,     7,   330,   351,    14,    35, 0x980, STR_101D_BUILD_RAILROAD_TUNNEL},
-{      WWT_PANEL,     7,   352,   373,    14,    35, 0x2CA, STR_101E_TOGGLE_BUILD_REMOVE_FOR},
-{      WWT_PANEL,     7,   374,   395,    14,    35, 0x12B7, STR_0329_PURCHASE_LAND_FOR_FUTURE},
+{      WWT_PANEL,     7,   224,   265,    14,    35, 0x512,		STR_101A_BUILD_RAILROAD_STATION},
+{      WWT_PANEL,     7,   266,   287,    14,    35, 0x50B,		STR_101B_BUILD_RAILROAD_SIGNALS},
+{      WWT_PANEL,     7,   288,   329,    14,    35, 0xA22,		STR_101C_BUILD_RAILROAD_BRIDGE},
+{      WWT_PANEL,     7,   330,   351,    14,    35, 0x980,		STR_101D_BUILD_RAILROAD_TUNNEL},
+{      WWT_PANEL,     7,   352,   373,    14,    35, 0x2CA,		STR_101E_TOGGLE_BUILD_REMOVE_FOR},
+{      WWT_PANEL,     7,   374,   395,    14,    35, 0x12B7,	STR_0329_PURCHASE_LAND_FOR_FUTURE},
 
 {      WWT_PANEL,     7,   202,   223,    14,    35, SPR_OPENTTD_BASE + 3, STR_CONVERT_RAIL_TO_CHECKPOINT_TIP},
 {      WWT_PANEL,     7,   396,   417,    14,    35, SPR_OPENTTD_BASE + 29, STR_CONVERT_RAIL_TIP},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _build_maglev_desc = {
@@ -774,7 +805,7 @@ void ShowBuildRailToolbar(int index, int button)
 
 	// don't recreate the window if we're clicking on a button and the window exists.
 	if (button < 0 || !(w = FindWindowById(WC_BUILD_TOOLBAR, 0)) || w->wndproc != BuildRailToolbWndProc) {
-		DeleteWindowById(WC_BUILD_TOOLBAR, 0);	
+		DeleteWindowById(WC_BUILD_TOOLBAR, 0);
 		_cur_railtype = (byte)index;
 		w = AllocateWindowDesc(_build_rr_desc[index]);
 	}
@@ -817,7 +848,7 @@ static void StationBuildWndProc(Window *w, WindowEvent *e) {
 		}
 		bits |= (1<<20) << (_station_show_coverage);
 		w->click_state = bits;
-		
+
 		if (_railstation.dragdrop) {
 			SetTileSelectSize(1, 1);
 		} else {
@@ -827,12 +858,12 @@ static void StationBuildWndProc(Window *w, WindowEvent *e) {
 			if(!_remove_button_clicked)
 				SetTileSelectSize(x, y);
 		}
-				
+
 		if (_station_show_coverage)
 			SetTileSelectBigSize(-4, -4, 8, 8);
 
 		DrawWindowWidgets(w);
-		
+
 		StationPickerDrawSprite(39, 42, _cur_railtype, 2);
 		StationPickerDrawSprite(107, 42, _cur_railtype, 3);
 
@@ -843,7 +874,7 @@ static void StationBuildWndProc(Window *w, WindowEvent *e) {
 
 		DrawStationCoverageAreaText(2, 166, (uint)-1);
 	} break;
-		
+
 	case WE_CLICK: {
 		switch(e->click.widget) {
 		case 0:
@@ -896,7 +927,7 @@ static void StationBuildWndProc(Window *w, WindowEvent *e) {
 			break;
 		}
 	} break;
-	
+
 	case WE_MOUSELOOP: {
 		if (WP(w,def_d).close) {
 			DeleteWindow(w);
@@ -908,34 +939,34 @@ static void StationBuildWndProc(Window *w, WindowEvent *e) {
 }
 
 static const Widget _station_builder_widgets[] = {
-{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
+{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,     7,    11,   147,     0,    13, STR_3000_RAIL_STATION_SELECTION, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{      WWT_PANEL,     7,     0,   147,    14,   199, 0x0, 0},
-{      WWT_PANEL,    14,     7,    72,    26,    73, 0x0, STR_304E_SELECT_RAILROAD_STATION},
-{      WWT_PANEL,    14,    75,   140,    26,    73, 0x0, STR_304E_SELECT_RAILROAD_STATION},
+{      WWT_PANEL,     7,     0,   147,    14,   199, 0x0,					STR_NULL},
+{      WWT_PANEL,    14,     7,    72,    26,    73, 0x0,					STR_304E_SELECT_RAILROAD_STATION},
+{      WWT_PANEL,    14,    75,   140,    26,    73, 0x0,					STR_304E_SELECT_RAILROAD_STATION},
 
-{   WWT_CLOSEBOX,    14,    22,    36,    87,    98, STR_00CB_1, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
-{   WWT_CLOSEBOX,    14,    37,    51,    87,    98, STR_00CC_2, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
-{   WWT_CLOSEBOX,    14,    52,    66,    87,    98, STR_00CD_3, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
-{   WWT_CLOSEBOX,    14,    67,    81,    87,    98, STR_00CE_4, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
-{   WWT_CLOSEBOX,    14,    82,    96,    87,    98, STR_00CF_5, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
-{   WWT_CLOSEBOX,    14,    97,   111,    87,    98, STR_0335_6, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
-{   WWT_CLOSEBOX,    14,   112,   126,    87,    98, STR_0336_7, STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,    22,    36,    87,    98, STR_00CB_1,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,    37,    51,    87,    98, STR_00CC_2,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,    52,    66,    87,    98, STR_00CD_3,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,    67,    81,    87,    98, STR_00CE_4,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,    82,    96,    87,    98, STR_00CF_5,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,    97,   111,    87,    98, STR_0335_6,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
+{   WWT_CLOSEBOX,    14,   112,   126,    87,    98, STR_0336_7,	STR_304F_SELECT_NUMBER_OF_PLATFORMS},
 
-{   WWT_CLOSEBOX,    14,    22,    36,   112,   123, STR_00CB_1, STR_3050_SELECT_LENGTH_OF_RAILROAD},
-{   WWT_CLOSEBOX,    14,    37,    51,   112,   123, STR_00CC_2, STR_3050_SELECT_LENGTH_OF_RAILROAD},
-{   WWT_CLOSEBOX,    14,    52,    66,   112,   123, STR_00CD_3, STR_3050_SELECT_LENGTH_OF_RAILROAD},
-{   WWT_CLOSEBOX,    14,    67,    81,   112,   123, STR_00CE_4, STR_3050_SELECT_LENGTH_OF_RAILROAD},
-{   WWT_CLOSEBOX,    14,    82,    96,   112,   123, STR_00CF_5, STR_3050_SELECT_LENGTH_OF_RAILROAD},
-{   WWT_CLOSEBOX,    14,    97,   111,   112,   123, STR_0335_6, STR_3050_SELECT_LENGTH_OF_RAILROAD},
-{   WWT_CLOSEBOX,    14,   112,   126,   112,   123, STR_0336_7, STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,    22,    36,   112,   123, STR_00CB_1,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,    37,    51,   112,   123, STR_00CC_2,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,    52,    66,   112,   123, STR_00CD_3,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,    67,    81,   112,   123, STR_00CE_4,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,    82,    96,   112,   123, STR_00CF_5,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,    97,   111,   112,   123, STR_0335_6,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
+{   WWT_CLOSEBOX,    14,   112,   126,   112,   123, STR_0336_7,	STR_3050_SELECT_LENGTH_OF_RAILROAD},
 
 //{   WWT_CLOSEBOX,    14,    14,    73,   137,   148, STR_02DB_OFF, STR_3065_DON_T_HIGHLIGHT_COVERAGE},
 //{   WWT_CLOSEBOX,    14,    74,   133,   137,   148, STR_02DA_ON, STR_3064_HIGHLIGHT_COVERAGE_AREA},
 {   WWT_CLOSEBOX,    14,    37,   111,   126,   137, STR_DRAG_DROP, STR_STATION_DRAG_DROP},
 {   WWT_CLOSEBOX,    14,    14,    73,   152,   163, STR_02DB_OFF, STR_3065_DON_T_HIGHLIGHT_COVERAGE},
 {   WWT_CLOSEBOX,    14,    74,   133,   152,   163, STR_02DA_ON, STR_3064_HIGHLIGHT_COVERAGE_AREA},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _station_builder_desc = {
@@ -982,7 +1013,7 @@ static void BuildTrainDepotWndProc(Window *w, WindowEvent *e)
 			break;
 		}
 	} break;
-	
+
 	case WE_MOUSELOOP:
 		if (WP(w,def_d).close)
 			DeleteWindow(w);
@@ -991,14 +1022,14 @@ static void BuildTrainDepotWndProc(Window *w, WindowEvent *e)
 }
 
 static const Widget _build_depot_widgets[] = {
-{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,     7,    11,   139,     0,    13, STR_1014_TRAIN_DEPOT_ORIENTATION,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{      WWT_PANEL,     7,     0,   139,    14,   121, 0x0, 0},
-{      WWT_PANEL,    14,    71,   136,    17,    66, 0x0, STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
-{      WWT_PANEL,    14,    71,   136,    69,   118, 0x0, STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
-{      WWT_PANEL,    14,     3,    68,    69,   118, 0x0, STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
-{      WWT_PANEL,    14,     3,    68,    17,    66, 0x0, STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
-{      WWT_LAST},
+{   WWT_CLOSEBOX,     7,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,     7,    11,   139,     0,    13, STR_1014_TRAIN_DEPOT_ORIENTATION, STR_018C_WINDOW_TITLE_DRAG_THIS},
+{      WWT_PANEL,     7,     0,   139,    14,   121, 0x0,			STR_NULL},
+{      WWT_PANEL,    14,    71,   136,    17,    66, 0x0,			STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
+{      WWT_PANEL,    14,    71,   136,    69,   118, 0x0,			STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
+{      WWT_PANEL,    14,     3,    68,    69,   118, 0x0,			STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
+{      WWT_PANEL,    14,     3,    68,    17,    66, 0x0,			STR_1020_SELECT_RAILROAD_DEPOT_ORIENTATIO},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _build_depot_desc = {
@@ -1016,7 +1047,7 @@ static void ShowBuildTrainDepotPicker()
 
 void InitializeRailGui()
 {
-	_build_depot_direction = 3;	
+	_build_depot_direction = 3;
 	_railstation.numtracks = 1;
 	_railstation.platlength = 1;
 	_railstation.dragdrop = true;
