@@ -9,7 +9,6 @@
 #include "station.h"
 #include "command.h"
 #include "player.h"
-//#include "city.h"
 #include "engine.h"
 
 
@@ -27,7 +26,7 @@ static void ShipRefitWndProc(Window *w, WindowEvent *e)
 {
 	switch(e->event) {
 	case WE_PAINT: {
-		Vehicle *v = &_vehicles[w->window_number];	
+		Vehicle *v = &_vehicles[w->window_number];
 		const byte *b;
 		int sel;
 		int x,y;
@@ -39,7 +38,7 @@ static void ShipRefitWndProc(Window *w, WindowEvent *e)
 		DrawWindowWidgets(w);
 
 		DrawString(1, 15, STR_983F_SELECT_CARGO_TYPE_TO_CARRY, 0);
-		
+
 		cargo = -1;
 		x = 6;
 		y = 25;
@@ -104,7 +103,7 @@ static void ShipRefitWndProc(Window *w, WindowEvent *e)
 			} break;
 		case 4: /* refit button */
 			if (WP(w,refit_d).cargo != 0xFF) {
-				Vehicle *v = &_vehicles[w->window_number];	
+				Vehicle *v = &_vehicles[w->window_number];
 				if (DoCommandP(v->tile, v->index, WP(w,refit_d).cargo, NULL, CMD_REFIT_SHIP | CMD_MSG(STR_9841_CAN_T_REFIT_SHIP)))
 					DeleteWindow(w);
 			}
@@ -116,12 +115,12 @@ static void ShipRefitWndProc(Window *w, WindowEvent *e)
 
 
 static const Widget _ship_refit_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   239,     0,    13, STR_983B_REFIT, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    14,     0,   239,    14,   135, 0x0, STR_983D_SELECT_TYPE_OF_CARGO_FOR},
-{     WWT_IMGBTN,    14,     0,   239,   136,   157, 0x0},
-{ WWT_PUSHTXTBTN,    14,     0,   239,   158,   169, STR_983C_REFIT_SHIP, STR_983E_REFIT_SHIP_TO_CARRY_HIGHLIGHTED},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,						STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   239,     0,    13, STR_983B_REFIT,			STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    14,     0,   239,    14,   135, 0x0,									STR_983D_SELECT_TYPE_OF_CARGO_FOR},
+{     WWT_IMGBTN,    14,     0,   239,   136,   157, 0x0,									STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,   239,   158,   169, STR_983C_REFIT_SHIP,	STR_983E_REFIT_SHIP_TO_CARRY_HIGHLIGHTED},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _ship_refit_desc = {
@@ -137,8 +136,8 @@ static void ShowShipRefitWindow(Vehicle *v)
 	Window *w;
 
 	DeleteWindowById(WC_VEHICLE_REFIT, v->index);
-	
-	_alloc_wnd_parent_num = v->index;	
+
+	_alloc_wnd_parent_num = v->index;
 	w = AllocateWindowDesc(&_ship_refit_desc);
 	w->window_number = v->index;
 	w->caption_color = v->owner;
@@ -154,6 +153,9 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 	switch(e->event) {
 	case WE_PAINT:
 		w->disabled_state = v->owner == _local_player ? 0 : (1 << 2);
+		if (!_patches.servint_ships) // disable service-scroller when interval is set to disabled
+			w->disabled_state |= (1 << 5) | (1 << 6);
+
 		SET_DPARAM16(0, v->string_id);
 		SET_DPARAM16(1, v->unitnumber);
 		DrawWindowWidgets(w);
@@ -164,7 +166,7 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 			StringID str;
 
 			SET_DPARAM16(1, year);
-			
+
 			str = STR_0199_YEAR;
 			if (year != 1) {
 				str++;
@@ -201,7 +203,7 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 		{
 			SET_DPARAM16(0, v->service_interval);
 			SET_DPARAM16(1, v->date_of_last_service);
-			DrawString(13, 90, STR_883C_SERVICING_INTERVAL_DAYS, 0);
+			DrawString(13, 90, _patches.servint_ispercent?STR_SERVICING_INTERVAL_PERCENT:STR_883C_SERVICING_INTERVAL_DAYS, 0);
 		}
 
 		DrawShipImage(v, 3, 57, INVALID_VEHICLE);
@@ -210,7 +212,7 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 		SET_DPARAM16(0, GetCustomEngineName(v->engine_type));
 		SET_DPARAM32(2, v->value);
 		DrawString(74, 57, STR_9816_BUILT_VALUE, 0);
-		
+
 		SET_DPARAM16(0, _cargoc.names_long_p[v->cargo_type]);
 		SET_DPARAM16(1, v->cargo_cap);
 		DrawString(74, 67, STR_9817_CAPACITY, 0);
@@ -232,14 +234,19 @@ static void ShipDetailsWndProc(Window *w, WindowEvent *e)
 			ShowQueryString(v->string_id, STR_9831_NAME_SHIP, 31, 150, w->window_class, w->window_number);
 			break;
 		case 5: /* increase int */
-			mod = 10;
+			mod = _ctrl_pressed? 5 : 10;
 			goto change_int;
 		case 6: /* decrease int */
-			mod = -10;
+			mod = _ctrl_pressed?- 5 : -10;
 change_int:
 			mod += v->service_interval;
-			if (!IS_INT_INSIDE(mod, 30, 800+1))
+
+			/*	%-based service interval max 5%-90%
+					day-based service interval max 30-800 days */
+			mod = _patches.servint_ispercent ? clamp(mod, MIN_SERVINT_PERCENT, MAX_SERVINT_PERCENT) : clamp(mod, MIN_SERVINT_DAYS, MAX_SERVINT_DAYS+1);
+			if (mod == v->service_interval)
 				return;
+
 			DoCommandP(v->tile, v->index, mod, NULL, CMD_CHANGE_SHIP_SERVICE_INT | CMD_MSG(STR_018A_CAN_T_CHANGE_SERVICING));
 			break;
 		}
@@ -263,15 +270,15 @@ change_int:
 
 
 static const Widget _ship_details_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,				STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,    14,    11,   364,     0,    13, STR_9811_DETAILS,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{ WWT_PUSHTXTBTN,    14,   365,   404,     0,    13, STR_01AA_NAME, STR_982F_NAME_SHIP},
-{     WWT_IMGBTN,    14,     0,   404,    14,    55, 0x0},
-{     WWT_IMGBTN,    14,     0,   404,    56,    88, 0x0},
-{ WWT_PUSHTXTBTN,    14,     0,    10,    89,    94, STR_0188,STR_884D_INCREASE_SERVICING_INTERVAL},
-{ WWT_PUSHTXTBTN,    14,     0,    10,    95,   100, STR_0189,STR_884E_DECREASE_SERVICING_INTERVAL},
-{     WWT_IMGBTN,    14,    11,   404,    89,   100, 0x0},
-{      WWT_LAST},
+{ WWT_PUSHTXTBTN,    14,   365,   404,     0,    13, STR_01AA_NAME,		STR_982F_NAME_SHIP},
+{     WWT_IMGBTN,    14,     0,   404,    14,    55, 0x0,							STR_NULL},
+{     WWT_IMGBTN,    14,     0,   404,    56,    88, 0x0,							STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,    10,    89,    94, STR_0188,				STR_884D_INCREASE_SERVICING_INTERVAL},
+{ WWT_PUSHTXTBTN,    14,     0,    10,    95,   100, STR_0189,				STR_884E_DECREASE_SERVICING_INTERVAL},
+{     WWT_IMGBTN,    14,    11,   404,    89,   100, 0x0,							STR_NULL},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _ship_details_desc = {
@@ -288,7 +295,7 @@ static void ShowShipDetailsWindow(Vehicle *v)
 	VehicleID veh = v->index;
 	DeleteWindowById(WC_VEHICLE_ORDERS, veh);
 	DeleteWindowById(WC_VEHICLE_DETAILS, veh);
-	_alloc_wnd_parent_num = veh;	
+	_alloc_wnd_parent_num = veh;
 	w = AllocateWindowDesc(&_ship_details_desc);
 	w->window_number = veh;
 	w->caption_color = v->owner;
@@ -307,7 +314,7 @@ static void CcBuildShip(bool success, uint tile, uint32 p1, uint32 p2)
 	ShowShipViewWindow(v);
 }
 
-static void NewShipWndProc(Window *w, WindowEvent *e) 
+static void NewShipWndProc(Window *w, WindowEvent *e)
 {
 	YearMonthDay ymd;
 	switch(e->event) {
@@ -347,7 +354,7 @@ static void NewShipWndProc(Window *w, WindowEvent *e)
 						DrawShipEngine(x+35, y+10, engine_id, SPRITE_PALETTE(PLAYER_SPRITE_COLOR(_local_player)));
 						y += 24;
 					}
-					sel--; 
+					sel--;
 				}
 			} while (++engine_id, ++e,--num);
 
@@ -363,12 +370,12 @@ static void NewShipWndProc(Window *w, WindowEvent *e)
 				SET_DPARAM16(4, ship_vehicle_info(selected_id).refittable ? STR_9842_REFITTABLE : STR_EMPTY);
 				SET_DPARAM32(5, ship_vehicle_info(selected_id).running_cost * _price.ship_running >> 8);
 
-				e = &_engines[selected_id];	
+				e = &_engines[selected_id];
 				SET_DPARAM16(7, e->lifelength);
 				SET_DPARAM8(8, e->reliability * 100 >> 16);
 				ConvertDayToYMD(&ymd, e->intro_date);
 				SET_DPARAM16(6, ymd.year + 1920);
-				
+
 				DrawString(2, 111, STR_980A_COST_SPEED_CAPACITY_RUNNING, 0);
 			}
 		}
@@ -421,14 +428,14 @@ static void NewShipWndProc(Window *w, WindowEvent *e)
 }
 
 static const Widget _new_ship_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   254,     0,    13, STR_9808_NEW_SHIPS, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_MATRIX,    14,     0,   243,    14,   109, 0x401, STR_9825_SHIP_SELECTION_LIST_CLICK},
-{  WWT_SCROLLBAR,    14,   244,   254,    14,   109, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{     WWT_IMGBTN,    14,     0,   254,   110,   161, 0x0},
-{ WWT_PUSHTXTBTN,    14,     0,   126,   162,   173, STR_9809_BUILD_SHIP, STR_9826_BUILD_THE_HIGHLIGHTED_SHIP},
-{ WWT_PUSHTXTBTN,    14,   127,   254,   162,   173, STR_9836_RENAME, STR_9837_RENAME_SHIP_TYPE},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,						STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   254,     0,    13, STR_9808_NEW_SHIPS,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_MATRIX,    14,     0,   243,    14,   109, 0x401,								STR_9825_SHIP_SELECTION_LIST_CLICK},
+{  WWT_SCROLLBAR,    14,   244,   254,    14,   109, 0x0,									STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{     WWT_IMGBTN,    14,     0,   254,   110,   161, 0x0,									STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,   126,   162,   173, STR_9809_BUILD_SHIP,	STR_9826_BUILD_THE_HIGHLIGHTED_SHIP},
+{ WWT_PUSHTXTBTN,    14,   127,   254,   162,   173, STR_9836_RENAME,			STR_9837_RENAME_SHIP_TYPE},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _new_ship_desc = {
@@ -449,13 +456,13 @@ static void ShowBuildShipWindow(TileIndex tile)
 	w = AllocateWindowDesc(&_new_ship_desc);
 	w->window_number = tile;
 	w->vscroll.cap = 4;
-	
+
 	if (tile != 0) {
 		w->caption_color = _map_owner[tile];
 	} else {
 		w->caption_color = _local_player;
 	}
-	
+
 }
 
 
@@ -472,7 +479,7 @@ static void ShipViewWndProc(Window *w, WindowEvent *e) {
 				v->u.ship.state == 0x80 &&
 				IsShipDepotTile(v->tile))
 			disabled = 0;
-					
+
 		if (v->owner != _local_player)
 			disabled |= 1<<7 | 1<<6;
 		w->disabled_state = disabled;
@@ -499,7 +506,7 @@ static void ShipViewWndProc(Window *w, WindowEvent *e) {
 
 			case OT_GOTO_DEPOT: {
 				Depot *dep = &_depots[v->next_order_param];
-				SET_DPARAM16(0, dep->city_index);
+				SET_DPARAM16(0, dep->town_index);
 				SET_DPARAM16(1, v->cur_speed * 10 >> 5);
 				str = STR_HEADING_FOR_SHIP_DEPOT + _patches.vehicle_speed;
 			} break;
@@ -528,13 +535,13 @@ static void ShipViewWndProc(Window *w, WindowEvent *e) {
 
 		switch(e->click.widget) {
 		case 4: /* start stop */
-			DoCommandP(v->tile, v->index, 0, NULL, CMD_START_STOP_SHIP | CMD_MSG(STR_9818_CAN_T_STOP_START_SHIP)); 
+			DoCommandP(v->tile, v->index, 0, NULL, CMD_START_STOP_SHIP | CMD_MSG(STR_9818_CAN_T_STOP_START_SHIP));
 			break;
 		case 5: /* center main view */
 			ScrollMainWindowTo(v->x_pos, v->y_pos);
 			break;
 		case 6: /* goto hangar */
-			DoCommandP(v->tile, v->index, 0, NULL, CMD_SEND_SHIP_TO_DEPOT | CMD_MSG(STR_9819_CAN_T_SEND_SHIP_TO_DEPOT)); 
+			DoCommandP(v->tile, v->index, 0, NULL, CMD_SEND_SHIP_TO_DEPOT | CMD_MSG(STR_9819_CAN_T_SEND_SHIP_TO_DEPOT));
 			break;
 		case 7: /* refit */
 			ShowShipRefitWindow(v);
@@ -550,24 +557,24 @@ static void ShipViewWndProc(Window *w, WindowEvent *e) {
 
 	case WE_DESTROY:
 		DeleteWindowById(WC_VEHICLE_ORDERS, w->window_number);
-		DeleteWindowById(WC_VEHICLE_REFIT, w->window_number);		
+		DeleteWindowById(WC_VEHICLE_REFIT, w->window_number);
 		DeleteWindowById(WC_VEHICLE_DETAILS, w->window_number);
 		break;
 	}
 }
 
 static const Widget _ship_view_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   249,     0,    13, STR_980F, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    14,     0,   231,    14,   103, 0x0},
-{          WWT_6,    14,     2,   229,    16,   101, 0},
-{ WWT_PUSHIMGBTN,    14,     0,   249,   104,   115, 0x0, STR_9827_CURRENT_SHIP_ACTION_CLICK},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    14,    31, 0x2AB, STR_9829_CENTER_MAIN_VIEW_ON_SHIP},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    32,    49, 0x2B0, STR_982A_SEND_SHIP_TO_DEPOT},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    50,    67, 0x2B4, STR_983A_REFIT_CARGO_SHIP_TO_CARRY},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    68,    85, 0x2B2, STR_9828_SHOW_SHIP_S_ORDERS},
-{ WWT_PUSHIMGBTN,    14,   232,   249,    86,   103, 0x2B3, STR_982B_SHOW_SHIP_DETAILS},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,	STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   249,     0,    13, STR_980F,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    14,     0,   231,    14,   103, 0x0,				STR_NULL},
+{          WWT_6,    14,     2,   229,    16,   101, 0x0,				STR_NULL},
+{ WWT_PUSHIMGBTN,    14,     0,   249,   104,   115, 0x0,				STR_9827_CURRENT_SHIP_ACTION_CLICK},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    14,    31, 0x2AB,			STR_9829_CENTER_MAIN_VIEW_ON_SHIP},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    32,    49, 0x2B0,			STR_982A_SEND_SHIP_TO_DEPOT},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    50,    67, 0x2B4,			STR_983A_REFIT_CARGO_SHIP_TO_CARRY},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    68,    85, 0x2B2,			STR_9828_SHOW_SHIP_S_ORDERS},
+{ WWT_PUSHIMGBTN,    14,   232,   249,    86,   103, 0x2B3,			STR_982B_SHOW_SHIP_DETAILS},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _ship_view_desc = {
@@ -625,7 +632,7 @@ static void DrawShipDepotWindow(Window *w)
 	/* locate the depot struct */
 	for(d=_depots; d->xy != (TileIndex)tile; d++) {}
 
-	SET_DPARAM16(0, d->city_index);
+	SET_DPARAM16(0, d->town_index);
 	DrawWindowWidgets(w);
 
 	x = 2;
@@ -634,17 +641,17 @@ static void DrawShipDepotWindow(Window *w)
 
 	FOR_ALL_VEHICLES(v) {
 		if (v->type == VEH_Ship &&
-				v->u.ship.state == 0x80 && 
+				v->u.ship.state == 0x80 &&
 				v->tile == (TileIndex)tile &&
 				--num < 0 && num >=	-6) {
 
 			DrawShipImage(v, x+19, y, WP(w,traindepot_d).sel);
-			
+
 			SET_DPARAM16(0, v->unitnumber);
 			DrawString(x, y, (uint16)(v->max_age-366) >= v->age ? STR_00E2 : STR_00E3, 0);
 
 			DrawSprite( (v->vehstatus & VS_STOPPED) ? 0xC12 : 0xC13, x, y + 9);
-			
+
 			if ((x+=90) == 2+90*3) {
 				x = 2;
 				y += 24;
@@ -659,7 +666,7 @@ static int GetVehicleFromShipDepotWndPt(Window *w, int x, int y, Vehicle **veh)
 	TileIndex tile;
 	Vehicle *v;
 	int pos;
-		
+
 	xt = x / 90;
 	xm = x % 90;
 	if (xt >= 5)
@@ -675,7 +682,7 @@ static int GetVehicleFromShipDepotWndPt(Window *w, int x, int y, Vehicle **veh)
 	tile = w->window_number;
 	FOR_ALL_VEHICLES(v) {
 		if (v->type == VEH_Ship &&
-				v->vehstatus&VS_HIDDEN && 
+				v->vehstatus&VS_HIDDEN &&
 				v->tile == (TileIndex)tile &&
 				--pos < 0) {
 					*veh = v;
@@ -688,31 +695,35 @@ static int GetVehicleFromShipDepotWndPt(Window *w, int x, int y, Vehicle **veh)
 	}
 
 	return 1; /* outside */
-	
+
 }
 
 static void ShipDepotClick(Window *w, int x, int y)
 {
 	Vehicle *v;
+	int mode = GetVehicleFromShipDepotWndPt(w, x, y, &v);
 
-	switch (GetVehicleFromShipDepotWndPt(w, x, y, &v)) {
-	case 1:
+	// share / copy orders
+	if (_thd.place_mode && mode <= 0) { _place_clicked_vehicle = v; return; }
+
+	switch (mode) {
+	case 1: // invalid
 		return;
 
-	case 0:
+	case 0: // start dragging of vehicle
 		if (v != NULL) {
 			WP(w,traindepot_d).sel = v->index;
 			SetWindowDirty(w);
-			SetObjectToPlaceWnd( SPRITE_PALETTE(PLAYER_SPRITE_COLOR(v->owner)) + 
+			SetObjectToPlaceWnd( SPRITE_PALETTE(PLAYER_SPRITE_COLOR(v->owner)) +
 				GetShipImage(v, 6), 4, w);
 		}
 		break;
 
-	case -1:
+	case -1: // show info window
 		ShowShipViewWindow(v);
 		break;
 
-	case -2:
+	case -2: // click start/stop flag
 		DoCommandP(v->tile, v->index, 0, NULL, CMD_START_STOP_SHIP | CMD_MSG(STR_9818_CAN_T_STOP_START_SHIP));
 		break;
 
@@ -732,7 +743,7 @@ static void ShipDepotWndProc(Window *w, WindowEvent *e) {
 		case 2:
 			ShipDepotClick(w, e->click.pt.x, e->click.pt.y);
 			break;
-			
+
 		case 5:
 			ShowBuildShipWindow(w->window_number);
 			break;
@@ -762,12 +773,12 @@ static void ShipDepotWndProc(Window *w, WindowEvent *e) {
 				ShowShipViewWindow(v);
 			}
 		} break;
-					
+
 		case 3:
 			if (!HASBIT(w->disabled_state, 3) &&
 					WP(w,traindepot_d).sel != INVALID_VEHICLE)	{
 				Vehicle *v;
-				
+
 				HandleButtonClick(w, 3);
 
 				v = &_vehicles[WP(w,traindepot_d).sel];
@@ -775,7 +786,7 @@ static void ShipDepotWndProc(Window *w, WindowEvent *e) {
 
 				_backup_orders_tile = v->tile;
 				BackupVehicleOrders(v, _backup_orders_data);
-	
+
 				if (!DoCommandP(v->tile, v->index, 0, NULL, CMD_SELL_SHIP | CMD_MSG(STR_980C_CAN_T_SELL_SHIP)))
 					_backup_orders_tile = 0;
 			}
@@ -798,7 +809,7 @@ static const Widget _ship_depot_widgets[] = {
 {  WWT_SCROLLBAR,    14,   294,   304,    14,    61, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
 { WWT_PUSHTXTBTN,    14,     0,   151,    62,    73, STR_9804_NEW_SHIPS, STR_9820_BUILD_NEW_SHIP},
 { WWT_PUSHTXTBTN,    14,   152,   304,    62,    73, STR_00E4_LOCATION, STR_9822_CENTER_MAIN_VIEW_ON_SHIP},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _ship_depot_desc = {
@@ -843,7 +854,7 @@ static void DrawSmallShipSchedule(Vehicle *v, int x, int y) {
 
 		if ((ord & OT_MASK) == OT_GOTO_STATION) {
 			st = DEREF_STATION(ord >> 8);
-			
+
 			if (!(st->had_vehicle_of_type & HVOT_BUOY)) {
 				SET_DPARAM16(0, ord >> 8);
 				DrawString(x, y, STR_A036, 0);
@@ -856,126 +867,244 @@ static void DrawSmallShipSchedule(Vehicle *v, int x, int y) {
 	}
 }
 
+// used to get a sorted list of the vehicles
+static SortStruct _ship_sort[NUM_NORMAL_VEHICLES];
+static uint16 _num_ship_sort[MAX_PLAYERS];
+
+static void GlobalSortShipList()
+{
+	const Vehicle *v;
+	uint16 *i;
+	uint32 n = 0;
+
+	// reset #-of ships to 0 because ++ is used for value-assignment
+	for (i = _num_ship_sort; i != endof(_num_ship_sort); i++) {*i = 0;}
+
+	FOR_ALL_VEHICLES(v) {
+		if(v->type == VEH_Ship) {
+			_ship_sort[n].index = v->index;
+			_ship_sort[n++].owner = v->owner;
+			_num_ship_sort[v->owner]++; // add number of ships of player
+		}
+	}
+
+	// create cumulative ship-ownership
+	// ships are stored as a cummulative index, eg 25, 41, 43. This means
+	// Player0: 25; Player1: (41-25) 16; Player2: (43-41) 2
+	for (i = &_num_ship_sort[1]; i != endof(_num_ship_sort); i++) {*i += *(i-1);}
+
+	qsort(_ship_sort, n, sizeof(_ship_sort[0]), GeneralOwnerSorter); // sort by owner
+
+	// since indexes are messed up after adding/removing a station, mark all lists dirty
+	memset(_ship_sort_dirty, true, sizeof(_ship_sort_dirty));
+	_vehicle_sort_dirty[VEHSHIP] = false;
+
+	DEBUG(misc, 1) ("Resorting global ships list...");	
+}
+
+static void MakeSortedShipList(byte owner)
+{
+	SortStruct *firstelement;
+	uint32 n = 0;
+
+	if (owner == 0) { // first element starts at 0th element and has n elements as described above
+		firstelement =	&_ship_sort[0];
+		n =							_num_ship_sort[0];
+	}	else { // nth element starts at the end of the previous one, and has n elements as described above
+		firstelement =	&_ship_sort[_num_ship_sort[owner-1]];
+		n =							_num_ship_sort[owner] - _num_ship_sort[owner-1];
+	}
+
+	_internal_sort_order			= _ship_sort_order[owner];
+	_internal_name_sorter_id	= STR_SV_SHIP_NAME;
+	_last_vehicle_idx = 0; // used for "cache" in namesorting
+	qsort(firstelement, n, sizeof(_ship_sort[0]), _vehicle_sorter[_ship_sort_type[owner]]);
+
+	_ship_sort_dirty[owner] = false;
+
+	DEBUG(misc, 1) ("Resorting Ships list player %d...", owner+1);
+}
+
 static void PlayerShipsWndProc(Window *w, WindowEvent *e)
 {
 	switch(e->event) {
-	case WE_PAINT:
-		/* determine amount of items for scroller */
-		{
-			Vehicle *v;
-			int num = 0;
-			byte owner = (byte)w->window_number;
+	case WE_PAINT: {
+		uint32 i;
+		const byte window_number = (byte)w->window_number;
 
-			FOR_ALL_VEHICLES(v) {
-				if (v->type == VEH_Ship && v->owner == owner)
-					num++;
-			}
-			SetVScrollCount(w, num);
+		if (_ship_sort_type[window_number] == SORT_BY_UNSORTED) // disable 'Sort By' tooltip on Unsorted sorting criteria
+			w->disabled_state |= (1 << 2);
+
+		// resort shipps window if roadvehicles have been added/removed
+		if (_vehicle_sort_dirty[VEHSHIP])
+			GlobalSortShipList();
+
+		if (_ship_sort_dirty[window_number]) {
+			MakeSortedShipList(window_number);
+			/* reset sorting timeout */
+			w->custom[0] = DAY_TICKS;
+			w->custom[1] = PERIODIC_RESORT_DAYS;
 		}
 
+		// ships are stored as a cummulative index, eg 25, 41, 43. This means
+		// Player0: 25; Player1: (41-25) 16; Player2: (43-41) 2 ships
+		i = (window_number == 0) ? 0 : _num_ship_sort[window_number-1];
+		SetVScrollCount(w, _num_ship_sort[window_number] - i);
+
 		/* draw the widgets */
 		{
-			Player *p = DEREF_PLAYER(w->window_number);
+			Player *p = DEREF_PLAYER(window_number);
+			/* Company Name -- (###) Ships */
 			SET_DPARAM16(0, p->name_1);
 			SET_DPARAM32(1, p->name_2);
+			SET_DPARAM16(2, w->vscroll.count);
 			DrawWindowWidgets(w);
 		}
+		/* draw sorting criteria string */
+		DrawString(85, 15, _vehicle_sort_listing[_ship_sort_type[window_number]], 0x10);
+		/* draw arrow pointing up/down for ascending/descending soring */
+		DoDrawString(_ship_sort_order[window_number] & 1 ? "\xAA" : "\xA0", 69, 15, 0x10);
 
-		/* draw the ships vehicles */
+		/* draw the ship vehicles */
 		{
 			Vehicle *v;
-			int pos = w->vscroll.pos;
-			byte owner = (byte)w->window_number;
-			int x = 2;
-			int y = 15;
-			
-			FOR_ALL_VEHICLES(v) {
-				if (v->type == VEH_Ship && v->owner == owner &&
-						--pos < 0 && pos >= -4) {
-					StringID str;
-					
-					DrawShipImage(v, x + 19, y + 6, INVALID_VEHICLE);
-					DrawVehicleProfitButton(v, x, y+13);
-					
-					SET_DPARAM16(0, v->unitnumber);
-					if (IsShipDepotTile(v->tile)) {
-						str = STR_021F;
-					} else {
-						str = v->age > v->max_age - 366 ? STR_00E3 : STR_00E2;
-					}
-					DrawString(x, y+2, str, 0);
+			int n = 0;
+			const int x = 2;			// offset from left side of widget
+			int y = PLY_WND_PRC__OFFSET_TOP_WIDGET;	// offset from top of widget
+			i += w->vscroll.pos;	// offset from sorted ship list of current player
 
-					SET_DPARAM32(0, v->profit_this_year);
-					SET_DPARAM32(1, v->profit_last_year);
-					DrawString(x + 12, y + 28, STR_0198_PROFIT_THIS_YEAR_LAST_YEAR, 0);
-					
-					if (v->string_id != STR_SV_SHIP_NAME) {
-						SET_DPARAM16(0, v->string_id);
-						DrawString(x+12, y, STR_01AB, 0);
-					}
+			while (i < _num_ship_sort[window_number]) {
+				StringID str;
+				v = DEREF_VEHICLE(_ship_sort[i].index);
 
-					DrawSmallShipSchedule(v, x+138, y);
+				assert(v->type == VEH_Ship && v->owner == window_number);
 
-					y += 36;
-				}				
-			}
-		}
-		break;
-	case WE_CLICK:
-		switch(e->click.widget) {
-		case 2: { /* click ship */
-			int sel;
-			Vehicle *v;
-			byte owner;
-	
-			sel = (e->click.pt.y - 14) / 36;
+				DrawShipImage(v, x + 19, y + 6, INVALID_VEHICLE);
+				DrawVehicleProfitButton(v, x, y+13);
 
-			if ((uint)sel >= 4)
-				break;
-			sel += w->vscroll.pos;
-			owner = (byte)w->window_number;
-			FOR_ALL_VEHICLES(v) {
-				if (v->type == VEH_Ship && v->owner == owner &&	--sel < 0) {	
-					ShowShipViewWindow(v);
-					break;
+				SET_DPARAM16(0, v->unitnumber);
+				if (IsShipDepotTile(v->tile)) {
+					str = STR_021F;
+				} else {
+					str = v->age > v->max_age - 366 ? STR_00E3 : STR_00E2;
 				}
+				DrawString(x, y+2, str, 0);
+
+				SET_DPARAM32(0, v->profit_this_year);
+				SET_DPARAM32(1, v->profit_last_year);
+				DrawString(x + 12, y + 28, STR_0198_PROFIT_THIS_YEAR_LAST_YEAR, 0);
+
+				if (v->string_id != STR_SV_SHIP_NAME) {
+					SET_DPARAM16(0, v->string_id);
+					DrawString(x+12, y, STR_01AB, 0);
+				}
+
+				DrawSmallShipSchedule(v, x+138, y);
+
+
+				y += PLY_WND_PRC__SIZE_OF_ROW_BIG;
+				i++; // next ship
+				if (++n == w->vscroll.cap) { break;} // max number of ships in the window
 			}
-			break;
 		}
-		case 4: {/* click buy */
+		}	break;
+
+	case WE_CLICK: {
+		switch(e->click.widget) {
+		case 2: /* Flip sorting method ascending/descending */
+			_ship_sort_order[(byte)w->window_number] ^= 1;
+			_ship_sort_dirty[(byte)w->window_number] = true;
+			SetWindowDirty(w);
+			break;
+		case 3: case 4:/* Select sorting criteria dropdown menu */
+			ShowDropDownMenu(w, _vehicle_sort_listing, _ship_sort_type[(byte)w->window_number], 4, 0); // do it for widget 4
+			return;
+		case 6: { /* Matrix to show vehicles */
+			uint32 id_v = (e->click.pt.y - PLY_WND_PRC__OFFSET_TOP_WIDGET) / PLY_WND_PRC__SIZE_OF_ROW_BIG;
+
+			if (id_v >= w->vscroll.cap) { return;} // click out of bounds
+
+			id_v += w->vscroll.pos;
+
+			{
+				const byte owner = (byte)w->window_number;
+				Vehicle *v;
+				id_v += (owner == 0) ? 0 : _num_ship_sort[owner - 1]; // first element in list
+
+				if (id_v >= _num_ship_sort[owner]) { return;} // click out of vehicle bound
+
+				v	= DEREF_VEHICLE(_ship_sort[id_v].index); // add the offset id_x to that
+
+				assert(v->type == VEH_Ship && v->owner == owner);
+
+				ShowShipViewWindow(v);
+			}
+		} break;
+
+		case 8: { /* Build new Vehicle */
 			uint tile;
 
 			tile = _last_built_ship_depot_tile;
 			do {
-				if (_map_owner[tile] == _local_player &&
-						IsShipDepotTile(tile)) {
-					
+				if (_map_owner[tile] == _local_player && IsShipDepotTile(tile)) {
 					ShowShipDepotWindow(tile);
 					ShowBuildShipWindow(tile);
 					return;
 				}
-				
+
 				tile = TILE_MASK(tile + 1);
 			} while(tile != _last_built_ship_depot_tile);
-			
+
 			ShowBuildShipWindow(0);
 		} break;
 		}
+	}	break;
+
+	case WE_DROPDOWN_SELECT: /* we have selected a dropdown item in the list */
+		if (_ship_sort_type[(byte)w->window_number] != e->dropdown.index) // if value hasn't changed, dont resort list
+			_ship_sort_dirty[(byte)w->window_number] = true;
+
+		_ship_sort_type[(byte)w->window_number] = e->dropdown.index;
+
+		if (_ship_sort_type[(byte)w->window_number] != SORT_BY_UNSORTED) // enable 'Sort By' if a sorter criteria is chosen
+			w->disabled_state &= ~(1 << 2);
+
+		SetWindowDirty(w);
+		break;
+	case WE_CREATE: /* set up resort timer */
+		w->custom[0] = DAY_TICKS;
+		w->custom[1] = PERIODIC_RESORT_DAYS;
+		break;
+	case WE_TICK: /* resort the list every 20 seconds orso (10 days) */
+		if (--w->custom[0] == 0) {
+			w->custom[0] = DAY_TICKS;
+			if (--w->custom[1] == 0) {
+				w->custom[1] = PERIODIC_RESORT_DAYS;
+				_ship_sort_dirty[(byte)w->window_number] = true;
+				DEBUG(misc, 1) ("Periodic resort Ships list player %d...", w->window_number+1);
+				SetWindowDirty(w);
+			}
+		}
 		break;
 	}
 }
 
 static const Widget _player_ships_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_9805_SHIPS, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_MATRIX,    14,     0,   248,    14,   157, 0x401, STR_9823_SHIPS_CLICK_ON_SHIP_FOR},
-{  WWT_SCROLLBAR,    14,   249,   259,    14,   157, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{ WWT_PUSHTXTBTN,    14,     0,   129,   158,   169, STR_9804_NEW_SHIPS, STR_9824_BUILD_NEW_SHIPS_REQUIRES},
-{     WWT_IMGBTN,    14,   130,   259,   158,   169, 0x0},
-{      WWT_LAST},
+{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,							STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_9805_SHIPS,				STR_018C_WINDOW_TITLE_DRAG_THIS},
+{ WWT_PUSHTXTBTN,    14,     0,    80,    14,    25, SRT_SORT_BY,						STR_SORT_TIP},
+{      WWT_PANEL,    14,    81,   237,    14,    25, 0x0,										STR_NULL},
+{   WWT_CLOSEBOX,    14,   238,   248,    14,    25, STR_0225,							STR_SORT_TIP},
+{      WWT_PANEL,    14,   249,   259,    14,    25, 0x0,										STR_NULL},
+{     WWT_MATRIX,    14,     0,   248,    26,   169, 0x401,									STR_9823_SHIPS_CLICK_ON_SHIP_FOR},
+{  WWT_SCROLLBAR,    14,   249,   259,    26,   169, 0x0,										STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{ WWT_PUSHTXTBTN,    14,     0,   129,   170,   181, STR_9804_NEW_SHIPS,		STR_9824_BUILD_NEW_SHIPS_REQUIRES},
+{      WWT_PANEL,    14,   130,   259,   170,   181, 0x0,										STR_NULL},
+{   WIDGETS_END},
 };
+
 static const WindowDesc _player_ships_desc = {
-	-1, -1, 260, 170,
+	-1, -1, 260, 182,
 	WC_SHIPS_LIST,0,
 	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
 	_player_ships_widgets,
@@ -983,14 +1112,19 @@ static const WindowDesc _player_ships_desc = {
 };
 
 static const Widget _other_player_ships_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_9805_SHIPS, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_MATRIX,    14,     0,   248,    14,   157, 0x401, STR_9823_SHIPS_CLICK_ON_SHIP_FOR},
-{  WWT_SCROLLBAR,    14,   249,   259,    14,   157, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{      WWT_LAST},
+{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,							STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   259,     0,    13, STR_9805_SHIPS,				STR_018C_WINDOW_TITLE_DRAG_THIS},
+{ WWT_PUSHTXTBTN,    14,     0,    80,    14,    25, SRT_SORT_BY,						STR_SORT_TIP},
+{      WWT_PANEL,    14,    81,   237,    14,    25, 0x0,										STR_NULL},
+{   WWT_CLOSEBOX,    14,   238,   248,    14,    25, STR_0225,							STR_SORT_TIP},
+{      WWT_PANEL,    14,   249,   259,    14,    25, 0x0,										STR_NULL},
+{     WWT_MATRIX,    14,     0,   248,    26,   169, 0x401,									STR_9823_SHIPS_CLICK_ON_SHIP_FOR},
+{  WWT_SCROLLBAR,    14,   249,   259,    26,   169, 0x0,										STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{   WIDGETS_END},
 };
+
 static const WindowDesc _other_player_ships_desc = {
-	-1, -1, 260, 158,
+	-1, -1, 260, 170,
 	WC_SHIPS_LIST,0,
 	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
 	_other_player_ships_widgets,
