@@ -1,65 +1,65 @@
 #include "stdafx.h"
 #include "ttd.h"
-
+#include "gui.h"
 #include "window.h"
 #include "gfx.h"
 #include "viewport.h"
 #include "player.h"
 #include "vehicle.h"
-#include "city.h"
+#include "town.h"
 
 static const Widget _smallmap_megabig_widgets[] = {
-{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
+{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,    13,    11,   592,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    13,   593,   606,     0,    13, 0x2AA,STR_01CC_TOGGLE_LARGE_SMALL_MAP},
-{     WWT_IMGBTN,    13,     0,   606,    14,   407, 0x0, 0},
-{          WWT_6,    13,     2,   604,    16,   405, 0x0, 0},
-{     WWT_IMGBTN,    13,   541,   562,   408,   429, 0x2E2,STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
-{     WWT_IMGBTN,    13,   563,   584,   408,   429, 0x2E3,STR_0192_SHOW_VEHICLES_ON_MAP},
-{     WWT_IMGBTN,    13,   585,   606,   408,   429, 0x2E5,STR_0193_SHOW_INDUSTRIES_ON_MAP},
-{     WWT_IMGBTN,    13,   519,   540,   430,   451, 0x2E4,STR_0194_SHOW_TRANSPORT_ROUTES_ON},
-{     WWT_IMGBTN,    13,   541,   562,   430,   451, 0x2E6,STR_0195_SHOW_VEGETATION_ON_MAP},
-{     WWT_IMGBTN,    13,   563,   584,   430,   451, 0x2E7,STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   585,   606,   430,   451, 0xFED,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   518,   408,   451, 0x0,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,   519,   540,   408,   429, 0x0,0},
-{      WWT_LAST},
+{     WWT_IMGBTN,    13,   593,   606,     0,    13, 0x2AA,				STR_01CC_TOGGLE_LARGE_SMALL_MAP},
+{     WWT_IMGBTN,    13,     0,   606,    14,   407, 0x0,					STR_NULL},
+{          WWT_6,    13,     2,   604,    16,   405, 0x0,					STR_NULL},
+{     WWT_IMGBTN,    13,   541,   562,   408,   429, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
+{     WWT_IMGBTN,    13,   563,   584,   408,   429, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
+{     WWT_IMGBTN,    13,   585,   606,   408,   429, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
+{     WWT_IMGBTN,    13,   519,   540,   430,   451, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
+{     WWT_IMGBTN,    13,   541,   562,   430,   451, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
+{     WWT_IMGBTN,    13,   563,   584,   430,   451, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
+{     WWT_IMGBTN,    13,   585,   606,   430,   451, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,     0,   518,   408,   451, 0x0,					STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,   519,   540,   408,   429, 0x0,					STR_NULL},
+{   WIDGETS_END},
 };
 
 static const Widget _smallmap_big_widgets[] = {
-{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
+{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,    13,    11,   431,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    13,   432,   445,     0,    13, 0x2AA,STR_01CC_TOGGLE_LARGE_SMALL_MAP},
-{     WWT_IMGBTN,    13,     0,   445,    14,   257, 0x0, 0},
-{          WWT_6,    13,     2,   443,    16,   255, 0x0, 0},
-{     WWT_IMGBTN,    13,   380,   401,   258,   279, 0x2E2,STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
-{     WWT_IMGBTN,    13,   402,   423,   258,   279, 0x2E3,STR_0192_SHOW_VEHICLES_ON_MAP},
-{     WWT_IMGBTN,    13,   424,   445,   258,   279, 0x2E5,STR_0193_SHOW_INDUSTRIES_ON_MAP},
-{     WWT_IMGBTN,    13,   380,   401,   280,   301, 0x2E4,STR_0194_SHOW_TRANSPORT_ROUTES_ON},
-{     WWT_IMGBTN,    13,   402,   423,   280,   301, 0x2E6,STR_0195_SHOW_VEGETATION_ON_MAP},
-{     WWT_IMGBTN,    13,   424,   445,   280,   301, 0x2E7,STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   358,   379,   280,   301, 0xFED,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   357,   258,   301, 0x0,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,   358,   379,   258,   279, 0x0,0},
-{      WWT_LAST},
+{     WWT_IMGBTN,    13,   432,   445,     0,    13, 0x2AA,				STR_01CC_TOGGLE_LARGE_SMALL_MAP},
+{     WWT_IMGBTN,    13,     0,   445,    14,   257, 0x0,					STR_NULL},
+{          WWT_6,    13,     2,   443,    16,   255, 0x0,					STR_NULL},
+{     WWT_IMGBTN,    13,   380,   401,   258,   279, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
+{     WWT_IMGBTN,    13,   402,   423,   258,   279, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
+{     WWT_IMGBTN,    13,   424,   445,   258,   279, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
+{     WWT_IMGBTN,    13,   380,   401,   280,   301, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
+{     WWT_IMGBTN,    13,   402,   423,   280,   301, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
+{     WWT_IMGBTN,    13,   424,   445,   280,   301, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
+{     WWT_IMGBTN,    13,   358,   379,   280,   301, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,     0,   357,   258,   301, 0x0,					STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,   358,   379,   258,   279, 0x0,					STR_NULL},
+{   WIDGETS_END},
 };
 
 
 static const Widget _smallmap_small_widgets[] = {
-{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,STR_018B_CLOSE_WINDOW},
+{    WWT_TEXTBTN,    13,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
 {    WWT_CAPTION,    13,    11,   233,     0,    13, STR_00B0_MAP,STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    13,   234,   247,     0,    13, 0x2AA,STR_01CC_TOGGLE_LARGE_SMALL_MAP},
-{     WWT_IMGBTN,    13,     0,   225,    14,   167, 0x0, 0},
-{          WWT_6,    13,     2,   223,    16,   165, 0x0, 0},
-{     WWT_IMGBTN,    13,   226,   247,    14,    35, 0x2E2,STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,    36,    57, 0x2E3,STR_0192_SHOW_VEHICLES_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,    58,    79, 0x2E5,STR_0193_SHOW_INDUSTRIES_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,    80,   101, 0x2E4,STR_0194_SHOW_TRANSPORT_ROUTES_ON},
-{     WWT_IMGBTN,    13,   226,   247,   102,   123, 0x2E6,STR_0195_SHOW_VEGETATION_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,   124,   145, 0x2E7,STR_0196_SHOW_LAND_OWNERS_ON_MAP},
-{     WWT_IMGBTN,    13,   226,   247,   146,   167, 0xFED,STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
-{     WWT_IMGBTN,    13,     0,   247,   168,   211, 0x0,0},
-{      WWT_LAST},
+{     WWT_IMGBTN,    13,   234,   247,     0,    13, 0x2AA,				STR_01CC_TOGGLE_LARGE_SMALL_MAP},
+{     WWT_IMGBTN,    13,     0,   225,    14,   167, 0x0,					STR_NULL},
+{          WWT_6,    13,     2,   223,    16,   165, 0x0,					STR_NULL},
+{     WWT_IMGBTN,    13,   226,   247,    14,    35, 0x2E2,				STR_0191_SHOW_LAND_CONTOURS_ON_MAP},
+{     WWT_IMGBTN,    13,   226,   247,    36,    57, 0x2E3,				STR_0192_SHOW_VEHICLES_ON_MAP},
+{     WWT_IMGBTN,    13,   226,   247,    58,    79, 0x2E5,				STR_0193_SHOW_INDUSTRIES_ON_MAP},
+{     WWT_IMGBTN,    13,   226,   247,    80,   101, 0x2E4,				STR_0194_SHOW_TRANSPORT_ROUTES_ON},
+{     WWT_IMGBTN,    13,   226,   247,   102,   123, 0x2E6,				STR_0195_SHOW_VEGETATION_ON_MAP},
+{     WWT_IMGBTN,    13,   226,   247,   124,   145, 0x2E7,				STR_0196_SHOW_LAND_OWNERS_ON_MAP},
+{     WWT_IMGBTN,    13,   226,   247,   146,   167, 0xFED,				STR_0197_TOGGLE_TOWN_NAMES_ON_OFF},
+{     WWT_IMGBTN,    13,     0,   247,   168,   211, 0x0,					STR_NULL},
+{   WIDGETS_END},
 };
 
 static int _smallmap_type;
@@ -78,7 +78,7 @@ static const uint16 _legend_land_contours[] = {
 	MK(0x5E,STR_00F2_300M),
 	MK(0x1F,STR_00F3_400M),
 	MK(0x27,STR_00F4_500M),
-	
+
 	MS(0xD7,STR_00EB_ROADS),
 	MK(0x0A,STR_00EC_RAILROADS),
 	MK(0x98,STR_00ED_STATIONS_AIRPORTS_DOCKS),
@@ -351,7 +351,8 @@ static void DrawSmallMapContours(byte *dst, uint xc, uint yc, int pitch, int rep
 {
 	do {
 		if (xc < TILE_X_MAX && yc < TILE_Y_MAX)
-			WRITE_PIXELS_OR( dst, GetSmallMapCountoursPixels(TILE_XY(xc,yc)) & mask );
+		    if (dst > _screen.dst_ptr && dst < (_screen.dst_ptr + _screen.width * _screen.height - _screen.width) )
+		        WRITE_PIXELS_OR( dst, GetSmallMapCountoursPixels(TILE_XY(xc,yc)) & mask );
 	} while (xc++,yc++,dst+=pitch,--reps != 0);
 }
 
@@ -508,7 +509,7 @@ static uint32 INLINE GetSmallMapVegetationPixels(uint tile)
 		if (i >= 0) i = (i >> 2);
 		bits = _vegetation_clear_bits[i + 4];
 	} else if (t == MP_INDUSTRY) {
-		bits = IS_BYTE_INSIDE(_map5[tile], 0x10, 0x12) ? MKCOLOR(0xD0D0D0D0) : MKCOLOR(0xB5B5B5B5); 
+		bits = IS_BYTE_INSIDE(_map5[tile], 0x10, 0x12) ? MKCOLOR(0xD0D0D0D0) : MKCOLOR(0xB5B5B5B5);
 	} else if (t == MP_TREES) {
 		bits = MKCOLOR(0x54575754);
 		if ((_map2[tile] & 0x30) == 0x20)
@@ -549,7 +550,7 @@ static uint32 INLINE GetSmallMapOwnerPixels(uint tile)
 
 	t = GET_TILETYPE(tile);
 	if (t == MP_HOUSE || _map_owner[tile] == OWNER_TOWN) {
-		t = 0x80;				
+		t = 0x80;
 	} else if (t == MP_INDUSTRY) {
 		t = 0xff;
 	} else {
@@ -640,10 +641,10 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 
 		/* setup owners ptr */
 		_owner_colors = tbl = alloca(256 * sizeof(*_owner_colors));
-		
-		/* clear the city colors */
+
+		/* clear the town colors */
 		memset(tbl + 128, 0xB4, 128 * sizeof(*_owner_colors));
-		
+
 		/* fill with some special colors */
 		tbl[0x10] = MKCOLOR(0x54545454);
 		tbl[0x11] = MKCOLOR(0xCACACACA);
@@ -654,7 +655,7 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 			if (p->is_active) {
 				tbl[p->index] = dup_byte32(GetSpritePtr(0x307 + p->player_color)[0xCB]);
 			}
-		}	
+		}
 	}
 
 	tile_x = (int)WP(w,smallmap_d).scroll_x >> 4;
@@ -711,7 +712,7 @@ static void DrawSmallMap(DrawPixelInfo *dpi, Window *w, int type, bool show_town
 			proc(ptr, tile_x, tile_y, dpi->pitch*2, reps, mask);
 		}
 
-skip_column:		
+skip_column:
 		if (y == 0) {
 			tile_y++;
 			y++;
@@ -735,7 +736,7 @@ skip_column:
 			if (v->type != 0 && v->type != 0x14 && (v->vehstatus & (VS_HIDDEN|VS_UNCLICKABLE)) == 0) {
 				// Remap into flat coordinates.
 				Point pt = RemapCoords(
-					(int)(v->x_pos - WP(w,smallmap_d).scroll_x) >> 4, 
+					(int)(v->x_pos - WP(w,smallmap_d).scroll_x) >> 4,
 					(int)(v->y_pos - WP(w,smallmap_d).scroll_y) >> 4,
 					0);
 				x = pt.x;
@@ -764,11 +765,11 @@ skip_column:
 						continue;
 					skip = true;
 				}
-				
+
 				// Calculate pointer to pixel and the color
 				ptr = dpi->dst_ptr + y * dpi->pitch + x;
 				color = (type == 1) ? _vehicle_type_colors[v->type-0x10] : 0xF;
-				
+
 				// And draw either one or two pixels depending on clipping
 				ptr[0] = color;
 				if (!skip)
@@ -778,29 +779,29 @@ skip_column:
 	}
 
 	if (show_towns) {
-		City *city;
-		FOR_ALL_CITIES(city) {
-			if (city->xy != 0) {
-				// Remap the city coordinate
+		Town *t;
+		FOR_ALL_TOWNS(t) {
+			if (t->xy != 0) {
+				// Remap the town coordinate
 				Point pt = RemapCoords(
-					(int)(GET_TILE_X(city->xy)*16 - WP(w,smallmap_d).scroll_x) >> 4,
-					(int)(GET_TILE_Y(city->xy)*16 - WP(w,smallmap_d).scroll_y) >> 4,
+					(int)(GET_TILE_X(t->xy)*16 - WP(w,smallmap_d).scroll_x) >> 4,
+					(int)(GET_TILE_Y(t->xy)*16 - WP(w,smallmap_d).scroll_y) >> 4,
 					0);
-				x = pt.x - WP(w,smallmap_d).subscroll + 3 - (city->sign.width_2 >> 1);
+				x = pt.x - WP(w,smallmap_d).subscroll + 3 - (t->sign.width_2 >> 1);
 				y = pt.y;
-								
-				// Check if the city sign is within bounds
-				if (x + city->sign.width_2 > dpi->left &&
+
+				// Check if the town sign is within bounds
+				if (x + t->sign.width_2 > dpi->left &&
 						x < dpi->left + dpi->width &&
-						y + 6 > dpi->top && 
+						y + 6 > dpi->top &&
 						y < dpi->top + dpi->height) {
 					// And draw it.
-					SET_DPARAM16(0, city->index);
+					SET_DPARAM16(0, t->index);
 					DrawString(x, y, STR_2056, 12);
 				}
 			}
 		}
-	} 
+	}
 
 	// Draw map indicators
 	{
@@ -808,7 +809,7 @@ skip_column:
 
 		// Find main viewport.
 		vp = FindWindowById(WC_MAIN_WINDOW,0)->viewport;
-		
+
 		pt = RemapCoords(
 			WP(w,smallmap_d).scroll_x,
 			WP(w,smallmap_d).scroll_y,
@@ -845,7 +846,7 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 		/* draw the window */
 		SET_DPARAM16(0, STR_00E5_CONTOURS + _smallmap_type);
 		DrawWindowWidgets(w);
-		
+
 		/* draw the legend */
 		tbl = _legend_table[(_smallmap_type != 2) ? _smallmap_type : (_opt.landscape + IND_OFFS)];
 		x = 4;
@@ -876,7 +877,7 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 	case WE_CLICK:
 		switch(e->click.widget) {
 		case 2: {/* big/small size */
-			// const Widget *wi = w->widget; 
+			// const Widget *wi = w->widget;
 			DeleteWindow(w);
 			SndPlayFx(0x13);
 			{
@@ -902,7 +903,7 @@ static void SmallMapWindowProc(Window *w, WindowEvent *e)
 		case 5: /* show land contours */
 		case 6: /* show vehicles */
 		case 7: /* show industries */
-		case 8: /* show transport routes */			
+		case 8: /* show transport routes */
 		case 9: /* show vegetation */
 		case 10: /* show land owners */
 			w->click_state &= ~(1<<5|1<<6|1<<7|1<<8|1<<9|1<<10);
@@ -977,7 +978,7 @@ static void DoShowSmallMap(int big)
 	int x,y;
 
 	_smallmap_size = big;
-	
+
 	w = AllocateWindowDescFront(_smallmap_descs[big], 0);
 	if (w) {
 		w->click_state = (1<<5) << _smallmap_type | _smallmap_show_towns << 11;
@@ -995,3 +996,96 @@ void ShowSmallMap()
 {
 	DoShowSmallMap(_smallmap_size);
 }
+
+/* Extra ViewPort Window Stuff */
+static Widget _extra_view_port_widgets[] = {
+{   WWT_CLOSEBOX,    14,     0,    10,     0,    13, STR_00C5,	STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   299,     0,    13, STR_EXTRA_VIEW_PORT_TITLE,				STR_018C_WINDOW_TITLE_DRAG_THIS},
+{      WWT_PANEL,    14,     0,   299,    14,   233, 0x0,				STR_NULL},
+{          WWT_6,    14,     2,   297,    16,   231, 0x0,				STR_NULL},
+{      WWT_PANEL,    14,     0,    21,   234,   255, 0x2DF,			STR_017F_ZOOM_THE_VIEW_IN},
+{      WWT_PANEL,    14,    22,    43,   234,   255, 0x2E0,			STR_0180_ZOOM_THE_VIEW_OUT},
+{ WWT_PUSHTXTBTN,    14,    44,   171,   234,   255, STR_EXTRA_VIEW_MOVE_MAIN_TO_VIEW,STR_EXTRA_VIEW_MOVE_MAIN_TO_VIEW_TT},
+{ WWT_PUSHTXTBTN,    14,   172,   299,   234,   255, STR_EXTRA_VIEW_MOVE_VIEW_TO_MAIN,STR_EXTRA_VIEW_MOVE_VIEW_TO_MAIN_TT},
+{   WIDGETS_END},
+};
+
+static void ExtraViewPortWndProc(Window *w, WindowEvent *e)
+{
+	ViewPort *vp = w->viewport;
+
+	switch(e->event) {
+	case WE_PAINT: {
+		// set the number in the title bar
+		SET_DPARAM16(0, (w->window_number+1));
+
+		DrawWindowWidgets(w);
+		DrawWindowViewport(w);
+	}	break;
+	case WE_CLICK: {
+		switch(e->click.widget) {
+		case 4: { /* zoom in */
+			DoZoomInOutWindow(ZOOM_IN,w);
+		} break;
+
+		case 5: { /* zoom out */
+			DoZoomInOutWindow(ZOOM_OUT,w);
+		} break;
+
+		case 6: { /* location button (move main view to same spot as this view) */
+			Window * w2 = FindWindowById(WC_MAIN_WINDOW, 0);
+			int x = WP(w,vp_d).scrollpos_x; // Where is the main looking at
+			int y = WP(w,vp_d).scrollpos_y;
+
+			// set this view to same location. Based on the center, adjusting for zoom
+			WP(w2,vp_d).scrollpos_x =  x - (w2->viewport->virtual_width - (294 <<vp->zoom) )/2;
+			WP(w2,vp_d).scrollpos_y =  y - (w2->viewport->virtual_height - (214 << vp->zoom) )/2;
+		} break;
+		case 7: { /* inverse location button (move this view to same spot as main view) */
+			Window * w2 = FindWindowById(WC_MAIN_WINDOW, 0);
+			int x = WP(w2,vp_d).scrollpos_x;
+			int y = WP(w2,vp_d).scrollpos_y;
+
+			WP(w,vp_d).scrollpos_x =  x + (w2->viewport->virtual_width - (294 <<vp->zoom) )/2;
+			WP(w,vp_d).scrollpos_y =  y + (w2->viewport->virtual_height - (214 << vp->zoom) )/2;
+		} break;
+		}
+	} break;
+	}
+}
+
+static const WindowDesc _extra_view_port_desc = {
+	-1,-1, 300, 256,
+	WC_EXTRA_VIEW_PORT,0,
+	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS,
+	_extra_view_port_widgets,
+	ExtraViewPortWndProc
+};
+
+void ShowExtraViewPortWindow()
+{
+	Window *w, *v;
+	int i = 0;
+
+	// find next free window number for extra viewport
+	while (FindWindowById(WC_EXTRA_VIEW_PORT,i) ) {
+		i++;
+	}
+
+	w = AllocateWindowDescFront(&_extra_view_port_desc,i);
+	if (w) {
+		int x,y;
+		// disable zoom in button
+		w->disabled_state = 1 << 4;
+		// the main window with the main view
+		v = FindWindowById(WC_MAIN_WINDOW, 0);
+		// New viewport start ats (zero,zero)
+		AssignWindowViewport(w, 3, 17, 294, 214, 0 , 0);
+
+		// center on same place as main window (zoom is maximum, no adjustment needed)
+		x = WP(v,vp_d).scrollpos_x;
+		y = WP(v,vp_d).scrollpos_y;
+		WP(w,vp_d).scrollpos_x =  x + (v->viewport->virtual_width - (294) )/2;
+		WP(w,vp_d).scrollpos_y =  y + (v->viewport->virtual_height - (214)  )/2;
+	}
+}
