@@ -5,13 +5,14 @@
 #include "window.h"
 #include "viewport.h"
 #include "command.h"
-#include "city.h"
+#include "town.h"
 #include "vehicle.h"
 #include "news.h"
 #include "saveload.h"
 #include "economy.h"
 #include "player.h"
 #include "airport.h"
+#include "table/directions.h"
 
 // FIXME -- need to be embedded into Airport variable. Is dynamically
 // deducteable from graphics-tile array, so will not be needed
@@ -43,6 +44,13 @@ static Station *GetStationAround(uint tile, int w, int h, int closest_station)
 		if (IS_TILETYPE(tile_cur, MP_STATION)) {
 			int t;
 			t = _map2[tile_cur];
+			{
+				Station *st = DEREF_STATION(t);
+				// you cannot take control of an oilrig!!
+				if (st->airport_type == AT_OILRIG && st->facilities == (FACIL_AIRPORT|FACIL_DOCK))
+					continue;
+			}
+
 			if (closest_station == -1) {
 				closest_station = t;
 			} else if (closest_station != t) {
@@ -54,6 +62,18 @@ static Station *GetStationAround(uint tile, int w, int h, int closest_station)
 	return (closest_station == -1) ? NULL : DEREF_STATION(closest_station);
 }
 
+TileIndex GetStationTileForVehicle(Vehicle *v, Station *st)
+{
+	switch (v->type) {
+		case VEH_Train: 		return st->train_tile;
+		case VEH_Aircraft:	return st->airport_tile;
+		case VEH_Ship:			return st->dock_tile;
+		case VEH_Road:			return (v->cargo_type == CT_PASSENGERS) ? st->bus_tile : st->lorry_tile;
+		default:
+			assert(false);
+			return 0;
+	}
+}
 
 static bool CheckStationSpreadOut(Station *st, uint tile, int w, int h)
 {
@@ -106,7 +126,7 @@ static Station *AllocateStation()
 		return NULL;
 	}
 
-	i = a_free->index;	
+	i = a_free->index;
 	memset(a_free, 0, sizeof(Station));
 	a_free->index = i;
 	return a_free;
@@ -123,7 +143,7 @@ static int CountMapSquareAround(uint tile, byte type, byte min, byte max) {
 		TILE_XY(-6,1),  1, 1, 1, 1, 1, 1,
 		TILE_XY(-6,1),  1, 1, 1, 1, 1, 1,
 		0,
-	};	
+	};
 	int j;
 	const TileIndexDiff *p = _count_square_table;
 	int num = 0;
@@ -151,7 +171,7 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 		1 << M(STR_SV_STNAME_HELIPORT),		/* 5 */
 	};
 
-	City *city = st->city;
+	Town *t = st->town;
 	uint32 free_names = (uint32)-1;
 	int found;
 	uint z,z2;
@@ -161,7 +181,7 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 		Station *s;
 
 		FOR_ALL_STATIONS(s) {
-			if (s != st && s->xy != 0 && s->city==city) {
+			if (s != st && s->xy != 0 && s->town==t) {
 				uint str = M(s->string_id);
 				if (str <= 0x20) {
 					if (str == M(STR_SV_STNAME_FOREST))
@@ -169,9 +189,9 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 					CLRBIT(free_names, str);
 				}
 			}
-		}	
+		}
 	}
-	
+
 	/* check default names */
 	tmp = free_names & _gen_station_name_bits[flag];
 	if (tmp != 0) {
@@ -191,8 +211,8 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 				}
 	}
 
-	/* check close enough to city to get central as name? */
-	if (GetTileDist1D(tile,city->xy) < 8) {
+	/* check close enough to town to get central as name? */
+	if (GetTileDist1D(tile,t->xy) < 8) {
 		found = M(STR_SV_STNAME);
 		if (HASBIT(free_names, M(STR_SV_STNAME))) goto done;
 
@@ -216,9 +236,9 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 				goto done;
 	}
 
-	/* check elevation compared to city */
+	/* check elevation compared to town */
 	z = GetTileZ(tile);
-	z2 = GetTileZ(city->xy);
+	z2 = GetTileZ(t->xy);
 	if (z < z2) {
 		found = M(STR_SV_STNAME_VALLEY);
 		if (HASBIT(free_names, M(STR_SV_STNAME_VALLEY))) goto done;
@@ -227,7 +247,7 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 		if (HASBIT(free_names, M(STR_SV_STNAME_HEIGHTS))) goto done;
 	}
 
-	/* check direction compared to city */
+	/* check direction compared to town */
 	{
 		static const int8 _direction_and_table[] = {
 			~( (1<<M(STR_SV_STNAME_WEST)) | (1<<M(STR_SV_STNAME_EAST)) | (1<<M(STR_SV_STNAME_NORTH)) ),
@@ -235,10 +255,10 @@ static bool GenerateStationName(Station *st, uint tile, int flag)
 			~( (1<<M(STR_SV_STNAME_SOUTH)) | (1<<M(STR_SV_STNAME_EAST)) | (1<<M(STR_SV_STNAME_NORTH)) ),
 			~( (1<<M(STR_SV_STNAME_SOUTH)) | (1<<M(STR_SV_STNAME_WEST)) | (1<<M(STR_SV_STNAME_EAST)) ),
 		};
-		
-		free_names &= _direction_and_table[ 
-			(GET_TILE_X(tile) < GET_TILE_X(city->xy)) + 
-			(GET_TILE_Y(tile) < GET_TILE_Y(city->xy))*2];
+
+		free_names &= _direction_and_table[
+			(GET_TILE_X(tile) < GET_TILE_X(t->xy)) +
+			(GET_TILE_Y(tile) < GET_TILE_Y(t->xy))*2];
 	}
 
 	tmp = free_names & ((1<<1)|(1<<2)|(1<<3)|(1<<4)|(1<<6)|(1<<7)|(1<<12)|(1<<26)|(1<<27)|(1<<28)|(1<<29)|(1<<30));
@@ -283,7 +303,6 @@ static void StationInitialize(Station *st, TileIndex tile)
 	st->delete_ctr = 0;
 	st->facilities = 0;
 
-	st->blocked_months = 0;
 	st->last_vehicle = INVALID_VEHICLE;
 
 	for(i=0,ge=st->goods; i!=NUM_CARGO; i++, ge++) {
@@ -294,7 +313,8 @@ static void StationInitialize(Station *st, TileIndex tile)
 		ge->last_speed = 0;
 		ge->last_age = 0xFF;
 	}
-	_station_sort_dirty = true;
+
+	_global_station_sort_dirty = true; // build a new station
 }
 
 // Update the virtual coords needed to draw the station sign.
@@ -316,7 +336,7 @@ void UpdateAllStationVirtCoord()
 	Station *st;
 	FOR_ALL_STATIONS(st) {
 		if (st->xy != 0)
-			UpdateStationVirtCoord(st);			
+			UpdateStationVirtCoord(st);
 	}
 }
 
@@ -469,7 +489,7 @@ static void UpdateStationAcceptance(Station *st, bool show_msg)
 	}
 	span[4] = st->airport_tile;
 	if (st->airport_tile != 0) {
-		span[5] = st->airport_tile + TILE_XY(_airport_size_x[st->airport_type]-1, _airport_size_y[st->airport_type]-1); 
+		span[5] = st->airport_tile + TILE_XY(_airport_size_x[st->airport_type]-1, _airport_size_y[st->airport_type]-1);
 	}
 	span[6] = st->dock_tile;
 
@@ -510,7 +530,7 @@ static void UpdateStationAcceptance(Station *st, bool show_msg)
 	new_acc = GetAcceptanceMask(st);
 	if (old_acc == new_acc)
 		return;
-	
+
 	// show a message to report that the acceptance was changed?
 	if (show_msg && st->owner == _local_player && st->facilities) {
 		uint32 accept=0, reject=0; /* these contain two string ids each */
@@ -518,7 +538,7 @@ static void UpdateStationAcceptance(Station *st, bool show_msg)
 
 		do {
 			if (new_acc & 1) {
-				if (!(old_acc & 1)) accept = (accept << 16) | *str; 
+				if (!(old_acc & 1)) accept = (accept << 16) | *str;
 			} else {
 				if (old_acc & 1) reject = (reject << 16) | *str;
 			}
@@ -558,7 +578,7 @@ int32 CheckFlatLandBelow(uint tile, uint w, uint h, uint flags, uint invalid_dir
 		tileh = GetTileSlope(tile_cur, &z);
 
 		// steep slopes are completely prohibited
-		if (tileh & 0x10 || ((_is_ai_player || !_patches.build_on_slopes) && tileh != 0)) {
+		if (tileh & 0x10 || (((!_patches.ainew_active && _is_ai_player) || !_patches.build_on_slopes) && tileh != 0)) {
 			_error_message = STR_0007_FLAT_LAND_REQUIRED;
 			return CMD_ERROR;
 		}
@@ -576,7 +596,7 @@ int32 CheckFlatLandBelow(uint tile, uint w, uint h, uint flags, uint invalid_dir
 			cost += _price.terraform;
 			flat_z += 8;
 		}
-		
+
 		// get corresponding flat level and make sure that all parts of the station have the same level.
 		if (allowed_z == -1) {
 			// first tile
@@ -586,7 +606,7 @@ int32 CheckFlatLandBelow(uint tile, uint w, uint h, uint flags, uint invalid_dir
 			return CMD_ERROR;
 		}
 
-		// if station is set, then we have special handling to allow building on top of already existing stations. 
+		// if station is set, then we have special handling to allow building on top of already existing stations.
 		// so station points to -1 if we can build on any station. or it points to a station if we're only allowed to build
 		// on exactly that station.
 		if (station && IS_TILETYPE(tile_cur, MP_STATION)) {
@@ -649,7 +669,7 @@ static bool CanExpandRailroadStation(Station *st, uint *fin, int direction)
 			return false;
 	}
 	// make sure the final size is not too big.
-	if (curw > 15 || curh > 15) return false;
+	if (curw > _patches.station_spread || curh > _patches.station_spread) return false;
 
 	// now tile contains the new value for st->train_tile
 	// curw, curh contain the new value for width and height
@@ -719,7 +739,7 @@ int32 CmdBuildRailroadStation(int x_org, int y_org, uint32 flags, uint32 p1, uin
 	tile_org = TILE_FROM_XY(x_org,y_org);
 
 	/* Does the authority allow this? */
-	if (!(flags & DC_NO_CITY_RATING) && !CheckIfAuthorityAllows(tile_org))
+	if (!(flags & DC_NO_TOWN_RATING) && !CheckIfAuthorityAllows(tile_org))
 		return CMD_ERROR;
 
 	{
@@ -763,8 +783,8 @@ int32 CmdBuildRailroadStation(int x_org, int y_org, uint32 flags, uint32 p1, uin
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
 
 		if (st->train_tile != 0) {
-			// check if we want to expanding an already existing station? Only human players can do this.
-			if (_is_ai_player || !_patches.join_stations || !CanExpandRailroadStation(st, finalvalues, direction))
+			// check if we want to expanding an already existing station?
+			if ((!_patches.ainew_active && _is_ai_player) || !_patches.join_stations || !CanExpandRailroadStation(st, finalvalues, direction))
 				return_cmd_error(STR_3005_TOO_CLOSE_TO_ANOTHER_RAILROAD);
 		}
 
@@ -776,10 +796,10 @@ int32 CmdBuildRailroadStation(int x_org, int y_org, uint32 flags, uint32 p1, uin
 		st = AllocateStation();
 		if (st == NULL)
 			return CMD_ERROR;
-		
-		st->city = ClosestCityFromTile(tile_org, (uint)-1);
+
+		st->town = ClosestTownFromTile(tile_org, (uint)-1);
 		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
-			SETBIT(st->city->have_ratings, _current_player);
+			SETBIT(st->town->have_ratings, _current_player);
 
 		if (!GenerateStationName(st, tile_org, 0))
 			return CMD_ERROR;
@@ -800,18 +820,20 @@ int32 CmdBuildRailroadStation(int x_org, int y_org, uint32 flags, uint32 p1, uin
 
 		st->trainst_w = finalvalues[1];
 		st->trainst_h = finalvalues[2];
-		
+
+		st->build_date = _date;
+
 		tile_delta = direction ? TILE_XY(0,1) : TILE_XY(1,0);
-		
+
 		layout_ptr = alloca(numtracks * plat_len);
 		GetStationLayout(layout_ptr, numtracks, plat_len);
-		
+
 		do {
 			int tile = tile_org;
 			int w = plat_len;
 			do {
 
-				ModifyTile(tile, 
+				ModifyTile(tile,
 					MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT |
 					MP_MAP2 | MP_MAP5 | MP_MAP3LO | MP_MAP3HI_CLEAR,
 					station_index, /* map2 parameter */
@@ -845,7 +867,7 @@ static void MakeRailwayStationAreaSmaller(Station *st)
 	uint i;
 
 restart:
-	
+
 	// too small?
 	if (w != 0 && h != 0) {
 		// check the left side, x = constant, y changes
@@ -885,7 +907,7 @@ int32 CmdRemoveFromRailroadStation(int x, int y, uint32 flags, uint32 p1, uint32
 	// make sure the specified tile belongs to the current player, and that it is a railroad station.
 	if (!IS_TILETYPE(tile, MP_STATION) || _map5[tile] >= 8 || !_patches.nonuniform_stations) return CMD_ERROR;
 	st = DEREF_STATION(_map2[tile]);
-	if (!CheckOwnership(st->owner) || !EnsureNoVehicle(tile)) return CMD_ERROR;
+	if (_current_player != OWNER_WATER && (!CheckOwnership(st->owner) || !EnsureNoVehicle(tile))) return CMD_ERROR;
 
 	// if we reached here, it means we can actually delete it. do that.
 	if (flags & DC_EXEC) {
@@ -928,12 +950,15 @@ uint GetStationPlatforms(Station *st, uint tile)
 }
 
 
-static int32 RemoveRailroadStation(Station *st, uint32 flags)
+static int32 RemoveRailroadStation(Station *st, TileIndex tile, uint32 flags)
 {
-	uint tile;
 	int w,h;
 	int32 cost;
 
+	/* if there is flooding and non-uniform stations are enabled, remove platforms tile by tile */
+	if (_current_player == OWNER_WATER && _patches.nonuniform_stations)
+		return DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_REMOVE_FROM_RAILROAD_STATION);
+
 	/* Current player owns the station? */
 	if (_current_player != OWNER_WATER && !CheckOwnership(st->owner))
 		return CMD_ERROR;
@@ -942,7 +967,7 @@ static int32 RemoveRailroadStation(Station *st, uint32 flags)
 	tile = st->train_tile;
 	w = st->trainst_w;
 	h = st->trainst_h;
-	
+
 	assert(w != 0 && h != 0);
 
 	/* cost is area * constant */
@@ -972,7 +997,7 @@ static int32 RemoveRailroadStation(Station *st, uint32 flags)
 		UpdateStationVirtCoordDirty(st);
 		DeleteStationIfEmpty(st);
 	}
-	
+
 	return cost;
 }
 
@@ -1013,7 +1038,7 @@ int32 CmdBuildBusStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	tile = TILE_FROM_XY(x,y);
 
-	if (!(flags & DC_NO_CITY_RATING) && !CheckIfAuthorityAllows(tile))
+	if (!(flags & DC_NO_TOWN_RATING) && !CheckIfAuthorityAllows(tile))
 		return CMD_ERROR;
 
 	if ((cost=CheckFlatLandBelow(tile, 1, 1, flags, 1 << p1, NULL)) == CMD_ERROR)
@@ -1032,23 +1057,23 @@ int32 CmdBuildBusStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (st != NULL) {
 		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
-		
+
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
 			return CMD_ERROR;
 
 		if (st->bus_tile != 0)
 			return_cmd_error(STR_3044_TOO_CLOSE_TO_ANOTHER_BUS);
 	} else {
-		City *cit;
+		Town *t;
 
 		st = AllocateStation();
 		if (st == NULL)
 			return CMD_ERROR;
 
-		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
+		st->town = t = ClosestTownFromTile(tile, (uint)-1);
 
 		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
-			SETBIT(cit->have_ratings, _current_player);
+			SETBIT(t->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
 
@@ -1068,7 +1093,9 @@ int32 CmdBuildBusStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->bus_stop_status = 3;
 		st->owner = _current_player;
 
-		ModifyTile(tile, 
+		st->build_date = _date;
+
+		ModifyTile(tile,
 			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT |
 			MP_MAP2 | MP_MAP5 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR,
 			st->index,			/* map2 parameter */
@@ -1091,7 +1118,7 @@ static int32 RemoveBusStation(Station *st, uint32 flags)
 		return CMD_ERROR;
 
 	tile = st->bus_tile;
-	
+
 	if (!EnsureNoVehicle(tile))
 		return CMD_ERROR;
 
@@ -1105,7 +1132,7 @@ static int32 RemoveBusStation(Station *st, uint32 flags)
 		DeleteStationIfEmpty(st);
 	}
 
-	return _price.remove_bus_station;	
+	return _price.remove_bus_station;
 }
 
 
@@ -1123,7 +1150,7 @@ int32 CmdBuildTruckStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	tile = TILE_FROM_XY(x,y);
 
-	if (!(flags & DC_NO_CITY_RATING) && !CheckIfAuthorityAllows(tile))
+	if (!(flags & DC_NO_TOWN_RATING) && !CheckIfAuthorityAllows(tile))
 		return CMD_ERROR;
 
 	if ((cost=CheckFlatLandBelow(tile, 1, 1, flags, 1 << p1, NULL)) == CMD_ERROR)
@@ -1142,23 +1169,23 @@ int32 CmdBuildTruckStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (st != NULL) {
 		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
-		
+
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
 			return CMD_ERROR;
 
 		if (st->lorry_tile != 0)
 			return_cmd_error(STR_3045_TOO_CLOSE_TO_ANOTHER_TRUCK);
 	} else {
-		City *cit;
+		Town *t;
 
 		st = AllocateStation();
 		if (st == NULL)
 			return CMD_ERROR;
 
-		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
+		st->town = t = ClosestTownFromTile(tile, (uint)-1);
 
 		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
-			SETBIT(cit->have_ratings, _current_player);
+			SETBIT(t->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
 
@@ -1178,8 +1205,10 @@ int32 CmdBuildTruckStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->truck_stop_status = 3;
 		st->owner = _current_player;
 
-		ModifyTile(tile, 
-			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT | 
+		st->build_date = _date;
+
+		ModifyTile(tile,
+			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT |
 			MP_MAP2 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR | MP_MAP5,
 			st->index,			/* map2 parameter */
 			p1 + 0x43       /* map5 parameter */
@@ -1199,7 +1228,7 @@ static int32 RemoveTruckStation(Station *st, uint32 flags)
 
 	if (_current_player != OWNER_WATER && !CheckOwnership(st->owner))
 		return CMD_ERROR;
-	
+
 	tile = st->lorry_tile;
 
 	if (!EnsureNoVehicle(tile))
@@ -1207,7 +1236,7 @@ static int32 RemoveTruckStation(Station *st, uint32 flags)
 
 	if (flags & DC_EXEC) {
 		DoClearSquare(tile);
-		
+
 		st->lorry_tile = 0;
 		st->facilities &= ~FACIL_TRUCK_STOP;
 
@@ -1215,7 +1244,7 @@ static int32 RemoveTruckStation(Station *st, uint32 flags)
 		DeleteStationIfEmpty(st);
 	}
 
-	return _price.remove_truck_station;	
+	return _price.remove_truck_station;
 }
 
 // FIXME -- need to move to its corresponding Airport variable
@@ -1227,7 +1256,7 @@ static const byte _airport_map5_tiles_country[] = {
 };
 
 // City Airport (large)
-static const byte _airport_map5_tiles_city[] = {
+static const byte _airport_map5_tiles_town[] = {
 	31, 9, 33, 9, 9, 32,
 	27, 36, 29, 34, 8, 10,
 	30, 11, 35, 13, 20, 21,
@@ -1264,9 +1293,9 @@ static const byte _airport_map5_tiles_heliport[] = {
 
 static const byte * const _airport_map5_tiles[] = {
 	_airport_map5_tiles_country,				// Country Airfield (small)
-	_airport_map5_tiles_city,						// City Airport (large)
+	_airport_map5_tiles_town,						// City Airport (large)
 	_airport_map5_tiles_heliport,				// Heliport
-	_airport_map5_tiles_metropolitan,   // Metropolitain Airport (large) 
+	_airport_map5_tiles_metropolitan,   // Metropolitain Airport (large)
 	_airport_map5_tiles_international,	// International Airport (xlarge)
 };
 
@@ -1277,30 +1306,30 @@ static const byte * const _airport_map5_tiles[] = {
 int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	uint tile;
-	City *city;
+	Town *t;
 	Station *st;
 	int32 cost;
 	int w,h;
-	bool airport_upgrade = true;	
+	bool airport_upgrade = true;
 
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	tile = TILE_FROM_XY(x,y);
-	
-	if (!(flags & DC_NO_CITY_RATING) && !CheckIfAuthorityAllows(tile))
+
+	if (!(flags & DC_NO_TOWN_RATING) && !CheckIfAuthorityAllows(tile))
 		return CMD_ERROR;
 
-	city = ClosestCityFromTile(tile, (uint)-1);
+	t = ClosestTownFromTile(tile, (uint)-1);
 
 	/* Check if local auth refuses a new airport */
 	{
 		uint num = 0;
 		FOR_ALL_STATIONS(st) {
-			if (st->xy != 0 && st->city == city && st->facilities&FACIL_AIRPORT && st->airport_type != AT_OILRIG)
+			if (st->xy != 0 && st->town == t && st->facilities&FACIL_AIRPORT && st->airport_type != AT_OILRIG)
 				num++;
 		}
 		if (num >= 2) {
-			SET_DPARAM16(0, city->index);
+			SET_DPARAM16(0, t->index);
 			return_cmd_error(STR_2035_LOCAL_AUTHORITY_REFUSES);
 		}
 	}
@@ -1325,14 +1354,14 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (st != NULL) {
 		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
-		
+
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
 			return CMD_ERROR;
 
 		if (st->airport_tile != 0)
 			return_cmd_error(STR_300D_TOO_CLOSE_TO_ANOTHER_AIRPORT);
 	} else {
-		City *cit;
+		Town *t;
 
 		airport_upgrade = false;
 
@@ -1340,10 +1369,10 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		if (st == NULL)
 			return CMD_ERROR;
 
-		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
+		st->town = t = ClosestTownFromTile(tile, (uint)-1);
 
 		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
-			SETBIT(cit->have_ratings, _current_player);
+			SETBIT(t->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
 
@@ -1370,6 +1399,8 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->airport_type = (byte)p1;
 		st->airport_flags = 0;
 
+		st->build_date = _date;
+
 		/*	if airport was demolished while planes were en-route to it, the positions can no longer
 				be the same (v->u.air.pos), since different airports have different indexes. So update
 				all planes en-route to this airport. Only update if
@@ -1377,11 +1408,11 @@ int32 CmdBuildAirport(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 				2. airport is added to existing station (unfortunately unavoideable)
 		*/
 		if (airport_upgrade) {UpdateAirplanesOnNewStation(st);}
-	
+
 		{
 			const byte *b = _airport_map5_tiles[p1];
 BEGIN_TILE_LOOP(tile_cur,w,h,tile)
-				ModifyTile(tile_cur, 
+				ModifyTile(tile_cur,
 					MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT |
 					MP_MAP2 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR | MP_MAP5,
 					st->index, *b++);
@@ -1437,7 +1468,7 @@ END_TILE_LOOP(tile_cur, w,h,tile)
 	return cost;
 }
 
-/* Build a buoy 
+/* Build a buoy
  * p1,p2 unused
  */
 
@@ -1457,7 +1488,7 @@ int32 CmdBuildBuoy(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (st == NULL)
 		return CMD_ERROR;
 
-	st->city = ClosestCityFromTile(ti.tile, (uint)-1);
+	st->town = ClosestTownFromTile(ti.tile, (uint)-1);
 	st->sign.width_1 = 0;
 
 	if (!GenerateStationName(st, ti.tile, 4))
@@ -1470,6 +1501,8 @@ int32 CmdBuildBuoy(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->had_vehicle_of_type |= HVOT_BUOY;
 		st->owner = OWNER_NONE;
 
+		st->build_date = _date;
+
 		ModifyTile(ti.tile,
 			MP_SETTYPE(MP_STATION) |
 			MP_MAP2 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR | MP_MAPOWNER | MP_MAP5,
@@ -1479,7 +1512,7 @@ int32 CmdBuildBuoy(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		);
 
 		UpdateStationVirtCoordDirty(st);
-		
+
 		UpdateStationAcceptance(st, false);
 		InvalidateWindow(WC_STATION_LIST, st->owner);
 	}
@@ -1506,7 +1539,7 @@ static int32 RemoveBuoy(Station *st, uint32 flags)
 		st->facilities &= ~FACIL_DOCK;
 		st->had_vehicle_of_type &= ~HVOT_BUOY;
 
-		ModifyTile(tile, 
+		ModifyTile(tile,
 			MP_SETTYPE(MP_WATER) |
 			MP_MAP2_CLEAR | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR | MP_MAPOWNER | MP_MAP5 | MP_MAP2_CLEAR,
 			OWNER_WATER, /* map_owner */
@@ -1570,13 +1603,13 @@ int32 CmdBuildDock(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	FindLandscapeHeightByTile(&ti, tile_cur);
 	if (ti.tileh != 0 || ti.type != MP_WATER)
 		return_cmd_error(STR_304B_SITE_UNSUITABLE);
-	
+
 	/* middle */
-	st = GetStationAround(tile + _dock_tileoffs_chkaround[direction], 
+	st = GetStationAround(tile + _dock_tileoffs_chkaround[direction],
 		_dock_w_chk[direction], _dock_h_chk[direction], -1);
 	if (st == CHECK_STATIONS_ERR)
 		return CMD_ERROR;
-	
+
 	/* Find a station close to us */
 	if (st == NULL) {
 		st = GetClosestStationFromTile(tile, 8, _current_player);
@@ -1586,23 +1619,23 @@ int32 CmdBuildDock(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (st != NULL) {
 		if (st->owner != OWNER_NONE && st->owner != _current_player)
 			return_cmd_error(STR_3009_TOO_CLOSE_TO_ANOTHER_STATION);
-		
+
 		if (!CheckStationSpreadOut(st, tile, 1, 1))
 			return CMD_ERROR;
 
 		if (st->dock_tile != 0)
 			return_cmd_error(STR_304C_TOO_CLOSE_TO_ANOTHER_DOCK);
 	} else {
-		City *cit;
+		Town *t;
 
 		st = AllocateStation();
 		if (st == NULL)
 			return CMD_ERROR;
 
-		st->city = cit = ClosestCityFromTile(tile, (uint)-1);
+		st->town = t = ClosestTownFromTile(tile, (uint)-1);
 
 		if (_current_player < MAX_PLAYERS && flags&DC_EXEC)
-			SETBIT(cit->have_ratings, _current_player);
+			SETBIT(t->have_ratings, _current_player);
 
 		st->sign.width_1 = 0;
 
@@ -1619,20 +1652,22 @@ int32 CmdBuildDock(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		st->facilities |= FACIL_DOCK;
 		st->owner = _current_player;
 
-		ModifyTile(tile, 
-			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT | 
+		st->build_date = _date;
+
+		ModifyTile(tile,
+			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT |
 			MP_MAP2 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR |
 			MP_MAP5,
 			st->index,
 			direction + 0x4C);
 
-		ModifyTile(tile + _tileoffs_by_dir[direction], 
-			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT | 
+		ModifyTile(tile + _tileoffs_by_dir[direction],
+			MP_SETTYPE(MP_STATION) | MP_MAPOWNER_CURRENT |
 			MP_MAP2 | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR |
 			MP_MAP5,
 			st->index,
 			(direction&1) + 0x50);
-		
+
 		UpdateStationVirtCoordDirty(st);
 		UpdateStationAcceptance(st, false);
 		InvalidateWindow(WC_STATION_LIST, st->owner);
@@ -1709,27 +1744,27 @@ static void DrawTile_Station(TileInfo *ti)
 
 	t = _station_display_datas[ti->map5];
 
-	image = *(uint32*)t;
+	image = *(const uint32*)t;
 	t += sizeof(uint32);
 	if (image & 0x8000)
 		image |= image_or_modificator;
-	DrawGroundSprite(image + base_img);	
+	DrawGroundSprite(image + base_img);
 
-	for(dtss = (DrawTileSeqStruct *)t; (byte)dtss->delta_x != 0x80; dtss++) {
+	for(dtss = (const DrawTileSeqStruct *)t; (byte)dtss->delta_x != 0x80; dtss++) {
 		if ((byte)dtss->delta_z != 0x80) {
 			image =	dtss->image + base_img;
 			if (_display_opt & DO_TRANS_BUILDINGS) {
-				if (image&0x8000) image |= image_or_modificator;	
+				if (image&0x8000) image |= image_or_modificator;
 			} else {
 				image = (image & 0x3FFF) | 0x03224000;
 			}
 
 			AddSortableSpriteToDraw(image, ti->x + dtss->delta_x, ti->y + dtss->delta_y, dtss->width, dtss->height, dtss->unk, ti->z + dtss->delta_z);
 		} else {
-			image = *(uint32*)&dtss->height + base_img; /* endian ok */
+			image = *(const uint32*)&dtss->height + base_img; /* endian ok */
 
 			if (_display_opt & DO_TRANS_BUILDINGS) {
-				if (image&0x8000) image |= image_or_modificator;	
+				if (image&0x8000) image |= image_or_modificator;
 			} else {
 				image = (image & 0x3FFF) | 0x03224000;
 			}
@@ -1751,13 +1786,13 @@ void StationPickerDrawSprite(int x, int y, int railtype, int image)
 
 	t = _station_display_datas[image];
 
-	img = *(uint32*)t;
+	img = *(const uint32*)t;
 	t += sizeof(uint32);
 	if (img & 0x8000)
 		img |= ormod;
 	DrawSprite(img, x, y);
 
-	for(dtss = (DrawTileSeqStruct *)t; (byte)dtss->delta_x != 0x80; dtss++) {
+	for(dtss = (const DrawTileSeqStruct *)t; (byte)dtss->delta_x != 0x80; dtss++) {
 		Point pt = RemapCoords(dtss->delta_x, dtss->delta_y, dtss->delta_z);
 		DrawSprite((dtss->image | ormod) + railtype, x + pt.x, y + pt.y);
 	}
@@ -1771,6 +1806,11 @@ static uint GetSlopeZ_Station(TileInfo *ti)
 	return z;
 }
 
+static uint GetSlopeTileh_Station(TileInfo *ti)
+{
+	return 0;
+}
+
 static void GetAcceptedCargo_Station(uint tile, AcceptedCargo *ac)
 {
 	/* not used */
@@ -1782,6 +1822,7 @@ static void GetTileDesc_Station(uint tile, TileDesc *td)
 	StringID str;
 
 	td->owner = _map_owner[tile];
+	td->build_date = DEREF_STATION(_map2[tile])->build_date;
 
 	m5 = _map5[tile];
 	(str=STR_305E_RAILROAD_STATION, m5 < 8) ||
@@ -1798,21 +1839,34 @@ static void GetTileDesc_Station(uint tile, TileDesc *td)
 
 static const byte _tile_track_status_rail[8] = { 1,2,1,2,1,2,1,2 };
 
-static uint32 GetTileTrackStatus_Station(uint tile, int mode) {
+static uint32 GetTileTrackStatus_Station(uint tile, TransportType mode) {
 	uint i = _map5[tile];
 	uint j = 0;
 
-	if (mode == 0) {
+	if (mode == TRANSPORT_RAIL) {
 		if (i < 8)
 			j = _tile_track_status_rail[i];
-	} else if (mode == 2) {
-		// not needed
-	} else if (mode == 4) {
-		// buoy
-		if (i == 0x52) j = 0x3F;
-	}
+		j += (j << 8);
+	} else if (mode == TRANSPORT_ROAD) {
+		Station *st = DEREF_STATION(_map2[tile]);
+		if ( (IS_BYTE_INSIDE(i, 0x43, 0x47) && (_patches.roadveh_queue || st->truck_stop_status&3)) ||
+		(IS_BYTE_INSIDE(i, 0x47, 0x4B) && (_patches.roadveh_queue || st->bus_stop_status&3)) ) {
+			/* This is a bus/truck stop, and there is free space
+			 * (or we allow queueing) */
 
-	return j + (j << 8);
+			/* We reverse the dir because it points out of the
+			 * exit, and we want to get in. Maybe we should return
+			 * both dirs here? */
+			byte dir = _reverse_dir[(i-0x43)&3];
+			j = 1 << _dir_to_straight_trackdir[dir];
+		}
+	} else if (mode == TRANSPORT_WATER) {
+		// buoy is coded as a station, it is always on open water
+		// (0x3F, all tracks available)
+		if (i == 0x52) j = 0x3F;
+		j += (j << 8);
+	}
+	return j;
 }
 
 static void TileLoop_Station(uint tile)
@@ -1825,10 +1879,15 @@ static void TileLoop_Station(uint tile)
   // 0x3A - flag small airport (58)
 	if (_map5[tile] == 39 || _map5[tile] == 58 || _map5[tile] == 90 || _map5[tile] == 102)
 		AddAnimatedTile(tile);
-	
+
 	// treat a bouy tile as water.
 	else if (_map5[tile] == 0x52)
 		TileLoop_Water(tile);
+
+	// treat a oilrig (the station part) as water
+	else if (_map5[tile] == 0x4B)
+		TileLoop_Water(tile);
+
 }
 
 
@@ -1842,7 +1901,7 @@ static void AnimateTile_Station(uint tile)
 	if (m5 >= 39 && m5 <= 50) { // turning radar (39 - 50)
 		if (_tick_counter & 3)
 			return;
-		
+
 		if (++m5 == 50+1)
 			m5 = 39;
 
@@ -1852,14 +1911,14 @@ static void AnimateTile_Station(uint tile)
 	} else if (m5 >= 90 && m5 <= 113) { // turning radar with ground under it (different fences) (90 - 101 | 102 - 113)
 		if (_tick_counter & 3)
 			return;
-		
+
 		m5++;
-		
+
 		if (m5 == 101+1) {m5 = 90;}  // radar with fences in south
 		else if (m5 == 113+1) {m5 = 102;} // radar with fences in north
 
 		_map5[tile] = m5;
-		MarkTileDirtyByTile(tile);		
+		MarkTileDirtyByTile(tile);
 	//added - end
 	} else if (m5 >= 0x3A && m5 <= 0x3D) {  // windsack (58 - 61)
 		if (_tick_counter & 1)
@@ -1867,7 +1926,7 @@ static void AnimateTile_Station(uint tile)
 
 		if (++m5 == 0x3D+1)
 			m5 = 0x3A;
-		
+
 		_map5[tile] = m5;
 		MarkTileDirtyByTile(tile);
 	}
@@ -1899,17 +1958,17 @@ static uint32 VehicleEnter_Station(Vehicle *v, uint tile, int x, int y)
 	uint16 spd;
 
 	if (v->type == VEH_Train) {
-		if (IS_BYTE_INSIDE(_map5[tile], 0, 8) && v->subtype == 0 && 
+		if (IS_BYTE_INSIDE(_map5[tile], 0, 8) && v->subtype == 0 &&
 			!IsTrainStationTile(tile + _tileoffs_by_dir[v->direction >> 1])) {
-			
+
 			station_id = _map2[tile];
-			if ((!(v->next_order & OF_NON_STOP) && !_patches.new_nonstop) || 
+			if ((!(v->next_order & OF_NON_STOP) && !_patches.new_nonstop) ||
 					 (((v->next_order & OT_MASK) == OT_GOTO_STATION && v->next_order_param == station_id))) {
 
 				if (!(_patches.new_nonstop && (v->next_order & OF_NON_STOP)) && v->next_order != OT_LEAVESTATION && v->last_station_visited != station_id) {
 					x &= 0xF;
 					y &= 0xF;
-					
+
 					dir = v->direction & 6;
 					if (dir & 2) intswap(x,y);
 					if (y == 8) {
@@ -1925,7 +1984,7 @@ static uint32 VehicleEnter_Station(Vehicle *v, uint tile, int x, int y)
 						}
 					}
 				}
-			}	
+			}
 		}
 	} else if (v->type == VEH_Road) {
 		if (v->u.road.state < 16 && (v->u.road.state&4)==0 && v->u.road.frame==0) {
@@ -1957,7 +2016,7 @@ static uint32 VehicleEnter_Station(Vehicle *v, uint tile, int x, int y)
 			}
 		}
 	}
-	
+
 	return 0;
 }
 
@@ -1968,7 +2027,8 @@ static void DeleteStation(Station *st)
 
 	DeleteName(st->string_id);
 	MarkStationDirty(st);
-	_station_sort_dirty = true;
+	_global_station_sort_dirty = true; // delete station, remove sign
+	InvalidateWindowClasses(WC_STATION_LIST);
 
 	index = st->index;
 	DeleteWindowById(WC_STATION_VIEW, index);
@@ -1982,7 +2042,7 @@ void DeleteAllPlayerStations()
 
 	FOR_ALL_STATIONS(st) {
 		if (st->xy && st->owner < MAX_PLAYERS)
-			DeleteStation(st);		
+			DeleteStation(st);
 	}
 }
 
@@ -2038,7 +2098,7 @@ static void UpdateStationRating(Station *st)
 							rating += _rating_boost[_opt.diff.competitor_intelligence];
 			}
 
-			if (st->owner < 8 && HASBIT(st->city->statues, st->owner))
+			if (st->owner < 8 && HASBIT(st->town->statues, st->owner))
 				rating += 26;
 
 			{
@@ -2052,7 +2112,7 @@ static void UpdateStationRating(Station *st)
 				(rating += 45, days > 3) ||
 				(rating += 35, true);
 			}
-			
+
 			{
 				waiting = ge->waiting_acceptance & 0xFFF;
 				(rating -= 90, waiting > 1500) ||
@@ -2068,7 +2128,7 @@ static void UpdateStationRating(Station *st)
 
 				// only modify rating in steps of -2, -1, 0, 1 or 2
 				ge->rating = rating = or + clamp(clamp(rating, 0, 255) - or, -2, 2);
-				
+
 				// if rating is <= 64 and more than 200 items waiting, remove some random amount of goods from the station
 				if (rating <= 64 && waiting >= 200) {
 					int dec = Random() & 0x1F;
@@ -2091,7 +2151,7 @@ static void UpdateStationRating(Station *st)
 			}
 		}
 	} while (++ge != endof(st->goods));
-	
+
 	index = st->index;
 
 	if (waiting_changed)
@@ -2141,13 +2201,6 @@ void OnTick_Station()
 
 void StationMonthlyLoop()
 {
-	Station *st;
-
-	FOR_ALL_STATIONS(st) {
-		if (st->blocked_months != 0)
-			st->blocked_months--;
-	}
-
 }
 
 
@@ -2171,8 +2224,8 @@ void ModifyStationRatingAround(TileIndex tile, byte owner, int amount, uint radi
 
 static void UpdateStationWaiting(Station *st, int type, uint amount)
 {
-	st->goods[type].waiting_acceptance = 
-		(st->goods[type].waiting_acceptance & ~0xFFF) + 
+	st->goods[type].waiting_acceptance =
+		(st->goods[type].waiting_acceptance & ~0xFFF) +
 			min(0xFFF, (st->goods[type].waiting_acceptance & 0xFFF) + amount);
 
 	st->goods[type].enroute_time = 0;
@@ -2188,14 +2241,14 @@ int32 CmdRenameStation(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	str = AllocateName((byte*)_decode_parameters, 6);
 	if (str == 0)
 		return CMD_ERROR;
-	
+
 	if (flags & DC_EXEC) {
 		st = DEREF_STATION(p1);
 		old_str = st->string_id;
 		st->string_id = str;
 		UpdateStationVirtCoord(st);
 		DeleteName(old_str);
-		_station_sort_dirty = true;
+		_station_sort_dirty[st->owner] = true; // rename a station
 		MarkWholeScreenDirty();
 	} else {
 		DeleteName(str);
@@ -2229,12 +2282,12 @@ uint MoveGoodsToStation(uint tile, int w, int h, int type, uint amount)
 				if (around[i] == 0xFF) {
 					st = DEREF_STATION(st_index);
 					if ((st->had_vehicle_of_type & HVOT_BUOY) == 0 &&
-							st->blocked_months == 0 &&
-							st->goods[type].rating != 0 && 
+							( !st->town->exclusive_counter || (st->town->exclusivity == st->owner) ) && // check exclusive transport rights
+							st->goods[type].rating != 0 &&
 							(!_patches.selectgoods || st->goods[type].last_speed) && // if last_speed is 0, no vehicle has been there.
-							((st->facilities & (byte)~FACIL_BUS_STOP)!=0 || type==CT_PASSENGERS) &&
-							((st->facilities & (byte)~FACIL_TRUCK_STOP)!=0 || type!=CT_PASSENGERS)) {
-						
+							((st->facilities & (byte)~FACIL_BUS_STOP)!=0 || type==CT_PASSENGERS) && // if we have other fac. than a bus stop, or the cargo is passengers
+							((st->facilities & (byte)~FACIL_TRUCK_STOP)!=0 || type!=CT_PASSENGERS)) { // if we have other fac. than a cargo bay or the cargo is not passengers
+
 						around[i] = st_index;
 						around_ptr[i] = st;
 					}
@@ -2268,10 +2321,10 @@ uint MoveGoodsToStation(uint tile, int w, int h, int type, uint amount)
 			st1 = around_ptr[i];
 		} else if (around_ptr[i]->goods[type].rating >= best_rating2) {
 			best_rating2 = around_ptr[i]->goods[type].rating;
-			st2 = around_ptr[i];			
+			st2 = around_ptr[i];
 		}
 	}
-	
+
 	assert(st1 != NULL);
 	assert(st2 != NULL);
 	assert(best_rating != 0 || best_rating2 != 0);
@@ -2286,7 +2339,7 @@ uint MoveGoodsToStation(uint tile, int w, int h, int type, uint amount)
 	if (t != 0) {
 		moved = (t * best_rating >> 8) + 1;
 		amount -= t;
-		UpdateStationWaiting(st1, type, moved); 
+		UpdateStationWaiting(st1, type, moved);
 	}
 
 	assert(amount >= 0);
@@ -2306,7 +2359,7 @@ void BuildOilRig(uint tile)
 
 	FOR_ALL_STATIONS(st) {
 		if (st->xy == 0) {
-			st->city = ClosestCityFromTile(tile, (uint)-1);
+			st->town = ClosestTownFromTile(tile, (uint)-1);
 			st->sign.width_1 = 0;
 			if (!GenerateStationName(st, tile, 2))
 				return;
@@ -2319,7 +2372,7 @@ void BuildOilRig(uint tile)
 			_map3_hi[tile] = 0;
 			_map2[tile] = st->index;
 
-			st->owner = 0x10;
+			st->owner = OWNER_NONE;
       st->airport_flags = 0;
 			st->airport_type = AT_OILRIG;
 			st->xy = tile;
@@ -2334,6 +2387,7 @@ void BuildOilRig(uint tile)
 			st->delete_ctr = 0;
 			st->last_vehicle = INVALID_VEHICLE;
 			st->facilities = FACIL_AIRPORT | FACIL_DOCK;
+			st->build_date = _date;
 			for(j=0; j!=NUM_CARGO; j++) {
 				st->goods[j].waiting_acceptance = 0;
 				st->goods[j].days_since_pickup = 0;
@@ -2345,7 +2399,7 @@ void BuildOilRig(uint tile)
 
 			UpdateStationVirtCoordDirty(st);
 			UpdateStationAcceptance(st, false);
-			return;		
+			return;
 		}
 	}
 }
@@ -2373,7 +2427,8 @@ static void ChangeTileOwner_Station(uint tile, byte old_player, byte new_player)
 		Station *st = DEREF_STATION(_map2[tile]);
 		_map_owner[tile] = new_player;
 		st->owner = new_player;
-		_station_sort_dirty = true;
+		_global_station_sort_dirty = true; // transfer ownership of station to another player
+		InvalidateWindowClasses(WC_STATION_LIST);
 	} else {
 		DoCommandByTile(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 	}
@@ -2397,15 +2452,15 @@ static int32 ClearTile_Station(uint tile, byte flags) {
 	st = DEREF_STATION(_map2[tile]);
 
 	if (m5 < 8)
-		return RemoveRailroadStation(st, flags);
+		return RemoveRailroadStation(st, tile, flags);
 
 	// original airports < 67, new airports between 83 - 114
 	if (m5 < 0x43 || ( m5 >= 83 && m5 <= 114) )
 		return RemoveAirport(st, flags);
-		
+
 	if (m5 < 0x47)
 		return RemoveTruckStation(st, flags);
-	
+
 	if (m5 < 0x4B)
 		return RemoveBusStation(st, flags);
 
@@ -2422,11 +2477,16 @@ static int32 ClearTile_Station(uint tile, byte flags) {
 void InitializeStations()
 {
 	int i;
+	
 	memset(_stations, 0, sizeof(_stations));
-	for(i=0;i!=lengthof(_stations); i++)
+	for(i = 0; i != lengthof(_stations); i++)
 		_stations[i].index=i;
+
 	_station_tick_ctr = 0;
-	_station_sort_dirty = true;	// set stations to be sorted on first load
+
+	// set stations to be sorted on load of savegame
+	memset(_station_sort_dirty, true, sizeof(_station_sort_dirty));
+	_global_station_sort_dirty = true; // load of savegame
 }
 
 
@@ -2444,6 +2504,7 @@ const TileTypeProcs _tile_type_station_procs = {
 	NULL,												/* get_produced_cargo_proc */
 	VehicleEnter_Station,				/* vehicle_enter_tile_proc */
 	NULL,												/* vehicle_leave_tile_proc */
+	GetSlopeTileh_Station,			/* get_slope_tileh_proc */
 };
 
 
@@ -2454,7 +2515,7 @@ static const byte _station_desc[] = {
 	SLE_VAR(Station,train_tile,			SLE_UINT16),
 	SLE_VAR(Station,airport_tile,		SLE_UINT16),
 	SLE_VAR(Station,dock_tile,			SLE_UINT16),
-	SLE_REF(Station,city,						REF_CITY),
+	SLE_REF(Station,town,						REF_TOWN),
 	SLE_VAR(Station,trainst_w,			SLE_UINT8),
 	SLE_CONDVAR(Station,trainst_h,	SLE_UINT8, 2, 255),
 
@@ -2472,7 +2533,9 @@ static const byte _station_desc[] = {
 	SLE_VAR(Station,airport_type,				SLE_UINT8),
 	SLE_VAR(Station,truck_stop_status,	SLE_UINT8),
 	SLE_VAR(Station,bus_stop_status,		SLE_UINT8),
-	SLE_VAR(Station,blocked_months,			SLE_UINT8),
+
+	// blocked_months was stored here in savegame format 0 - 4.0
+	SLE_CONDVAR(Station,blocked_months_obsolete,	SLE_UINT8, 0, 4),
 
 	SLE_CONDVAR(Station,airport_flags,			SLE_VAR_U32 | SLE_FILE_U16, 0, 2),
 	SLE_CONDVAR(Station,airport_flags,			SLE_UINT32, 3, 255),
