@@ -5,7 +5,7 @@
 #include "station.h"
 #include "gfx.h"
 #include "player.h"
-#include "city.h"
+#include "town.h"
 #include "command.h"
 
 static void StationsWndShowStationRating(int x, int y, int type, uint acceptance, int rating)
@@ -16,7 +16,7 @@ static void StationsWndShowStationRating(int x, int y, int type, uint acceptance
 
 	if (acceptance > 575)
 		acceptance = 575;
-	
+
 	acceptance = (acceptance + 7) >> 3;
 
 	/* draw cargo */
@@ -31,12 +31,12 @@ static void StationsWndShowStationRating(int x, int y, int type, uint acceptance
 	}
 
 	x -= (acceptance>>3);
-	
+
 	DrawString(x+1, y, _cargoc.names_short[type], 0x10);
 
 	/* draw green/red ratings bar */
 	GfxFillRect(x+1, y+8, x+7, y+8, 0xB8);
-	
+
 	rating = (rating >> 5);
 
 	if (rating != 0) {
@@ -44,52 +44,42 @@ static void StationsWndShowStationRating(int x, int y, int type, uint acceptance
 	}
 }
 
-// used to get a sorted list of the stations
-typedef struct StationSort {
-	uint16	index;
-	byte		owner;
-} StationSort;
-
-static StationSort _station_sort[lengthof(_stations)];
+static SortStruct _station_sort[lengthof(_stations)];
 static uint16 _num_station_sort[MAX_PLAYERS];
 
 static char _bufcache[64];
 static uint16 _last_station_idx;
 
-static int CDECL StationSorterByName(const void *a, const void *b)
+static int CDECL StationNameSorter(const void *a, const void *b)
 {
 	char buf1[64];
 	Station *st;
-	StationSort *cmp1, *cmp2;
-	cmp1 = (StationSort*)a;
-	cmp2 = (StationSort*)b;
+	const SortStruct *cmp1 = (const SortStruct*)a;
+	const SortStruct *cmp2 = (const SortStruct*)b;
 
-	// sort stations by owner, and inside owner by name
-	if (cmp1->owner == cmp2->owner) {	// if same owner, sort by name
-		st = DEREF_STATION(cmp1->index);
-		SET_DPARAM16(0, st->city->citynametype);
-		SET_DPARAM32(1, st->city->citynameparts);
-		GetString(buf1, st->string_id);
+	st = DEREF_STATION(cmp1->index);
+	SET_DPARAM16(0, st->town->townnametype);
+	SET_DPARAM32(1, st->town->townnameparts);
+	GetString(buf1, st->string_id);
 
-		if ( cmp2->index != _last_station_idx) {
-			_last_station_idx = cmp2->index;
-			st = DEREF_STATION(cmp2->index);
-			SET_DPARAM16(0, st->city->citynametype);
-			SET_DPARAM32(1, st->city->citynameparts);
-			GetString(_bufcache, st->string_id);
-		}
-
-		return strcmp(buf1, _bufcache);	// sort by name
+	if ( cmp2->index != _last_station_idx) {
+		_last_station_idx = cmp2->index;
+		st = DEREF_STATION(cmp2->index);
+		SET_DPARAM16(0, st->town->townnametype);
+		SET_DPARAM32(1, st->town->townnameparts);
+		GetString(_bufcache, st->string_id);
 	}
-	return cmp1->owner - cmp2->owner;	// sort by owner
+
+	return strcmp(buf1, _bufcache);	// sort by name
 }
 
-static void MakeSortedStationList(Window *w)
+static void GlobalSortStationList()
 {
-	Station *st;
-	uint16 n = 0;
+	const Station *st;
+	uint32 n = 0;
 	uint16 *i;
-	// reset to 0 just to be sure
+
+	// reset #-of stations to 0 because ++ is used for value-assignment
 	for (i = _num_station_sort; i != endof(_num_station_sort); i++) {*i = 0;}
 
 	FOR_ALL_STATIONS(st) {
@@ -100,36 +90,64 @@ static void MakeSortedStationList(Window *w)
 		}
 	}
 
-	// create cumulative station-ownage
+	// create cumulative station-ownership
 	// stations are stored as a cummulative index, eg 25, 41, 43. This means
 	// Player0: 25; Player1: (41-25) 16; Player2: (43-41) 2
 	for (i = &_num_station_sort[1]; i != endof(_num_station_sort); i++) {*i += *(i-1);}
 
-	_last_station_idx = 255; // used for "cache"
-	qsort(_station_sort, n, sizeof(_station_sort[0]), StationSorterByName);
-	//#ifdef _DEBUG
-	//	{printf("Resorting Stations list...\n");}
-	//#endif
+	qsort(_station_sort, n, sizeof(_station_sort[0]), GeneralOwnerSorter); // sort by owner
+
+	// since indexes are messed up after adding/removing a station, mark all lists dirty
+	memset(_station_sort_dirty, true, sizeof(_station_sort_dirty));
+	_global_station_sort_dirty = false;
+
+	DEBUG(misc, 1) ("Resorting global station list...");
+}
+
+static void MakeSortedStationList(byte owner)
+{
+	SortStruct *firstelement;
+	uint32 n = 0;
+
+	if (owner == 0) { // first element starts at 0th element and has n elements as described above
+		firstelement =	&_station_sort[0];
+		n =							_num_station_sort[0];
+	}	else { // nth element starts at the end of the previous one, and has n elements as described above
+		firstelement =	&_station_sort[_num_station_sort[owner-1]];
+		n =							_num_station_sort[owner] - _num_station_sort[owner-1];
+	}
+
+	_last_station_idx = 0; // used for "cache" in namesorting
+	qsort(firstelement, n, sizeof(_station_sort[0]), StationNameSorter); // sort by name
+
+	_station_sort_dirty[owner] = false;
+
+	DEBUG(misc, 1) ("Resorting Stations list player %d...", owner+1);
 }
 
 static void PlayerStationsWndProc(Window *w, WindowEvent *e)
 {
 	switch(e->event) {
 	case WE_PAINT: {
-		byte i;
-		if (_station_sort_dirty) {
-			_station_sort_dirty = false;
-			MakeSortedStationList(w);
+		uint32 i;
+		const byte window_number = (byte)w->window_number;
+
+		// resort station window if stations have been added/removed
+		if (_global_station_sort_dirty)
+			GlobalSortStationList();
+
+		if (_station_sort_dirty[window_number]) { // resort in case of a station rename.
+			MakeSortedStationList(window_number);
 		}
 
 		// stations are stored as a cummulative index, eg 25, 41, 43. This means
 		// Player0: 25; Player1: (41-25) 16; Player2: (43-41) 2 stations
-		i = (byte)(w->window_number == 0) ? 0 : _num_station_sort[w->window_number-1];;
-		SetVScrollCount(w, _num_station_sort[w->window_number] - i);
+		i = (window_number == 0) ? 0 : _num_station_sort[window_number-1];
+		SetVScrollCount(w, _num_station_sort[window_number] - i);
 
 		/* draw widgets, with player's name in the caption */
 		{
-			Player *p = DEREF_PLAYER(w->window_number);
+			Player *p = DEREF_PLAYER(window_number);
 			SET_DPARAM16(0, p->name_1);
 			SET_DPARAM32(1, p->name_2);
 			SET_DPARAM16(2, w->vscroll.count);
@@ -147,13 +165,15 @@ static void PlayerStationsWndProc(Window *w, WindowEvent *e)
 				DrawString(xb, y, STR_304A_NONE, 0);
 				return;
 			}
-				
-			i += w->vscroll.pos;	// offset from sorted station list of current player
-			assert(i < _num_station_sort[w->window_number]); // at least one station must exist
 
-			while (i < _num_station_sort[w->window_number]) {	// do until max number of stations of owner
+			i += w->vscroll.pos;	// offset from sorted station list of current player
+			assert(i < _num_station_sort[window_number]); // at least one station must exist
+
+			while (i < _num_station_sort[window_number]) {	// do until max number of stations of owner
 				st = DEREF_STATION(_station_sort[i].index);
 
+				assert(st->xy && st->owner == window_number);
+
 				SET_DPARAM16(0, st->index);
 				SET_DPARAM8(1, st->facilities);
 				x = DrawString(xb, y, STR_3049_0, 0) + 5;
@@ -168,23 +188,31 @@ static void PlayerStationsWndProc(Window *w, WindowEvent *e)
 				}
 				y += 10;
 				i++;	// next station
-				if (++p == 12) { break;} // max number of stations in 1 window
+				if (++p == w->vscroll.cap) { break;} // max number of stations in 1 window
 			}
 		}
 	} break;
 	case WE_CLICK: {
 		switch(e->click.widget) {
 		case 2: {
-			uint y = (e->click.pt.y - 15) / 10;
-			uint16 p;
+			uint32 id_v = (e->click.pt.y - 15) / 10;
 
-			if (!IS_INT_INSIDE(y, 0, 12)) {	return;}
+			if (id_v >= w->vscroll.cap) { return;} // click out of bounds
 
-			// get the p-th OUR station from globally sorted list
-			p = y + w->vscroll.pos; 
-			p += (w->window_number == 0) ? 0 : _num_station_sort[w->window_number-1];	// get offset of first station in list
-			if (p < _num_station_sort[w->window_number]) {
-				ScrollMainWindowToTile(DEREF_STATION(_station_sort[p].index)->xy);
+			id_v += w->vscroll.pos;
+
+			{
+				const byte owner = (byte)w->window_number;
+				Station *st;
+				id_v	+= (owner == 0) ? 0 : _num_station_sort[owner - 1]; // first element in list
+
+				if (id_v >= _num_station_sort[owner]) { return;} // click out of station bound
+
+				st = DEREF_STATION(_station_sort[id_v].index);
+
+				assert(st->xy && st->owner == owner);
+
+				ScrollMainWindowToTile(st->xy);
 			}
 		} break;
 		}
@@ -205,7 +233,7 @@ static const Widget _player_stations_widgets[] = {
 {    WWT_CAPTION,    14,    11,   357,     0,    13, STR_3048_STATIONS, STR_018C_WINDOW_TITLE_DRAG_THIS},
 {      WWT_PANEL,    14,     0,   346,    14,   137, 0x0, STR_3057_STATION_NAMES_CLICK_ON},
 {  WWT_SCROLLBAR,    14,   347,   357,    14,   137, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{      WWT_LAST},
+{   WIDGETS_END},
 };
 
 static const WindowDesc _player_stations_desc = {
@@ -229,29 +257,29 @@ void ShowPlayerStations(int player)
 }
 
 static const Widget _station_view_expanded_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   248,     0,    13, STR_300A_0, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    14,     0,   237,    14,    65, 0x0, 0},
-{  WWT_SCROLLBAR,    14,   238,   248,    14,    65, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{      WWT_EMPTY,     0,     0,     0,     0,     0, 0x0, 0},
-{     WWT_IMGBTN,    14,     0,   248,    66,   197, 0x0, 0},
-{ WWT_PUSHTXTBTN,    14,     0,    82,   198,   209, STR_00E4_LOCATION, STR_3053_CENTER_MAIN_VIEW_ON_STATION},
-{ WWT_PUSHTXTBTN,    14,    83,   165,   198,   209, STR_3033_ACCEPTS, STR_3056_SHOW_LIST_OF_ACCEPTED_CARGO},
-{ WWT_PUSHTXTBTN,    14,   166,   248,   198,   209, STR_0130_RENAME, STR_3055_CHANGE_NAME_OF_STATION},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   248,     0,    13, STR_300A_0,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    14,     0,   237,    14,    65, 0x0,					STR_NULL},
+{  WWT_SCROLLBAR,    14,   238,   248,    14,    65, 0x0,					STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{      WWT_EMPTY,     0,     0,     0,     0,     0, 0x0,					STR_NULL},
+{     WWT_IMGBTN,    14,     0,   248,    66,   197, 0x0,					STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,    82,   198,   209, STR_00E4_LOCATION,	STR_3053_CENTER_MAIN_VIEW_ON_STATION},
+{ WWT_PUSHTXTBTN,    14,    83,   165,   198,   209, STR_3033_ACCEPTS,	STR_3056_SHOW_LIST_OF_ACCEPTED_CARGO},
+{ WWT_PUSHTXTBTN,    14,   166,   248,   198,   209, STR_0130_RENAME,		STR_3055_CHANGE_NAME_OF_STATION},
+{   WIDGETS_END},
 };
 
 static const Widget _station_view_widgets[] = {
-{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5, STR_018B_CLOSE_WINDOW},
-{    WWT_CAPTION,    14,    11,   248,     0,    13, STR_300A_0, STR_018C_WINDOW_TITLE_DRAG_THIS},
-{     WWT_IMGBTN,    14,     0,   237,    14,    65, 0x0, 0},
-{  WWT_SCROLLBAR,    14,   238,   248,    14,    65, 0x0, STR_0190_SCROLL_BAR_SCROLLS_LIST},
-{     WWT_IMGBTN,    14,     0,   248,    66,    97, 0x0, 0},
-{      WWT_EMPTY,     0,     0,     0,     0,     0, 0x0, 0},
-{ WWT_PUSHTXTBTN,    14,     0,    82,    98,   109, STR_00E4_LOCATION, STR_3053_CENTER_MAIN_VIEW_ON_STATION},
-{ WWT_PUSHTXTBTN,    14,    83,   165,    98,   109, STR_3032_RATINGS, STR_3054_SHOW_STATION_RATINGS},
-{ WWT_PUSHTXTBTN,    14,   166,   248,    98,   109, STR_0130_RENAME, STR_3055_CHANGE_NAME_OF_STATION},
-{      WWT_LAST},
+{    WWT_TEXTBTN,    14,     0,    10,     0,    13, STR_00C5,		STR_018B_CLOSE_WINDOW},
+{    WWT_CAPTION,    14,    11,   248,     0,    13, STR_300A_0,	STR_018C_WINDOW_TITLE_DRAG_THIS},
+{     WWT_IMGBTN,    14,     0,   237,    14,    65, 0x0,					STR_NULL},
+{  WWT_SCROLLBAR,    14,   238,   248,    14,    65, 0x0,					STR_0190_SCROLL_BAR_SCROLLS_LIST},
+{     WWT_IMGBTN,    14,     0,   248,    66,    97, 0x0,					STR_NULL},
+{      WWT_EMPTY,     0,     0,     0,     0,     0, 0x0,					STR_NULL},
+{ WWT_PUSHTXTBTN,    14,     0,    82,    98,   109, STR_00E4_LOCATION,	STR_3053_CENTER_MAIN_VIEW_ON_STATION},
+{ WWT_PUSHTXTBTN,    14,    83,   165,    98,   109, STR_3032_RATINGS,	STR_3054_SHOW_STATION_RATINGS},
+{ WWT_PUSHTXTBTN,    14,   166,   248,    98,   109, STR_0130_RENAME,		STR_3055_CHANGE_NAME_OF_STATION},
+{   WIDGETS_END},
 };
 
 static void DrawStationViewWindow(Window *w)
@@ -304,7 +332,7 @@ static void DrawStationViewWindow(Window *w)
 		uint waiting = (st->goods[i].waiting_acceptance & 0xFFF);
 		if (waiting == 0)
 			continue;
-		
+
 		num = (waiting + 5) / 10;
 		if (num != 0) {
 			int cur_x = x;
@@ -338,7 +366,7 @@ static void DrawStationViewWindow(Window *w)
 			}
 		}
 	} while (pos > -5 && ++i != 12);
-	
+
 	if (w->widget == _station_view_widgets) {
 		b = _userstring;
 		b[0] = 0x81;
@@ -354,7 +382,7 @@ static void DrawStationViewWindow(Window *w)
 				b += 5;
 			}
 		}
-		
+
 		if (b == (byte*)&_userstring[3]) {
 			b[0] = 0x81;
 			b[1] = STR_00D0_NOTHING;
@@ -366,7 +394,7 @@ static void DrawStationViewWindow(Window *w)
 
 		DrawStringMultiLine(2, 67, STR_SPEC_USERSTRING, 245);
 	} else {
-		
+
 		DrawString(2, 67, STR_3034_LOCAL_RATING_OF_TRANSPORT, 0);
 
 		y = 77;
@@ -398,7 +426,7 @@ static void StationViewWndProc(Window *w, WindowEvent *e)
 
 		case 7:
 			SetWindowDirty(w);
-			
+
 			/* toggle height/widget set */
 			w->height ^= (210 ^ 110);
 			*(uint32*)&w->widget ^= (uint32)_station_view_expanded_widgets ^ (uint32)_station_view_widgets;
@@ -408,8 +436,8 @@ static void StationViewWndProc(Window *w, WindowEvent *e)
 
 		case 8: {
 			Station *st = DEREF_STATION(w->window_number);
-			SET_DPARAM16(0, st->city->citynametype);
-			SET_DPARAM32(1, st->city->citynameparts);
+			SET_DPARAM16(0, st->town->townnametype);
+			SET_DPARAM32(1, st->town->townnameparts);
 			ShowQueryString(st->string_id, STR_3030_RENAME_STATION_LOADING, 31, 180, w->window_class, w->window_number);
 		}	break;
 		}
@@ -424,7 +452,7 @@ static void StationViewWndProc(Window *w, WindowEvent *e)
 
 		st = DEREF_STATION(w->window_number);
 		DoCommandP(st->xy, w->window_number, 0, NULL, CMD_RENAME_STATION | CMD_MSG(STR_3031_CAN_T_RENAME_STATION));
-	} break;	
+	} break;
 	}
 }
 
