@@ -4,7 +4,7 @@
 #include "viewport.h"
 #include "command.h"
 #include "player.h"
-#include "city.h"
+#include "town.h"
 
 extern void DrawCanalWater(uint tile);
 
@@ -17,7 +17,7 @@ static const byte _bridge_minlen[MAX_BRIDGES] = {
 };
 
 static const byte _bridge_maxlen[MAX_BRIDGES] = {
-	16, 2, 5, 10, 16, 16, 7, 8, 9, 2, 16, 32, 32, 
+	16, 2, 5, 10, 16, 16, 7, 8, 9, 2, 16, 32, 32,
 };
 
 const uint16 _bridge_type_price_mod[MAX_BRIDGES] = {
@@ -31,8 +31,8 @@ const uint16 _bridge_speeds[MAX_BRIDGES] = {
 const PalSpriteID _bridge_sprites[MAX_BRIDGES] = {
 	0x0A24, 0x31E8A26, 0x0A25, 0x3208A22,
 	0x0A22, 0x3218A22, 0x0A23, 0x31C8A23,
-	0x31E8A23, 0x0A27, 0x0A28, 0x3218A28, 
-	0x3238A28, 
+	0x31E8A23, 0x0A27, 0x0A28, 0x3218A28,
+	0x3238A28,
 };
 
 const StringID _bridge_material[MAX_BRIDGES] = {
@@ -66,44 +66,65 @@ int CalcBridgeLenCostFactor(int x)
 }
 
 enum {
-	// foundation, whole tile is leveled up (tileh's 7, 11, 13, 14)
+	// foundation, whole tile is leveled up (tileh's 7, 11, 13, 14) --> 3 corners raised
 	BRIDGE_FULL_LEVELED_FOUNDATION = 1 << 7 | 1 << 11 | 1 << 13 | 1 << 14,
-	// foundation, tile is partly leveled up (tileh's 1, 2, 4, 8)
+	// foundation, tile is partly leveled up (tileh's 1, 2, 4, 8) --> 1 corner raised
 	BRIDGE_PARTLY_LEVELED_FOUNDATION = 1 << 1 | 1 << 2 | 1 << 4 | 1 << 8,
 	// no foundations (X,Y direction) (tileh's 0, 3, 6, 9, 12)
-	BRIDGE_NO_FOUNDATION = 1 << 0 | 1 << 3 | 1 << 6 | 1 << 9 | 1 << 12
+	BRIDGE_NO_FOUNDATION = 1 << 0 | 1 << 3 | 1 << 6 | 1 << 9 | 1 << 12,
 };
 
 /*	check if bridge can be built on slope
-		start-end_tile = false	<-- end tile
-		start-end_tile = true		<-- start tile
-*/
-static uint32 CheckBridgeSlope(uint direction, uint tileh, bool start_end_tile)
+ *	direction 0 = X-axis, direction 1 = Y-axis
+ *	is_start_tile = false		<-- end tile
+ *	is_start_tile = true		<-- start tile
+ */
+static uint32 CheckBridgeSlope(uint direction, uint tileh, bool is_start_tile)
 {
 	if (!(tileh & 0x10)) {	// disable building on very steep slopes
-		// slope foundations
-		if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << tileh) || BRIDGE_PARTLY_LEVELED_FOUNDATION & (1 << tileh))
-			return _price.terraform;
 
-		if (start_end_tile) {
+		if (is_start_tile) {
 			/* check slope at start tile
 					- no extra cost
-					- direction X: tiles 0, 9
-					- direction Y: tiles 0,12
-			*/		
+					- direction X: tiles 0,12
+					- direction Y: tiles 0, 9
+			*/
 			if ((direction?0x201:0x1001) & (1 << tileh))
 				return 0;
+
+			// disallow certain start tiles to avoid certain crooked bridges
+			if (tileh == 2)
+				return CMD_ERROR;
+
 		}
 		else {
 			/*	check slope at end tile
 					- no extra cost
-					- direction X: tiles 0, 6
-					- direction Y: tiles 0, 3
+					- direction X: tiles 0, 3
+					- direction Y: tiles 0, 6
 			*/
 			if ((direction?0x41:0x9) & (1 << tileh))
 				return 0;
+
+			// disallow certain end tiles to avoid certain crooked bridges
+			if (tileh == 8)
+				return CMD_ERROR;
+
 		}
+
+		/*	disallow common start/end tiles to avoid certain crooked bridges e.g.
+		 *	start-tile:	X 2,1 Y 2,4 (2 was disabled before)
+		 *	end-tile:		X 8,4 Y 8,1 (8 was disabled before)
+		 */
+		if ( (tileh == 1 && (is_start_tile != (bool)direction)) ||
+			   (tileh == 4 && (is_start_tile == (bool)direction))    )
+			return CMD_ERROR;
+
+		// slope foundations
+		if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << tileh) || BRIDGE_PARTLY_LEVELED_FOUNDATION & (1 << tileh))
+			return _price.terraform;
 	}
+
 	return CMD_ERROR;
 }
 
@@ -114,7 +135,7 @@ uint32 GetBridgeLength(TileIndex begin, TileIndex end)
 	y1 = GET_TILE_Y(begin);
 	x2 = GET_TILE_X(end);
 	y2 = GET_TILE_Y(end);
-	
+
 	return abs((x2 + y2 - x1 - y1)) - 1;
 }
 
@@ -135,7 +156,7 @@ bool CheckBridge_Stuff(byte bridge_type, int bridge_len)
 	return true;
 }
 
-/* Build a Bridge 
+/* Build a Bridge
  * x,y - end tile coord
  * p1  - packed start tile coords (~ dx)
  * p2&0xFF - bridge type (hi bh)
@@ -144,20 +165,20 @@ bool CheckBridge_Stuff(byte bridge_type, int bridge_len)
 int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	int bridge_type;
-	byte rail_or_road, railtype, m5, lookflag;
+	byte rail_or_road, railtype, m5;
 	int sx,sy;
 	TileInfo ti_start, ti_end, ti; /* OPT: only 2 of those are ever used */
-	int bridge_len, bridge_odd_len;
+	int bridge_len, odd_middle_part;
 	uint direction;
 	int i;
 	int32 cost, terraformcost, ret;
 
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
-	
+
 	/* unpack parameters */
 	bridge_type = p2 & 0xFF;
 	railtype = (byte)(p2 >> 8);
-	
+
 	// type of bridge
 	if (railtype & 0x80) {
 		railtype = 0;
@@ -209,7 +230,6 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (ti_start.z != ti_end.z)
 		return_cmd_error(STR_5009_LEVEL_LAND_OR_WATER_REQUIRED);
 
-
 	_error_message = STR_500C;
 
 	/* try and clear the start landscape */
@@ -219,9 +239,9 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	terraformcost = CheckBridgeSlope(direction, ti_start.tileh, true);	// true - bridge-start-tile, false - bridge-end-tile
 
-	// the AI and towns are not allowed to use bridges on slopes.
-	if (terraformcost == CMD_ERROR || 
-		 (terraformcost && (_is_ai_player || _current_player == OWNER_TOWN || !_patches.build_on_slopes)))
+	// towns are not allowed to use bridges on slopes.
+	if (terraformcost == CMD_ERROR ||
+		 (terraformcost && ((!_patches.ainew_active && _is_ai_player) || _current_player == OWNER_TOWN || !_patches.build_on_slopes)))
 		return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
 
 	cost += terraformcost;
@@ -230,12 +250,12 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if ((ret=DoCommandByTile(ti_end.tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR)) == CMD_ERROR)
 		return CMD_ERROR;
 	cost += ret;
-	
+
 	terraformcost = CheckBridgeSlope(direction, ti_end.tileh, false);	// false - end tile slope check
 
-	// the AI and towns are not allowed to use bridges on slopes.
-	if (terraformcost == CMD_ERROR || 
-		 (terraformcost && (_is_ai_player || _current_player == OWNER_TOWN || !_patches.build_on_slopes)))
+	// towns are not allowed to use bridges on slopes.
+	if (terraformcost == CMD_ERROR ||
+		 (terraformcost && ((!_patches.ainew_active && _is_ai_player) || _current_player == OWNER_TOWN || !_patches.build_on_slopes)))
 		return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
 
 	cost += terraformcost;
@@ -263,11 +283,9 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	/* set various params */
 	bridge_len = ((sx + sy - x - y) >> 4) - 1;
-	bridge_odd_len = (bridge_len & 1) ? (bridge_len>>1) : -1;
 
-	lookflag = 0;
-	if ((bridge_len & 3) == 3)
-		lookflag |= 2;
+	//position of middle part of the odd bridge (larger than MAX(i) otherwise)
+	odd_middle_part=(bridge_len%2)?(bridge_len/2):bridge_len;
 
 	for(i = 0; i != bridge_len; i++) {
 		if (direction != 0)
@@ -281,6 +299,13 @@ int32 CmdBuildBridge(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 		if (ti.tileh != 0 && ti.z >= ti_start.z)
 			return CMD_ERROR;
 
+		// Find ship below
+		if ( ti.type == MP_WATER && !EnsureNoVehicle(ti.tile) )
+		{
+			_error_message = STR_980E_SHIP_IN_THE_WAY;
+			return CMD_ERROR;
+		}
+
 		if (ti.type == MP_WATER) {
 			if (ti.map5 != 0) goto not_valid_below;
 			m5 = 0xC8;
@@ -313,24 +338,29 @@ not_valid_below:;
 			_map_type_and_height[ti.tile] &= ~0xF0;
 			_map_type_and_height[ti.tile] |= MP_TUNNELBRIDGE << 4;
 
-			if ( (m5=0, i == 0) || (m5=1, i+1 == bridge_len) ) {
+			//bridges pieces sequence (middle parts)
+			// bridge len 1: 0
+			// bridge len 2: 0 1
+			// bridge len 3: 0 4 1
+			// bridge len 4: 0 2 3 1
+			// bridge len 5: 0 2 5 3 1
+			// bridge len 6: 0 2 3 2 3 1
+			// bridge len 7: 0 2 3 4 2 3 1
+			// #0 - alwats as first, #1 - always as last (if len>1)
+			// #2,#3 are to pair in order
+			// for odd bridges: #5 is going in the bridge middle if on even position, #4 on odd (counting from 0)
 
-			} else if (lookflag & 2) {
-				if ( (m5=3,lookflag&1) ||
-				     (m5=2, (uint)i < (uint)bridge_odd_len) ) {
-					lookflag ^= 1;
-				} else {
-					m5 = 4;
-					bridge_odd_len = -1;
-				}
-			} else {
-				if ( (m5=2,!(lookflag&1)) ||
-				     (m5=3,(uint)i < (uint)bridge_odd_len) ) {
-					lookflag ^= 1;
-				} else {
-					m5 = 5;
-					bridge_odd_len = -1;
-				}
+			if(i==0)                    //first tile
+				m5 = 0;
+			else if (i==bridge_len-1)   //last tile
+				m5 = 1;
+			else if(i==odd_middle_part) //we are on the middle of odd bridge: #5 on even pos, #4 on odd
+				m5 = 5 - (i%2);
+			else {
+					// generate #2 and #3 in turns [i%2==0], after the middle of odd bridge
+					// this sequence swaps [... XOR (i>odd_middle_part)],
+					// for even bridges XOR does not apply as odd_middle_part==bridge_len
+					m5 = 2 + ((i%2==0)^(i>odd_middle_part));
 			}
 
 			_map2[ti.tile] = (bridge_type << 4) | m5;
@@ -350,10 +380,10 @@ not_valid_below:;
 	if (!(flags & DC_QUERY_COST)) {
 		bridge_len += 2;	// begin and end tiles/ramps
 
-		if (_current_player < MAX_PLAYERS && IS_HUMAN_PLAYER(_current_player))
+		if (_current_player < MAX_PLAYERS && !(_is_ai_player && !_patches.ainew_active))
 			bridge_len = CalcBridgeLenCostFactor(bridge_len);
 
-		cost += ((bridge_len * _price.build_bridge) * _bridge_type_price_mod[bridge_type]) >> 8;
+		cost += ((int64)bridge_len * _price.build_bridge * _bridge_type_price_mod[bridge_type]) >> 8;
 	}
 
 	return cost;
@@ -545,42 +575,42 @@ int32 CmdBuildTunnel(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 
 	if (flags & DC_EXEC && DoBuildTunnel(x,y,tiorg.x,tiorg.y,flags&~DC_EXEC,excavated_tile) == CMD_ERROR)
 		return CMD_ERROR;
-	
+
 	return DoBuildTunnel(x,y,tiorg.x, tiorg.y,flags,excavated_tile);
 }
 
 static const byte _updsignals_tunnel_dir[4] = { 5, 7, 1, 3};
 
-static uint CheckTunnelBusy(uint tile, int *length)
+uint CheckTunnelBusy(uint tile, int *length)
 {
 	int z = GetTileZ(tile);
 	byte m5 = _map5[tile];
 	int delta = _tileoffs_by_dir[m5 & 3], len = 0;
 	uint starttile = tile;
 	Vehicle *v;
-	
+
 	do { tile += delta; len++; } while (!IS_TILETYPE(tile, MP_TUNNELBRIDGE) || _map5[tile]&0xF0 || (byte)(_map5[tile] ^ 2) != m5 || GetTileZ(tile) != z);
 
 	if ((v=FindVehicleBetween(starttile, tile, z)) != NULL) {
 		_error_message = v->type == VEH_Train ? STR_5000_TRAIN_IN_TUNNEL : STR_5001_ROAD_VEHICLE_IN_TUNNEL;
 		return (uint)-1;
 	}
-	
+
 	if (length) *length = len;
 	return tile;
 }
 
 static int32 DoClearTunnel(uint tile, uint32 flags)
 {
+	Town *t;
 	uint endtile;
 	int length;
-	City *c = ClosestCityFromTile(tile, (uint)-1); //needed for city rating penalty
-	
+
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	// in scenario editor you can always destroy tunnels
 	if (_game_mode != GM_EDITOR && !CheckTileOwnership(tile)) {
-		if (!_patches.extra_dynamite || _map_owner[tile] != OWNER_TOWN)
+		if (!(_patches.extra_dynamite || _cheats.magic_bulldozer.value) || _map_owner[tile] != OWNER_TOWN)
 			return CMD_ERROR;
 	}
 
@@ -588,22 +618,28 @@ static int32 DoClearTunnel(uint tile, uint32 flags)
 	if (endtile == (uint)-1) return CMD_ERROR;
 
 	_build_tunnel_endtile = endtile;
+
+	t = ClosestTownFromTile(tile, (uint)-1); //needed for town rating penalty
 	// check if you're allowed to remove the tunnel owned by a town
-	// no removal if rating is lower than "Good" (250) <- this should depend on difficulty settings
+	// removal allowal depends on difficulty settings
 	if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR ) {
-		if (c->ratings[_current_player] < 250 && !(flags & DC_NO_CITY_RATING) && !_cheats.magic_bulldozer.value) {
-			SET_DPARAM16(0, c->index);
+		if (!CheckforTownRating(tile, flags, t, TUNNELBRIDGE_REMOVE)) {
+			SET_DPARAM16(0, t->index);
 			return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
 		}
 	}
 
 	if (flags & DC_EXEC) {
+		// We first need to request the direction before calling DoClearSquare
+		//  else the direction is always 0.. dah!! ;)
+		byte tile_dir = _map5[tile]&3;
+		byte endtile_dir = _map5[endtile]&3;
 		DoClearSquare(tile);
 		DoClearSquare(endtile);
-		UpdateSignalsOnSegment(tile, _updsignals_tunnel_dir[_map5[tile]&3]);
-		UpdateSignalsOnSegment(endtile, _updsignals_tunnel_dir[_map5[endtile]&3]);
-		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) 
-			ChangeCityRating(c, -250, 0);
+		UpdateSignalsOnSegment(tile, _updsignals_tunnel_dir[tile_dir]);
+		UpdateSignalsOnSegment(endtile, _updsignals_tunnel_dir[endtile_dir]);
+		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
+			ChangeTownRating(t, -250, 0);
 	}
 	return  _price.clear_tunnel * (length + 1);
 }
@@ -638,22 +674,21 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 {
 	uint endtile;
 	Vehicle *v;
+	Town *t;
 	int direction;
-	
-	City *c = ClosestCityFromTile(tile, (uint)-1); //needed for city rating penalty
-	
+
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	direction = _map5[tile]&1;
 
-	// delete stuff under the middle part if there's a transport route there..?
+	/* delete stuff under the middle part if there's a transport route there..? */
 	if ((_map5[tile] & 0xE0) == 0xE0) {
 		int32 cost;
 
 		// check if we own the tile below the bridge..
 		if (_current_player != OWNER_WATER && (!CheckTileOwnership(tile) || !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)) ))
 			return CMD_ERROR;
-		
+
 		cost = (_map5[tile] & 8) ? _price.remove_road * 2 : _price.remove_rail;
 
 		if (flags & DC_EXEC) {
@@ -662,13 +697,28 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 			MarkTileDirtyByTile(tile);
 		}
 		return cost;
+
+	/* delete canal under bridge */
+	} else if(_map5[tile]==0xC8 && GET_TILEHEIGHT(tile)!=0) {
+		int32 cost;
+
+		// check for vehicles under bridge
+		if ( !EnsureNoVehicleZ(tile, GET_TILEHEIGHT(tile)) )
+			return CMD_ERROR;
+		cost = _price.clear_water;
+		if (flags & DC_EXEC) {
+			_map5[tile] = _map5[tile] & ~0x38;
+			_map_owner[tile] = OWNER_NONE;
+			MarkTileDirtyByTile(tile);
+		}
+		return cost;
 	}
 
 	tile = FindEdgesOfBridge(tile, &endtile);
 
 	// floods, scenario editor can always destroy bridges
 	if (_current_player != OWNER_WATER && _game_mode != GM_EDITOR && !CheckTileOwnership(tile)) {
-		if (!_patches.extra_dynamite || _map_owner[tile] != OWNER_TOWN)
+		if (!(_patches.extra_dynamite || _cheats.magic_bulldozer.value) || _map_owner[tile] != OWNER_TOWN)
 			return CMD_ERROR;
 	}
 
@@ -682,7 +732,7 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 	tile		+= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
 	endtile	-= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
 	if ((v=FindVehicleBetween(tile, endtile, GET_TILEHEIGHT(tile) + 8)) != NULL) {
-		VehicleInTheWayErrMsg(v);		
+		VehicleInTheWayErrMsg(v);
 		return CMD_ERROR;
 	}
 
@@ -690,27 +740,27 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 	tile		-= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
 	endtile	+= direction ? TILE_XY(0, 1) : TILE_XY( 1,0);
 
-	// check if you're allowed to remove the bridge owned by a town. 
-	// no removal if rating is lower than "Good" (250) <- this should depend on difficult settings
+
+	t = ClosestTownFromTile(tile, (uint)-1); //needed for town rating penalty
+	// check if you're allowed to remove the bridge owned by a town.
+	// removal allowal depends on difficulty settings
 	if(_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) {
-		if (c->ratings[_current_player] < 250 && !(flags & DC_NO_CITY_RATING) && !_cheats.magic_bulldozer.value) {
-			SET_DPARAM16(0, c->index);
-			return_cmd_error(STR_2009_LOCAL_AUTHORITY_REFUSES);
-		}
+		if (!CheckforTownRating(tile, flags, t, TUNNELBRIDGE_REMOVE))
+			return CMD_ERROR;
 	}
 
 	if (flags & DC_EXEC) {
 		byte m5;
-		uint t = tile;
+		uint c = tile;
 		uint16 new_data;
 
-		//checks if the owner is town then decrease city rating by 250 until
-		// you have a "Poor" (0) city rating
-		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR) 
-			ChangeCityRating(c, -250, 0); 
+		//checks if the owner is town then decrease town rating by 250 until
+		// you have a "Poor" (0) town rating
+		if (_map_owner[tile] == OWNER_TOWN && _game_mode != GM_EDITOR)
+			ChangeTownRating(t, -250, 0);
 
 		do {
-			m5 = _map5[t];
+			m5 = _map5[c];
 
 			if (m5 & 0x40) {
 				if (m5 & 0x20) {
@@ -721,23 +771,23 @@ static int32 DoClearBridge(uint tile, uint32 flags)
 					new_data = 0x6000;
 				}
 
-				_map_type_and_height[t] &= 0x0F;
-				_map_type_and_height[t] |= new_data >> 8;
-				_map5[t] = (byte)new_data;
-				_map2[t] = 0;
+				_map_type_and_height[c] &= 0x0F;
+				_map_type_and_height[c] |= new_data >> 8;
+				_map5[c] = (byte)new_data;
+				_map2[c] = 0;
 
-				MarkTileDirtyByTile(t);
+				MarkTileDirtyByTile(c);
 
 			} else {
 clear_it:;
-				DoClearSquare(t);
+				DoClearSquare(c);
 			}
-			t += direction ? TILE_XY(0,1) : TILE_XY(1,0);
-		} while (t <= endtile);
+			c += direction ? TILE_XY(0,1) : TILE_XY(1,0);
+		} while (c <= endtile);
 
 		SetSignalsOnBothDir(tile, direction);
 		SetSignalsOnBothDir(endtile, direction);
-		
+
 	}
 
 	return ((((endtile - tile) >> (direction?8:0))&0xFF)+1) * _price.clear_bridge;
@@ -776,7 +826,7 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 		if (!CheckTileOwnership(tile)) return CMD_ERROR;
 
 		if ( (uint)(_map3_lo[tile] & 0xF) == totype) return CMD_ERROR;
-		
+
 		endtile = CheckTunnelBusy(tile, &length);
 		if (endtile == (uint)-1) return CMD_ERROR;
 
@@ -811,7 +861,7 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 		starttile = tile = FindEdgesOfBridge(tile, &endtile);
 		// Make sure there's no vehicle on the bridge
 		if ((v=FindVehicleBetween(tile, endtile, 0xff)) != NULL) {
-			VehicleInTheWayErrMsg(v);		
+			VehicleInTheWayErrMsg(v);
 			return CMD_ERROR;
 		}
 
@@ -831,13 +881,13 @@ int32 DoConvertTunnelBridgeRail(uint tile, uint totype, bool exec)
 		} while (tile <= endtile);
 
 		return cost;
-	} else 
+	} else
 		return CMD_ERROR;
 }
 
 
 // fast routine for getting the height of a middle bridge tile. 'tile' MUST be a middle bridge tile.
-static uint GetBridgeHeight(const TileInfo *ti)
+uint GetBridgeHeight(const TileInfo *ti)
 {
 	uint delta;
 	TileInfo ti_end;
@@ -856,7 +906,7 @@ static uint GetBridgeHeight(const TileInfo *ti)
 	FindLandscapeHeightByTile(&ti_end, tile);
 	if (HASBIT(1 << 7, ti_end.tileh))
 		z_correction += 8;
-	
+
 	// return the height there (the height of the NORTH CORNER)
 	return GET_TILEHEIGHT(tile) + z_correction;
 }
@@ -916,14 +966,14 @@ static void DrawBridgePillars(TileInfo *ti, int x, int y, int z)
 	}
 }
 
-static uint GetBridgeFoundation(uint tileh, byte direction) {
+uint GetBridgeFoundation(uint tileh, byte direction) {
 	int i;
 	// normal level sloped building (7, 11, 13, 14)
 	if (BRIDGE_FULL_LEVELED_FOUNDATION & (1 << tileh))
 		return tileh;
 
 	// inclined sloped building
-	if (	((i=0, tileh == 1) || (i+=2, tileh == 2) || (i+=2, tileh == 4) || (i+=2, tileh == 8)) && 
+	if (	((i=0, tileh == 1) || (i+=2, tileh == 2) || (i+=2, tileh == 4) || (i+=2, tileh == 8)) &&
 				( direction == 0 || (i++, direction == 1)) )
 		return i + 15;
 
@@ -936,8 +986,8 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 	uint tmp;
 	const uint32 *b;
 	bool ice = _map3_hi[ti->tile] & 0x80;
-	
-	// draw tunnel? 
+
+	// draw tunnel?
 	if ( (byte)(ti->map5&0xF0) == 0) {
 		/* railway type */
 		image = (_map3_lo[ti->tile] & 0xF) * 8;
@@ -968,11 +1018,11 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 		tmp = (ti->map5&3) + (tmp*2);
 
 		if (!(ti->map5 & 0x40)) {	// bridge ramps
-			
+
 			if (!(BRIDGE_NO_FOUNDATION & (1 << ti->tileh))) {	// no foundations for 0, 3, 6, 9, 12
 				int f = GetBridgeFoundation(ti->tileh, ti->map5 & 0x1);	// pass direction
 				if (f) DrawFoundation(ti, f);
-				
+
 				// default sloped sprites..
 				if (ti->tileh != 0) image = _track_sloped_sprites[ti->tileh - 1] + 0x3F3;
 			}
@@ -983,8 +1033,8 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 			b += (tmp&1); // direction
 			if (ti->tileh == 0) b += 4; // sloped "entrance" ?
 			if (ti->map5 & 0x20) b += 2; // which side
-				
-			image = *b; 
+
+			image = *b;
 
 			if (!ice) {
 				DrawClearLandTile(ti, 3);
@@ -1012,7 +1062,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 				if (ti->map5 & 8 && ti->z != 0) DrawCanalWater(ti->tile);
 			} else {
 				// draw transport route under bridge
-			
+
 				// draw foundation?
 				if (ti->tileh) {
 					int f = _bridge_foundations[ti->map5&1][ti->tileh];
@@ -1037,7 +1087,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 			b = _bridge_sprite_table[_map2[ti->tile]>>4][_map2[ti->tile]&0xF] + tmp * 4;
 
 			z = GetBridgeHeight(ti) + 5;
-			
+
 			// draw rail
 			image = b[0];
 			if (!(_display_opt & DO_TRANS_BUILDINGS)) image = (image & 0x3FFF) | 0x03224000;
@@ -1056,7 +1106,7 @@ static void DrawTile_TunnelBridge(TileInfo *ti)
 				y += 12;
 				if (image&0x3FFF) AddSortableSpriteToDraw(image, x,y, 16, 1, 0x28, z);
 			}
-			
+
 			if (ti->z + 5 == z ) {
 				// draw poles below for small bridges
 				image = b[2];
@@ -1086,7 +1136,7 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 		if ( (ti->map5 & 0xF0) == 0)
 			return z;
 
-		// bridge?	
+		// bridge?
 		if ( ti->map5 & 0x80 ) {
 			// bridge ending?
 			if (!(ti->map5 & 0x40)) {
@@ -1104,18 +1154,18 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 					// ramp in opposite dir
 					return (z + ((x^0xF)>>1));
 				}
-			
+
 			// bridge middle part
 			} else {
 				// build on slopes?
 				if (ti->tileh) z+=8;
 
 				// keep the same elevation because we're on the bridge?
-				if (_get_z_hint >= z + 8) 
+				if (_get_z_hint >= z + 8)
 					return _get_z_hint;
-				
+
 				// actually on the bridge, but not yet in the shared area.
-				if (!IS_INT_INSIDE(x, 5, 10+1)) 
+				if (!IS_INT_INSIDE(x, 5, 10+1))
 					return GetBridgeHeight(ti) + 8;
 
 				// in the shared area, assume that we're below the bridge, cause otherwise the hint would've caught it.
@@ -1124,11 +1174,11 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 					uint f = _bridge_foundations[ti->map5&1][ti->tileh];
 					// make sure that the slope is not inclined foundation
 					if (IS_BYTE_INSIDE(f, 1, 15)) return z;
-					
+
 					// change foundation type?
 					if (f) ti->tileh = _inclined_tileh[f - 15];
 				}
-	
+
 				// no transport route, fallback to default
 			}
 		}
@@ -1148,6 +1198,12 @@ static uint GetSlopeZ_TunnelBridge(TileInfo *ti) {
 	return GetPartialZ(ti->x&0xF, ti->y&0xF, ti->tileh) + z;
 }
 
+static uint GetSlopeTileh_TunnelBridge(TileInfo *ti) {
+	// not accurate, but good enough for slope graphics drawing
+	return 0;
+}
+
+
 static void GetAcceptedCargo_TunnelBridge(uint tile, AcceptedCargo *ac)
 {
 	/* not used */
@@ -1226,7 +1282,7 @@ static void TileLoop_TunnelBridge(uint tile)
 	} else if (_opt.landscape == LT_DESERT) {
 		if (GetMapExtraBits(tile) == 1 && !(_map3_hi[tile]&0x80)) {
 			_map3_hi[tile] |= 0x80;
-			MarkTileDirtyByTile(tile);			
+			MarkTileDirtyByTile(tile);
 		}
 	}
 
@@ -1240,34 +1296,49 @@ static void ClickTile_TunnelBridge(uint tile)
 }
 
 
-static uint32 GetTileTrackStatus_TunnelBridge(uint tile, int mode)
+static uint32 GetTileTrackStatus_TunnelBridge(uint tile, TransportType mode)
 {
 	uint32 result;
-	byte t, m5 = _map5[tile];
+	byte m5 = _map5[tile];
 
 	if ((m5 & 0xF0) == 0) {
-		if (((m5 & 0xC) >> 1) == mode) { /* XXX: possible problem when switching mode constants */
+		/* This is a tunnel */
+		if (((m5 & 0xC) >> 2) == mode) {
+			/* Tranport in the tunnel is compatible */
 			return m5&1 ? 0x202 : 0x101;
 		}
 	} else if (m5 & 0x80) {
+		/* This is a bridge */
 		result = 0;
-		if ((m5 & 6) == mode) {
+		if (((m5 & 0x6) >> 1) == mode) {
+			/* Transport over the bridge is compatible */
 			result = m5&1 ? 0x202 : 0x101;
 		}
 		if (m5 & 0x40) {
-			t = m5;
-			if (!(t & 0x20)) {
-				if ((t &= 0x18) != 8)
+			/* Bridge middle part */
+			if (!(m5 & 0x20)) {
+				/* Clear ground or water underneath */
+				if ((m5 & 0x18) != 8)
+					/* Clear ground */
+					return result;
+				else
+					if (mode != TRANSPORT_WATER)
+						return result;
+			} else {
+				/* Transport underneath */
+				if ((m5 & 0x18) >> 3 != mode)
+					/* Incompatible transport underneath */
 					return result;
-				t = (t&0xE7) | 0x30;
 			}
-
-			if ((t & 0x18) >> 2 != mode)
-				return result;
-
+			/* If we've not returned yet, there is a compatible
+			 * transport or water beneath, so we can add it to
+			 * result */
+			/* Why is this xor'd ? Can't it just be or'd? */
 			result ^= m5&1 ? 0x101 : 0x202;
 		}
 		return result;
+	} else {
+		assert(0); /* This should never occur */
 	}
 	return 0;
 }
@@ -1276,7 +1347,7 @@ static void ChangeTileOwner_TunnelBridge(uint tile, byte old_player, byte new_pl
 {
 	if (_map_owner[tile] != old_player)
 		return;
-	
+
 	if (new_player != 255) {
 		_map_owner[tile] = new_player;
 	}	else {
@@ -1323,7 +1394,7 @@ static uint32 VehicleEnter_TunnelBridge(Vehicle *v, uint tile, int x, int y)
 
 		if (v->type == VEH_Train) {
 			fc = (x&0xF)+(y<<4);
-			
+
 			dir = _map5[tile] & 3;
 			vdir = v->direction >> 1;
 
@@ -1342,6 +1413,7 @@ static uint32 VehicleEnter_TunnelBridge(Vehicle *v, uint tile, int x, int y)
 			}
 
 			if (dir == (vdir^2) && fc == _tunnel_fractcoord_3[dir] && z == 0) {
+				/* We're at the tunnel exit ?? */
 				v->tile = tile;
 				v->u.rail.track = _exit_tunnel_track[dir];
 				v->vehstatus &= ~VS_HIDDEN;
@@ -1356,10 +1428,10 @@ static uint32 VehicleEnter_TunnelBridge(Vehicle *v, uint tile, int x, int y)
 			if (v->u.road.state != 0xFF && dir == vdir) {
 				if (fc == _tunnel_fractcoord_4[dir] ||
 						fc == _tunnel_fractcoord_5[dir]) {
-					
+
 					v->tile = tile;
 					v->u.road.state = 0xFF;
-					v->vehstatus |= VS_HIDDEN;	
+					v->vehstatus |= VS_HIDDEN;
 					return 4;
 				} else {
 					return 0;
@@ -1367,6 +1439,7 @@ static uint32 VehicleEnter_TunnelBridge(Vehicle *v, uint tile, int x, int y)
 			}
 
 			if (dir == (vdir^2) && (
+				/* We're at the tunnel exit ?? */
 					fc == _tunnel_fractcoord_6[dir] ||
 					fc == _tunnel_fractcoord_7[dir]) &&
 					z == 0) {
@@ -1426,5 +1499,5 @@ const TileTypeProcs _tile_type_tunnelbridge_procs = {
 	NULL,														/* get_produced_cargo_proc */
 	VehicleEnter_TunnelBridge,			/* vehicle_enter_tile_proc */
 	NULL,														/* vehicle_leave_tile_proc */
+	GetSlopeTileh_TunnelBridge,			/* get_slope_tileh_proc */
 };
-
