@@ -3,7 +3,10 @@
 #include "vehicle.h"
 #include "viewport.h"
 #include "command.h"
-#include "city.h"
+#include "town.h"
+#include "news.h"
+
+static void FloodVehicle(Vehicle *v);
 
 bool IsShipDepotTile(TileIndex tile)
 {
@@ -24,7 +27,7 @@ bool IsClearWaterTile(uint tile)
 int32 CmdBuildShipDepot(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 {
 	uint tile, tile2;
-	
+
 	int32 cost, ret;
 	Depot *dep;
 
@@ -46,7 +49,7 @@ int32 CmdBuildShipDepot(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	ret = DoCommandByTile(tile2, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 	if (ret == CMD_ERROR)
 		return CMD_ERROR;
-	
+
 	// pretend that we're not making land from the water even though we actually are.
 	cost = 0;
 
@@ -57,14 +60,14 @@ int32 CmdBuildShipDepot(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	if (flags & DC_EXEC) {
 		dep->xy = tile;
 		_last_built_ship_depot_tile = tile;
-		dep->city_index = ClosestCityFromTile(tile, (uint)-1)->index;
+		dep->town_index = ClosestTownFromTile(tile, (uint)-1)->index;
 
-		ModifyTile(tile, 
+		ModifyTile(tile,
 			MP_SETTYPE(MP_WATER) | MP_MAPOWNER_CURRENT | MP_MAP5 | MP_MAP2_CLEAR | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR,
 			(0x80 + p1*2)
 		);
-	
-		ModifyTile(tile2, 
+
+		ModifyTile(tile2,
 			MP_SETTYPE(MP_WATER) | MP_MAPOWNER_CURRENT | MP_MAP5 | MP_MAP2_CLEAR | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR,
 			(0x81 + p1*2)
 		);
@@ -98,8 +101,8 @@ static int32 RemoveShipDepot(uint tile, uint32 flags)
 		// Kill the entry from the depot table
 		for(d=_depots; d->xy != tile; d++) {}
 		d->xy = 0;
-				
-		DeleteWindowById(WC_VEHICLE_DEPOT, tile);		
+
+		DeleteWindowById(WC_VEHICLE_DEPOT, tile);
 	}
 
 	return _price.remove_ship_depot;
@@ -114,7 +117,7 @@ static int32 DoBuildShiplift(uint tile, int dir, uint32 flags)
 	// middle tile
 	ret = DoCommandByTile(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
 	if (ret == CMD_ERROR) return CMD_ERROR;
-	
+
 	delta = _tileoffs_by_dir[dir];
 	// lower tile
 	ret = DoCommandByTile(tile - delta, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
@@ -148,7 +151,7 @@ static int32 RemoveShiplift(uint tile, uint32 flags)
 		DoClearSquare(tile + delta);
 		DoClearSquare(tile - delta);
 	}
-	
+
 	return _price.clear_water * 2;
 }
 
@@ -165,7 +168,7 @@ int32 CmdBuildLock(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	uint tile = TILE_FROM_XY(x,y);
 	int32 ret;
 	uint th;
-	th = GetTileSlope(tile, NULL); 
+	th = GetTileSlope(tile, NULL);
 
 	if (th==3 || th==6 || th==9 || th==12) {
 		static const byte _shiplift_dirs[16] = {0,0,0,2,0,0,1,0,0,3,0,0,0};
@@ -186,17 +189,15 @@ int32 CmdBuildCanal(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 	uint endtile = (uint)p1;
 	int delta;
 	int32 cost;
-
-	
 	SET_EXPENSES_TYPE(EXPENSES_CONSTRUCTION);
 
 	// move in which direction?
 	delta = (GET_TILE_X(tile) == GET_TILE_X(endtile)) ? TILE_XY(0,1) : TILE_XY(1,0);
 	if (endtile < tile) delta = -delta;
-	
-	cost = 0;
 
+	cost = 0;
 	for(;;) {
+		ret = 0;
 		th = GetTileSlope(tile, NULL);
 		if(th!=0)
 			return_cmd_error(STR_1000_LAND_SLOPED_IN_WRONG_DIRECTION);
@@ -205,13 +206,33 @@ int32 CmdBuildCanal(int x, int y, uint32 flags, uint32 p1, uint32 p2)
 			if (IS_TILETYPE(tile, MP_WATER)) {
 				_error_message = STR_1007_ALREADY_BUILT;
 			} else {
-				// try to clear it.
-				ret = DoCommandByTile(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+
+				/* is middle piece of a bridge? */
+				if (IS_TILETYPE(tile, MP_TUNNELBRIDGE) && _map5[tile] & 0x40) { /* build under bridge */
+					if(_map5[tile] & 0x20) { // transport route under bridge
+						_error_message = STR_5800_OBJECT_IN_THE_WAY;
+						ret = CMD_ERROR;
+					}
+					else if (_map5[tile] & 0x18) { // already water under bridge
+						_error_message = STR_1007_ALREADY_BUILT;
+						ret = CMD_ERROR;
+					}
+
+				/* no bridge? then try to clear it. */
+				} else  {
+					ret = DoCommandByTile(tile, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+				}
 				if (ret == CMD_ERROR) return ret;
 				cost += ret;
 
+				/* execute modifications */
 				if (flags & DC_EXEC) {
-					ModifyTile(tile, MP_SETTYPE(MP_WATER) | MP_MAPOWNER | MP_MAP5 | MP_MAP2_CLEAR | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR, OWNER_WATER, 0);
+					if(IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
+						// change owner to OWNER_WATER and set land under bridge bit to water
+						ModifyTile(tile, MP_MAP5 | MP_MAPOWNER, OWNER_WATER, _map5[tile] | 0x08);
+					} else {
+						ModifyTile(tile, MP_SETTYPE(MP_WATER) | MP_MAPOWNER | MP_MAP5 | MP_MAP2_CLEAR | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR, OWNER_WATER, 0);
+					}
 					// mark the tiles around dirty too
 					MarkTilesAroundDirty(tile);
 				}
@@ -256,7 +277,7 @@ static int32 ClearTile_Water(uint tile, byte flags) {
 					DoClearSquare(tile);
 				return _price.clear_water;
 			}
-			if (flags & DC_EXEC)			
+			if (flags & DC_EXEC)
 				DoClearSquare(tile);
 			return _price.purchase_land;
 		} else
@@ -269,7 +290,7 @@ static int32 ClearTile_Water(uint tile, byte flags) {
 			TILE_XY(-1, 0),TILE_XY(0, 1),TILE_XY(1, 0),TILE_XY(0, -1), // lower
 			TILE_XY(1, 0),TILE_XY(0, -1),TILE_XY(-1, 0),TILE_XY(0, 1), // upper
 		};
-				
+
 		if (flags & DC_AUTO) return_cmd_error(STR_2004_BUILDING_MUST_BE_DEMOLISHED);
 		// don't allow water to delete it.
 		if (_current_player == OWNER_WATER) return CMD_ERROR;
@@ -295,9 +316,9 @@ static bool IsWateredTile(uint tile)
 {
 	byte m5 = _map5[tile];
 	if (IS_TILETYPE(tile, MP_WATER)) {
-		return m5 != 1;	
+		return m5 != 1;
 	} else if (IS_TILETYPE(tile, MP_STATION)) {
-		// returns true if it is a dock-station (m5 inside values is m5<75 all stations, 
+		// returns true if it is a dock-station (m5 inside values is m5<75 all stations,
 		// 83<=m5<=114 new airports
 		return !(m5 < 75 || (m5 >= 83 && m5 <= 114));
 	} else if (IS_TILETYPE(tile, MP_TUNNELBRIDGE)) {
@@ -310,13 +331,13 @@ static bool IsWateredTile(uint tile)
 void DrawCanalWater(uint tile)
 {
 	uint wa;
-	
+
 	// determine the edges around with water.
 	wa = IsWateredTile(TILE_ADDXY(tile, -1, 0)) << 0;
 	wa += IsWateredTile(TILE_ADDXY(tile, 0, 1)) << 1;
 	wa += IsWateredTile(TILE_ADDXY(tile, 1, 0)) << 2;
 	wa += IsWateredTile(TILE_ADDXY(tile, 0, -1)) << 3;
-	
+
 	if (!(wa & 1)) DrawGroundSprite(SPR_CANALS_BASE + 57);
 	if (!(wa & 2)) DrawGroundSprite(SPR_CANALS_BASE + 58);
 	if (!(wa & 4)) DrawGroundSprite(SPR_CANALS_BASE + 59);
@@ -362,10 +383,10 @@ static void DrawWaterStuff(TileInfo *ti, const byte *t, uint32 palette, uint bas
 	const WaterDrawTileStruct *wdts;
 	uint32 image;
 
-	DrawGroundSprite(*(uint16*)t);	
+	DrawGroundSprite(*(const uint16*)t);
 	t += sizeof(uint16);
-		
-	for(wdts = (WaterDrawTileStruct *)t; (byte)wdts->delta_x != 0x80; wdts++) {
+
+	for(wdts = (const WaterDrawTileStruct *)t; (byte)wdts->delta_x != 0x80; wdts++) {
 		image =	wdts->image + base;
 		if (_display_opt & DO_TRANS_BUILDINGS) {
 			image |= palette;
@@ -408,10 +429,10 @@ void DrawShipDepotSprite(int x, int y, int image)
 	const WaterDrawTileStruct *wdts;
 
 	t = _shipdepot_display_seq[image];
-	DrawSprite(*(uint16*)t, x, y);
+	DrawSprite(*(const uint16*)t, x, y);
 	t += sizeof(uint16);
-		
-	for(wdts = (WaterDrawTileStruct *)t; (byte)wdts->delta_x != 0x80; wdts++) {
+
+	for(wdts = (const WaterDrawTileStruct *)t; (byte)wdts->delta_x != 0x80; wdts++) {
 		Point pt = RemapCoords(wdts->delta_x, wdts->delta_y, wdts->delta_z);
 		DrawSprite(wdts->image + PLAYER_SPRITE_COLOR(_local_player), x + pt.x, y + pt.y);
 	}
@@ -419,10 +440,15 @@ void DrawShipDepotSprite(int x, int y, int image)
 
 
 uint GetSlopeZ_Water(TileInfo *ti)
-{ 
+{
 	return GetPartialZ(ti->x&0xF, ti->y&0xF, ti->tileh) + ti->z;
 }
 
+uint GetSlopeTileh_Water(TileInfo *ti)
+{
+	return ti->tileh;
+}
+
 static void GetAcceptedCargo_Water(uint tile, AcceptedCargo *ac)
 {
 	/* not used */
@@ -483,11 +509,63 @@ static void TileLoopWaterHelper(uint tile, const int16 *offs)
 		}
 
 		_current_player = OWNER_WATER;
+		{
+			Vehicle *v = FindVehicleBetween(tile, tile, 0);
+			if (v != NULL) {FloodVehicle(v);}
+		}
 		if (DoCommandByTile(tile,0,0,DC_EXEC, CMD_LANDSCAPE_CLEAR) != CMD_ERROR)
 			ModifyTile(tile, MP_SETTYPE(MP_WATER) | MP_MAPOWNER | MP_MAP5 | MP_MAP2_CLEAR | MP_MAP3LO_CLEAR | MP_MAP3HI_CLEAR,OWNER_WATER,0);
 	}
 }
 
+static void FloodVehicle(Vehicle *v)
+{
+	Vehicle *u;
+	if (!(v->vehstatus & VS_CRASHED)) {
+		uint16 pass = 0;
+
+		if (v->type == VEH_Road) {	// flood bus/truck
+			pass = 1;	// driver
+			if (v->cargo_type == CT_PASSENGERS)
+				pass += v->cargo_count;
+
+			v->vehstatus |= VS_CRASHED;
+			v->u.road.crashed_ctr = 2000;	// max 2220, disappear pretty fast
+			_vehicle_sort_dirty[VEHROAD] = true;
+			InvalidateWindow(WC_ROADVEH_LIST, v->owner);
+		}
+
+		else if (v->type == VEH_Train) {
+			v = GetFirstVehicleInChain(v);
+			u = v;
+			pass = 4;	// driver
+
+			// crash all wagons, and count passangers
+			BEGIN_ENUM_WAGONS(v)
+				if (v->cargo_type == CT_PASSENGERS) pass += v->cargo_count;
+				v->vehstatus |= VS_CRASHED;
+			END_ENUM_WAGONS(v)
+
+			v = u;
+			v->u.rail.crash_anim_pos = 4000; // max 4440, disappear pretty fast
+			_vehicle_sort_dirty[VEHTRAIN] = true;
+			InvalidateWindow(WC_TRAINS_LIST, v->owner);
+		} else
+			return;
+
+		InvalidateWindowWidget(WC_VEHICLE_VIEW, v->index, 4);
+		InvalidateWindow(WC_VEHICLE_DEPOT, v->tile);
+
+		SET_DPARAM16(0, pass);
+		AddNewsItem(STR_B006_FLOOD_VEHICLE_DESTROYED,
+			NEWS_FLAGS(NM_THIN, NF_VIEWPORT|NF_VEHICLE, NT_ACCIDENT, 0),
+			v->index,
+			0);
+	}
+	CreateEffectVehicleRel(v,4,4,8,EV_CRASHED_SMOKE);	// show cool destruction effects
+	SndPlayVehicleFx(16, v); // create sound
+}
+
 // called from tunnelbridge_cmd
 void TileLoop_Water(uint tile)
 {
@@ -505,18 +583,32 @@ void TileLoop_Water(uint tile)
 		for(i=0; i!=4; i++)
 			TileLoopWaterHelper(tile, _tile_loop_offs_array[i]);
 	}
+
+	// edges
+	if ( GET_TILE_X(tile)==0 && IS_INT_INSIDE(GET_TILE_Y(tile),1,TILES_Y-3+1)) //NE
+		TileLoopWaterHelper(tile, _tile_loop_offs_array[2]);
+
+	if ( GET_TILE_X(tile)==(TILES_X-2) && IS_INT_INSIDE(GET_TILE_Y(tile),1,TILES_Y-3+1)) //SW
+		TileLoopWaterHelper(tile, _tile_loop_offs_array[0]);
+
+	if ( GET_TILE_Y(tile)==0 && IS_INT_INSIDE(GET_TILE_X(tile),1,TILES_X-3+1)) //NW
+		TileLoopWaterHelper(tile, _tile_loop_offs_array[1]);
+
+	if ( GET_TILE_Y(tile)==(TILES_Y-2) && IS_INT_INSIDE(GET_TILE_X(tile),1,TILES_X-3+1)) //SE
+		TileLoopWaterHelper(tile, _tile_loop_offs_array[3]);
+
 }
 
 
 static const byte _coast_tracks[16] = {0, 32, 4, 0, 16, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0};
 static const byte _shipdepot_tracks[4] = {1,1,2,2};
 static const byte _shiplift_tracks[12] = {1,2,1,2,1,2,1,2,1,2,1,2};
-static uint32 GetTileTrackStatus_Water(uint tile, int mode)
+static uint32 GetTileTrackStatus_Water(uint tile, TransportType mode)
 {
 	uint m5;
 	uint b;
 
-	if (mode != 4)
+	if (mode != TRANSPORT_WATER)
 		return 0;
 
 	m5 = _map5[tile];
@@ -529,7 +621,7 @@ static uint32 GetTileTrackStatus_Water(uint tile, int mode)
 	}
 
 	if ( (m5 & 0x10) == 0x10) {
-		// 
+		//
 		b = _shiplift_tracks[m5 & 0xF];
 		return b + (b<<8);
 	}
@@ -546,7 +638,7 @@ extern void ShowShipDepotWindow(uint tile);
 static void ClickTile_Water(uint tile)
 {
 	byte m5 = _map5[tile] - 0x80;
-	
+
 	if (IS_BYTE_INSIDE(m5, 0, 3+1)) {
 		if (m5 & 1)
 			tile += (m5==1) ? TILE_XY(-1,0) : TILE_XY(0,-1);
@@ -590,5 +682,6 @@ const TileTypeProcs _tile_type_water_procs = {
 	NULL,											/* get_produced_cargo_proc */
 	VehicleEnter_Water,				/* vehicle_enter_tile_proc */
 	NULL,											/* vehicle_leave_tile_proc */
+	GetSlopeTileh_Water,			/* get_slope_tileh_proc */
 };
 
